/*
 * JBoss, the OpenSource J2EE webOS
 * 
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package examples;

import org.jboss.cache.DefaultCacheFactory;
import org.jboss.cache.Cache;
import org.jboss.cache.Node;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Cache store transforming example
 *
 * @author <a href="mailto:galder.zamarreno@jboss.com">Galder Zamarreno</a>
 */
public class TransformStore
{
   /* Type of cache loader, either use "jdbccl" for JDBCCacheLoader or "fcl" for FileCacheLoader */   
   public static final String CL_TYPE = System.getProperty("cacheloader.type", "jdbccl");

   private static Log log = LogFactory.getLog(TransformStore.class);
   
   public static void main(String[] args) throws Exception
   {
      TransformStore ts = new TransformStore();

      String transformConfigName = "transform-" + CL_TYPE + "-service.xml";
      log.info("Transforming configuration file: " + transformConfigName);
      Cache transformingCache = ts.createAndStartCache(transformConfigName);
      ts.transformCacheStore(transformingCache);
      ts.stopAndDestroyCache(transformingCache);

      String originalConfigName = "original-" + CL_TYPE + "-service.xml";
      log.info("Original configuration file: " + originalConfigName);
      Cache checkingCache = ts.createAndStartCache(originalConfigName);
      ts.checkCacheStore(checkingCache);
      ts.stopAndDestroyCache(checkingCache);
   }

   public Cache createAndStartCache(String configFile)
   {
      return DefaultCacheFactory.getInstance().createCache(configFile, true);
   }

   public void stopAndDestroyCache(Cache cache)
   {
      cache.stop();
      cache.destroy();
   }   

   public void transformCacheStore(Cache cache)
   {
      log.info("Transforming 1.x cache store");
      transformNode(cache, cache.getRoot());
      log.info("Cache store transformed to 2.x format");
   }

   public void transformNode(Cache cache, Node<Object, Object> node)
   {
      cache.put(node.getFqn(), node.getData());
      log.info(node.getFqn() + " transformed");               

      for (Node child : node.getChildren())
      {
         transformNode(cache, child);
      }
   }

   public void checkCacheStore(Cache cache)
   {
      checkNode(cache, cache.getRoot());
   }

   public void checkNode(Cache cache, Node<Object, Object> node)
   {
      log.info("Node: " + node.getFqn() + " | Data: " + node.getData());

      for (Node child : node.getChildren())
      {
         checkNode(cache, child);
      }
   }
}
