/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache;


import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.jboss.cache.config.Configuration;
import org.jgroups.util.Util;

import java.util.HashMap;

/**
 * Tests {@link Fqn}.
 *
 * @author <a href="mailto:bela@jboss.org">Bela Ban</a> May 9, 2003
 * @version $Revision: 1.24 $
 */
public class FqnTest extends TestCase
{
   private Cache cache;

   protected void setUp()
   {
      cache = null;
   }

   protected void tearDown()
   {
      if (cache != null)
      {
         cache.stop();
         cache = null;
      }
   }

   public FqnTest(String s)
   {
      super(s);
   }

   public void testNull()
   {
      Fqn fqn = new Fqn();
      log("null fqn is " + fqn);
      assertEquals(0, fqn.size());
      int hcode = fqn.hashCode();
      assertTrue(hcode != -1);
   }

   public void testOne()
   {
      Fqn<Integer> fqn = new Fqn<Integer>(22);
      log("one fqn is " + fqn);
      assertEquals(1, fqn.size());
      int hcode = fqn.hashCode();
      assertTrue(hcode != -1);
   }

   public void testEmptyFqn()
   {
      Fqn f1 = new Fqn();
      Fqn f2 = new Fqn();
      assertEquals(f1, f2);
   }

   public void testFqn()
   {
      Fqn<String> fqn = Fqn.fromString("/a/b/c");
      log("fqn is " + fqn);
      assertEquals(3, fqn.size());

      Fqn<String> fqn2 = new Fqn<String>("a", "b", "c");
      log("fqn2 is " + fqn2);
      assertEquals(3, fqn2.size());
      assertEquals("fqn should equal fqn2", fqn, fqn2);
      assertEquals(fqn.hashCode(), fqn2.hashCode());
   }

   public void testHereogeneousNames()
   {
      Fqn<Object> fqn = new Fqn<Object>("string", 38, true);
      log("fqn is " + fqn);
      assertEquals(3, fqn.size());

      Fqn<Object> fqn2 = new Fqn<Object>("string", 38, true);
      assertEquals(fqn, fqn2);
      assertEquals(fqn.hashCode(), fqn2.hashCode());
   }

   public void testHashcode()
   {
      Fqn fqn1, fqn2;
      fqn1 = new Fqn<Object>("a", "b", "c");
      fqn2 = Fqn.fromString("/a/b/c");
      log("fqn is " + fqn1);
      assertEquals(fqn1, fqn2);

      HashMap<Fqn, Integer> map = new HashMap<Fqn, Integer>();
      map.put(fqn1, 33);
      map.put(fqn2, 34);
      assertEquals(1, map.size());
      assertEquals(new Integer(34), map.get(fqn1));
   }

   public void testHashcode2()
   {
      Fqn<Integer> fqn = new Fqn<Integer>(-1);
      log("one fqn is " + fqn);
      assertEquals(1, fqn.size());
      int hcode = fqn.hashCode();
      assertTrue(hcode == -1);
   }

   public void testEquals()
   {
      Fqn<String> fqn1 = new Fqn<String>("person/test");

      Fqn<String> f1;
      Fqn<String> f2;
      Fqn<String> f3;

      f1 = new Fqn<String>(fqn1, "0");
      f2 = new Fqn<String>(fqn1, "1");
      f3 = new Fqn<String>(fqn1, "2");

      HashMap<Fqn<String>, String> map = new HashMap<Fqn<String>, String>();
      map.put(f1, "0");
      map.put(f2, "1");
      map.put(f3, "2");

      assertNotNull("f1 ", map.get(new Fqn<String>(fqn1, "0")));
      assertNotNull("f2 ", map.get(new Fqn<String>(fqn1, "1")));
      assertNotNull("f3 ", map.get(new Fqn<String>(fqn1, "2")));

   }


   public void testEquals2()
   {
      Fqn<String> f1;
      Fqn<String> f2;
      f1 = Fqn.fromString("/a/b/c");
      f2 = Fqn.fromString("/a/b/c");
      assertEquals(f1, f2);

      f2 = Fqn.fromString("/a/b");
      assertFalse(f1.equals(f2));

      f2 = Fqn.fromString("/a/b/c/d");
      assertFalse(f1.equals(f2));
   }

   public void testEquals2WithMarshalling() throws Exception
   {
      Fqn<String> f1, f2;
      f1 = Fqn.fromString("/a/b/c");
      f2 = marshalAndUnmarshal(f1);
      assertEquals(f1, f2);
   }


   public void testEquals3()
   {
      Fqn<Object> f1;
      Fqn<? extends Object> f2;
      f1 = new Fqn<Object>("a", 322649, Boolean.TRUE);
      f2 = new Fqn<String>();
      assertFalse(f1.equals(f2));
      assertFalse(f2.equals(f1));

      f2 = Fqn.fromString("a/322649/TRUE");
      assertFalse(f1.equals(f2));

      f2 = new Fqn<Object>("a", 322649, Boolean.FALSE);
      assertFalse(f1.equals(f2));

      f2 = new Fqn<Object>("a", 322649, Boolean.TRUE);
      assertEquals(f1, f2);
   }

   public void testEquals3WithMarshalling() throws Exception
   {
      Fqn f1, f2;
      f1 = new Fqn<Object>("a", 322649, Boolean.TRUE);
      f2 = marshalAndUnmarshal(f1);
      assertEquals(f1, f2);
      assertEquals(f2, f1);

      f2 = Fqn.fromString("a/322649/TRUE");
      f2 = marshalAndUnmarshal(f2);
      assertFalse(f1.equals(f2));

      f2 = new Fqn<Object>("a", 322649, Boolean.FALSE);
      f2 = marshalAndUnmarshal(f2);
      assertFalse(f1.equals(f2));

      f2 = new Fqn<Object>("a", 322649, Boolean.TRUE);
      f2 = marshalAndUnmarshal(f2);
      assertEquals(f1, f2);
   }

   public void testEquals4()
   {
      Fqn<String> fqn = Fqn.fromString("X");
      // Check casting
      assertFalse("Casting ok", fqn.equals("X"));
      // Check null
      assertFalse("null ok", fqn.equals(null));
   }

   public void testClone() throws CloneNotSupportedException
   {
      Fqn<String> fqn1 = Fqn.fromString("/a/b/c");
      Fqn<String> fqn2 = fqn1.clone();
      assertEquals(fqn1, fqn2);
      assertEquals(fqn1.hashCode(), fqn2.hashCode());
   }

   public void testNullElements() throws CloneNotSupportedException
   {
      Fqn<Object> fqn0 = new Fqn<Object>((Object) null);
      assertEquals(1, fqn0.size());

      Fqn<Object> fqn1 = new Fqn<Object>("NULL", null, 0);
      assertEquals(3, fqn1.size());

      Fqn<Object> fqn2 = new Fqn<Object>("NULL", null, 0);
      assertEquals(fqn1.hashCode(), fqn2.hashCode());
      assertEquals(fqn1, fqn2);
      assertEquals(fqn1, fqn1.clone());
   }

   public void testIteration()
   {
      Fqn<String> fqn = Fqn.fromString("/a/b/c");
      assertEquals(3, fqn.size());
      Fqn<Object> tmp_fqn = new Fqn<Object>();
      assertEquals(0, tmp_fqn.size());
      for (int i = 0; i < fqn.size(); i++)
      {
         String s = fqn.get(i);
         tmp_fqn = new Fqn<Object>(tmp_fqn, s);
         assertEquals(tmp_fqn.size(), i + 1);
      }
      assertEquals(3, tmp_fqn.size());
      assertEquals(fqn, tmp_fqn);
   }

   public void testIsChildOf()
   {
      Fqn<String> child = Fqn.fromString("/a/b");
      Fqn<String> parent = Fqn.fromString("/a");
      assertTrue("Is child of ", child.isChildOf(parent));
      assertFalse("Is child of ", parent.isChildOf(child));
      assertTrue("Is same ", child.isChildOrEquals(child));

      parent = Fqn.fromString("/a/b/c");
      child = Fqn.fromString("/a/b/c/d/e/f/g/h/e/r/e/r/t/tt/");
      assertTrue(child.isChildOf(parent));
   }

   public void testIsChildOf2()
   {
      Fqn<String> child = Fqn.fromString("/a/b/c/d");
      assertEquals("Fqn ", "/b/c/d", child.getSubFqn(1, child.size()).toString());
   }

   public void testParentage()
   {
      Fqn<String> fqnRoot = new Fqn<String>();
      Fqn<String> parent = fqnRoot.getParent();
      assertEquals(parent, fqnRoot);

      Fqn<String> fqnOne = Fqn.fromString("/one");
      parent = fqnOne.getParent();
      assertEquals(parent, fqnRoot);
      assertTrue(fqnOne.isChildOf(parent));

      Fqn<String> fqnTwo = Fqn.fromString("/one/two");
      parent = fqnTwo.getParent();
      assertEquals(parent, fqnOne);
      assertTrue(fqnTwo.isChildOf(parent));

      Fqn<String> fqnThree = Fqn.fromString("/one/two/three");
      parent = fqnThree.getParent();
      assertEquals(parent, fqnTwo);
      assertTrue(fqnThree.isChildOf(parent));

   }

   public void testRoot()
   {
      Fqn<String> fqn = new Fqn<String>();
      assertTrue(fqn.isRoot());

      fqn = Fqn.fromString("/one/two");
      assertFalse(fqn.isRoot());
   }

   public void testGetName()
   {
      Fqn<Integer> integerFqn = new Fqn<Integer>(1);
      assertEquals("1", integerFqn.getLastElementAsString());

      Object object = new Object();
      Fqn<Object> objectFqn = new Fqn<Object>(object);
      assertEquals(object.toString(), objectFqn.getLastElementAsString());
   }

   public void testCloningString() throws CloneNotSupportedException
   {
      Fqn<String> f = Fqn.fromString("/a/b/c");
      assertEquals(f, f.clone());
   }

   public void testCloningOtherTypes() throws CloneNotSupportedException
   {
      Fqn<Object> f = new Fqn<Object>("blah", 10, Boolean.TRUE);
      assertEquals(f, f.clone());
   }

   public void testRemovalNonString() throws Exception
   {
      Fqn<Object> f = new Fqn<Object>("test", 1);

      Configuration c = new Configuration();
      c.setCacheMode("LOCAL");
      cache = DefaultCacheFactory.getInstance().createCache(c);

      cache.put(f, "key", "value");

      assertEquals("value", cache.get(f, "key"));
      assertTrue(cache.getRoot().hasChild(f));

      cache.removeNode(f);

      assertNull(cache.get(f, "key"));
      assertFalse(cache.getRoot().hasChild(f));
   }

   Fqn marshalAndUnmarshal(Fqn fqn) throws Exception
   {
      byte[] buf = Util.objectToByteBuffer(fqn);
      return (Fqn) Util.objectFromByteBuffer(buf);
   }

   // testing generics

   public void testGenerics()
   {
      Fqn<String> f = Fqn.fromString("/blah/blah");
      Fqn<String> f2 = Fqn.fromString("/blah/blah");

      assertEquals(f, f2);

      Fqn<Integer> f3 = new Fqn<Integer>(1, 2, 3, 5);

      Fqn<Integer> f4 = new Fqn<Integer>(1, 2, 3);

      assertEquals(f3.getParent(), f4);
   }

   public void testGenerations()
   {
      Fqn<Integer> f = new Fqn<Integer>(1, 2, 3, 4, 5, 6, 7);

      assertEquals(f, f.getAncestor(f.size()));
      assertEquals(f.getParent(), f.getAncestor(f.size() - 1));
      assertEquals(Fqn.ROOT, f.getAncestor(0));
      assertEquals(new Fqn<Integer>(1), f.getAncestor(1));
      assertEquals(new Fqn<Integer>(1, 2), f.getAncestor(2));
      assertEquals(new Fqn<Integer>(1, 2, 3), f.getAncestor(3));
      assertEquals(new Fqn<Integer>(1, 2, 3, 4), f.getAncestor(4));
      assertEquals(new Fqn<Integer>(1, 2, 3, 4, 5), f.getAncestor(5));

      try
      {
         f.getAncestor(-1);
         fail("Should fail");
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }

      try
      {
         f.getAncestor(f.size() + 1);
         fail("Should fail");
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }
   }

   void log(String msg)
   {
      System.out.println("-- " + msg);
   }

   public static Test suite()
   {
      return new TestSuite(FqnTest.class);
   }

   public static void main(String[] args)
   {
      junit.textui.TestRunner.run(suite());
   }
}
