package org.jboss.cache;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.jboss.cache.config.Configuration;
import org.jboss.cache.lock.IsolationLevel;

import javax.transaction.Transaction;
import java.util.HashMap;

/**
 * Simple functional tests for CacheImpl
 *
 * @author Bela Ban
 * @version $Id: TreeCacheFunctionalTest.java,v 1.9 2007/02/07 22:06:39 genman Exp $
 */
public class TreeCacheFunctionalTest extends TestCase
{
   CacheImpl cache = null;
   Transaction tx = null;
   final Fqn FQN = Fqn.fromString("/myNode");
   final String KEY = "key";
   final String VALUE = "value";
   Exception ex;


   protected void setUp() throws Exception
   {
      super.setUp();
      cache = (CacheImpl) DefaultCacheFactory.getInstance().createCache(false);
      cache.getConfiguration().setCacheMode(Configuration.CacheMode.LOCAL);
      cache.getConfiguration().setTransactionManagerLookupClass("org.jboss.cache.transaction.DummyTransactionManagerLookup");
      cache.getConfiguration().setIsolationLevel(IsolationLevel.REPEATABLE_READ);
      cache.create();
      cache.start();
      ex = null;
   }

   protected void tearDown() throws Exception
   {
      super.tearDown();
      if (cache != null)
      {
         cache.stop();
         cache.destroy();
         cache = null;
      }
      if (ex != null)
      {
         throw ex;
      }
   }


   public void testPut() throws CacheException
   {
      cache.put("/a/b/c", "age", 38);
      assertEquals(cache.get("/a/b/c", "age"), 38);
      assertNotNull(cache.get("/a/b/c"));
      assertEquals(0, cache.getNumberOfLocksHeld());
      assertEquals(0, cache.getLockTable().size());
   }


   public void testPutNullKey() throws CacheException
   {
      Object key = null;
      cache.put("/a/b/c", key, "val");
      System.out.println("value of /a/b/c " + cache.print("/a/b/c"));
   }

   public void testPutNullValue() throws CacheException
   {
      Object val = null;
      cache.put("/a/b/c", "key", val);
      System.out.println("value of /a/b/c " + cache.print("/a/b/c"));
   }

   public void testPutNullKeyAndValues() throws CacheException
   {
      Object key = null, val = null;
      cache.put("/a/b/c", key, val);
      System.out.println("value of /a/b/c " + cache.print("/a/b/c"));
   }

   public void testPutMapsWithNullValues() throws CacheException
   {
      HashMap map = new HashMap();
      map.put("key", null);
      map.put(null, "val");
      map.put("a", "b");
      map.put(null, null);
      cache.put("/a/b/c", map);
      System.out.println("value of /a/b/c " + cache.print("/a/b/c"));
   }

   public void testPutKeys() throws CacheException
   {
      cache.put("/a/b/c", "age", 38);
      cache.put("/a/b/c", "name", "Bela");
      assertEquals(cache.get("/a/b/c", "age"), 38);
      assertNotNull(cache.get("/a/b/c"));
      assertEquals(cache.getKeys("/a/b/c").size(), 2);
      assertEquals(cache.exists("/a/b/c"), true);
      assertEquals(0, cache.getNumberOfLocksHeld());
      assertEquals(0, cache.getLockTable().size());
   }

   public void testRemove() throws CacheException
   {
      cache.put("/a/b/c", null);
      cache.put("/a/b/c/1", null);
      cache.put("/a/b/c/2", null);
      cache.put("/a/b/c/3", null);
      cache.put("/a/b/c/3/a/b/c", null);

      cache.remove("/a/b/c");
      assertEquals(0, cache.getNumberOfLocksHeld());
      assertEquals(0, cache.getLockTable().size());
   }

   public static Test suite()
   {
      return new TestSuite(TreeCacheFunctionalTest.class);
   }

}
