package org.jboss.cache.api;

import junit.framework.TestCase;
import org.jboss.cache.CacheSPI;
import org.jboss.cache.DefaultCacheFactory;
import org.jboss.cache.config.Configuration;
import org.jboss.cache.config.Configuration.CacheMode;
import org.jboss.cache.factories.UnitTestCacheConfigurationFactory;
import org.jboss.cache.factories.XmlConfigurationParser;

import java.util.List;

public class CacheSPITest extends TestCase
{
   private CacheSPI cache1;
   private CacheSPI cache2;

   protected boolean optimistic = false;

   protected void setUp() throws Exception
   {
      super.setUp();

      Configuration conf1 = UnitTestCacheConfigurationFactory.createConfiguration(CacheMode.REPL_SYNC);

      Configuration conf2 = UnitTestCacheConfigurationFactory.createConfiguration(CacheMode.REPL_SYNC);

      conf1.setNodeLockingScheme(optimistic ? Configuration.NodeLockingScheme.OPTIMISTIC : Configuration.NodeLockingScheme.PESSIMISTIC);
      conf2.setNodeLockingScheme(optimistic ? Configuration.NodeLockingScheme.OPTIMISTIC : Configuration.NodeLockingScheme.PESSIMISTIC);

      cache1 = (CacheSPI) DefaultCacheFactory.getInstance().createCache(conf1, false);
      cache2 = (CacheSPI) DefaultCacheFactory.getInstance().createCache(conf2, false);
   }

   protected void tearDown() throws Exception
   {
      super.tearDown();

      if (cache1 != null)
      {
         try
         {
            cache1.stop();
         }
         catch (Exception e) {}
      }

      if (cache2 != null)
      {
         try
         {
            cache2.stop();
         }
         catch (Exception e) {}
      }
   }

   public void testGetMembers() throws Exception
   {

      cache1.start();
      List memb1 = cache1.getMembers();
      assertEquals("View has one member", 1, memb1.size());

      Object coord = memb1.get(0);

      cache2.start();
      memb1 = cache1.getMembers();
      List memb2 = cache2.getMembers();
      assertEquals("View has two members", 2, memb1.size());
      assertEquals("Both caches have same view", memb1, memb2);

      cache1.stop();
      memb2 = cache2.getMembers();
      assertEquals("View has one member", 1, memb2.size());
      assertFalse("Coordinator changed", coord.equals(memb2.get(0)));
   }

   public void testIsCoordinator() throws Exception
   {
      XmlConfigurationParser parser = new XmlConfigurationParser();
      Configuration conf1 = UnitTestCacheConfigurationFactory.createConfiguration(CacheMode.REPL_SYNC);

      Configuration conf2 = UnitTestCacheConfigurationFactory.createConfiguration(CacheMode.REPL_SYNC);

      cache1 = (CacheSPI) DefaultCacheFactory.getInstance().createCache(conf1, false);
      cache2 = (CacheSPI) DefaultCacheFactory.getInstance().createCache(conf2, false);

      cache1.start();
      assertTrue("Cache1 is coordinator", cache1.getRPCManager().isCoordinator());

      cache2.start();
      assertTrue("Cache1 is still coordinator", cache1.getRPCManager().isCoordinator());
      assertFalse("Cache2 is not coordinator", cache2.getRPCManager().isCoordinator());

      cache1.stop();
      assertTrue("Cache2 is coordinator", cache2.getRPCManager().isCoordinator());

   }
}
