/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache.buddyreplication;

import org.jboss.cache.CacheImpl;
import org.jboss.cache.Fqn;
import org.jboss.cache.misc.TestingUtil;

/**
 * Tests how groups are formed and disbanded
 *
 * @author <a href="mailto:manik@jboss.org">Manik Surtani (manik@jboss.org)</a>
 */
public class BuddyAssignmentStateTransferTest extends BuddyReplicationTestsBase
{

   protected int timeout = 10000; // !!!

   protected int getSleepTimeout()
   {
      return timeout;
   }

   public void testNonRegionBasedStateTransfer() throws Exception
   {
      caches = new CacheImpl[2];
      caches[0] = createCache(1, "TEST", false, true);

      Fqn main = Fqn.fromString("/a/b/c");
      caches[0].put(main, "name", "Joe");

      caches[1] = createCache(1, "TEST", false, true);

      TestingUtil.blockUntilViewsReceived(caches, VIEW_BLOCK_TIMEOUT);
      TestingUtil.sleepThread(getSleepTimeout());

      Fqn test = new Fqn(BuddyManager.BUDDY_BACKUP_SUBTREE_FQN,
              BuddyManager.getGroupNameFromAddress(caches[0].getLocalAddress()));
      test = new Fqn(test, main);

      assertEquals("State transferred", "Joe", caches[1].get(test, "name"));

      CacheImpl[] old = caches;
      caches = new CacheImpl[3];
      System.arraycopy(old, 0, caches, 0, old.length);
      caches[2] = createCache(1, "TEST", false, true);

      TestingUtil.blockUntilViewsReceived(caches, VIEW_BLOCK_TIMEOUT);
      TestingUtil.sleepThread(getSleepTimeout());

      assertNull("State not transferred", caches[2].get(test, "name"));

      // Make 2 the buddy of 0
      caches[1].stop();
      caches[1] = null;

      TestingUtil.sleepThread(getSleepTimeout());

      assertEquals("State transferred", "Joe", caches[2].get(test, "name"));
   }

   public void testPersistentStateTransfer() throws Exception
   {
      caches = new CacheImpl[2];

      caches[0] = createCacheWithCacheLoader(false, false, false, true, false);
      caches[0].getConfiguration().setFetchInMemoryState(false);

      caches[0].start();

      Fqn main = Fqn.fromString("/a/b/c");
      caches[0].put(main, "name", "Joe");

      caches[1] = createCacheWithCacheLoader(false, false, false, true, false);
      caches[1].getConfiguration().setFetchInMemoryState(false);

      caches[1].start();

      TestingUtil.blockUntilViewsReceived(caches, VIEW_BLOCK_TIMEOUT);
      TestingUtil.sleepThread(getSleepTimeout());

      Fqn test = BuddyManager.getBackupFqn(caches[0].getLocalAddress(), main);

      assertFalse("/a/b/c shld not be bin memory", caches[1].exists(test));
      assertNotNull("/a/b/c shld be in CL", caches[1].getCacheLoader().get(test));
      assertEquals("/a/b/c shld in cache loader", "Joe", caches[1].get(test, "name"));
   }
}
