package org.jboss.cache.factories;

import junit.framework.TestCase;
import org.jboss.cache.interceptors.Interceptor;

import java.util.List;

/**
 * @author <a href="mailto:manik@jboss.org">Manik Surtani</a>
 */
public abstract class InterceptorChainTestBase extends TestCase
{
   protected void assertLast(Interceptor first, Interceptor last)
   {
      assertNotNull("First interceptor in the chain cannot be null", first);
      assertNotNull("Last interceptor in the chain cannot be null", last);

      Interceptor i = first;
      while (i != null)
      {
         assertEquals("Expected last interceptor (in " + i + ") to be " + last, last, i.getLast());
         i = i.getNext();
      }
   }

   protected void assertInterceptorLinkage(List<Interceptor> list)
   {
      Interceptor previous = null;
      for (Interceptor i : list)
      {
         if (previous == null)
         {
            previous = i;
            continue;
         }

         assertEquals("Expecting the next interceptor after " + previous + " to be " + i, i, previous.getNext());

         previous = i;
      }

      assertLast(list.get(0), previous);
   }
}
