package org.jboss.cache.lock;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.jboss.cache.CacheImpl;
import org.jboss.cache.DefaultCacheFactory;
import org.jboss.cache.Fqn;
import org.jboss.cache.NodeSPI;
import org.jboss.cache.config.Configuration;
import org.jboss.cache.transaction.DummyTransactionManager;

import javax.transaction.Transaction;

/**
 * @author Bela Ban
 * @version $Id: AcquireAllTest.java,v 1.11 2007/02/07 22:06:49 genman Exp $
 */
public class AcquireAllTest extends TestCase
{
   CacheImpl cache = null, cache2;
   Transaction tx = null;
   final Fqn FQN = Fqn.fromString("/myNode");
   final String KEY = "key";
   final String VALUE = "value";


   protected void setUp() throws Exception
   {
      super.setUp();
   }

   protected void tearDown() throws Exception
   {
      super.tearDown();
      if (cache != null)
      {
         cache.stop();
         cache.destroy();
         cache = null;
      }
      if (tx != null)
      {
         tx.commit();
         tx = null;
      }
   }


   public void testAcquireAll() throws Exception
   {
      NodeSPI root;
      Object owner = Thread.currentThread();

      cache = createCache(Configuration.CacheMode.LOCAL, IsolationLevel.SERIALIZABLE);
      cache.put("/a/b/c", null);
      cache.put("/1/2/3", null);

      root = cache.getRoot();
      NodeLock lock = root.getLock();

      lock.acquireAll(owner, 2000, NodeLock.LockType.READ);
      lock.releaseAll(owner);

      assertEquals(0, cache.getNumberOfLocksHeld());

      lock.acquireAll(owner, 2000, NodeLock.LockType.WRITE);
      lock.releaseAll(owner);

      assertEquals(0, cache.getNumberOfLocksHeld());
   }


   public void testAcquireAllReplicated() throws Exception
   {
      NodeSPI root;
      Object owner = Thread.currentThread();

      cache2 = createCache(Configuration.CacheMode.REPL_ASYNC, IsolationLevel.SERIALIZABLE);
      cache2.put("/a/b/c", null);
      cache2.put("/1/2/3", null);

      cache = createCache(Configuration.CacheMode.REPL_ASYNC, IsolationLevel.SERIALIZABLE);
      root = cache.getRoot();
      NodeLock lock = root.getLock();

      lock.acquireAll(owner, 2000, NodeLock.LockType.READ);
      lock.releaseAll(owner);

      assertEquals(0, cache.getNumberOfLocksHeld());

      lock.acquireAll(owner, 2000, NodeLock.LockType.WRITE);
      lock.releaseAll(owner);

      assertEquals(0, cache.getNumberOfLocksHeld());
   }


   CacheImpl createCache(Configuration.CacheMode mode, IsolationLevel level) throws Exception
   {
      CacheImpl c = (CacheImpl) DefaultCacheFactory.getInstance().createCache(false);
      c.getConfiguration().setCacheMode(mode);
      c.getConfiguration().setIsolationLevel(level);
      c.getConfiguration().setTransactionManagerLookupClass("org.jboss.cache.transaction.DummyTransactionManagerLookup");
      c.create();
      c.start();
      return c;
   }

   Transaction startTransaction()
   {
      DummyTransactionManager mgr = DummyTransactionManager.getInstance();
      try
      {
         mgr.begin();
         return mgr.getTransaction();
      }
      catch (Throwable t)
      {
         return null;
      }
   }


   public static Test suite()
   {
      return new TestSuite(AcquireAllTest.class);
   }

   public static void main(String[] args)
   {
      junit.textui.TestRunner.run(suite());
   }

}
