package org.jboss.cache.lock;

import junit.framework.TestCase;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.cache.Cache;
import org.jboss.cache.DefaultCacheFactory;
import org.jboss.cache.Fqn;
import org.jboss.cache.NodeSPI;
import org.jboss.cache.transaction.DummyTransactionManagerLookup;

import javax.transaction.TransactionManager;

/**
 * basic locking test
 *
 * @author <a href="mailto:manik@jboss.org">Manik Surtani</a>
 * @since 2.0.0
 */
public class PessimisticLockTest extends TestCase
{
   private Cache cache;
   private TransactionManager tm;
   private Fqn fqn = Fqn.fromString("/a/b/c");
   private static final Log log = LogFactory.getLog(PessimisticLockTest.class);

   protected void setUp()
   {
      cache = DefaultCacheFactory.getInstance().createCache(false);
      cache.getConfiguration().setTransactionManagerLookupClass(DummyTransactionManagerLookup.class.getName());
      cache.start();
      tm = cache.getConfiguration().getRuntimeConfig().getTransactionManager();
   }

   protected void tearDown()
   {
      cache.stop();
   }

   public void testPut() throws Exception
   {
      cache.put(fqn, "k", "v");

      tm.begin();
      cache.put(fqn, "k2", "v2");
      NodeSPI n = (NodeSPI) cache.getRoot().getChild(fqn);

      assertFalse(n.getLock().isReadLocked());
      assertTrue(n.getLock().isWriteLocked());
      assertTrue(n.getParent().getLock().isReadLocked());
      assertFalse(n.getParent().getLock().isWriteLocked());
      assertTrue(n.getParent().getParent().getLock().isReadLocked());
      assertFalse(n.getParent().getParent().getLock().isWriteLocked());

      tm.commit();
   }

   public void testGet() throws Exception
   {
      cache.put(fqn, "k", "v");

      tm.begin();
      cache.get(fqn, "k2");
      NodeSPI n = (NodeSPI) cache.getRoot().getChild(fqn);

      assertTrue(n.getLock().isReadLocked());
      assertFalse(n.getLock().isWriteLocked());
      assertTrue(n.getParent().getLock().isReadLocked());
      assertFalse(n.getParent().getLock().isWriteLocked());
      assertTrue(n.getParent().getParent().getLock().isReadLocked());
      assertFalse(n.getParent().getParent().getLock().isWriteLocked());

      tm.commit();
   }

   public void testRemove() throws Exception
   {
      cache.put(fqn, "k", "v");
      tm.begin();
      cache.remove(fqn, "k2");
      NodeSPI n = (NodeSPI) cache.getRoot().getChild(fqn);

      assertFalse(n.getLock().isReadLocked());
      assertTrue(n.getLock().isWriteLocked());
      assertTrue(n.getParent().getLock().isReadLocked());
      assertFalse(n.getParent().getLock().isWriteLocked());
      assertTrue(n.getParent().getParent().getLock().isReadLocked());
      assertFalse(n.getParent().getParent().getLock().isWriteLocked());

      tm.commit();

   }
}
