package org.jboss.cache.lock;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

/**
 * Tests the various ReadWriteLock implementations
 * @author Bela Ban
 * @version $Id: ReadWriteLockTest.java,v 1.4 2007/01/31 17:28:41 msurtani Exp $
 */
public class ReadWriteLockTest extends TestCase {
   ReadWriteLock lock;
   Exception  ex=null;


   protected void setUp() throws Exception {
      super.setUp();
      ex=null;
   }

   protected void tearDown() throws Exception {
      super.tearDown();
      lock=null;
      if(ex != null)
         throw ex;
   }


   public void testMoreWriteReleasesThanAcquisitions() throws InterruptedException {
      lock = new ReentrantReadWriteLock();
      lock.writeLock().lock();
      lock.writeLock().unlock();
      try
      {
         lock.writeLock().unlock();
         fail("Should have barfed");
      }
      catch (IllegalMonitorStateException imse)
      {
         // should barf
      }         
   }

   public void testMoreReadReleasesThanAcquisitions() throws InterruptedException {
      lock = new ReentrantReadWriteLock();
      lock.readLock().lock();
      lock.readLock().unlock();
      try
      {
         lock.readLock().unlock();
         fail("read locks cannot be released more than acquired");
      }
      catch(IllegalMonitorStateException illegalStateEx)
      {

      }
   }

   public void testSimple() throws InterruptedException {
      lock = new ReentrantReadWriteLock();
      lock.readLock().lock();
      // upgrades must be manual; involving giving up the RL first.  Sucks.
      // see http://java.sun.com/j2se/1.5.0/docs/api/java/util/concurrent/locks/ReentrantReadWriteLock.html
      lock.readLock().unlock();

      lock.writeLock().lock();
      lock.writeLock().lock();
      lock.readLock().lock();
      lock.readLock().lock();

      // since the thread currently has a WL and we are reentrant, this works without a manual upgrade.
      lock.writeLock().lock();
      lock.writeLock().lock();
   }


   public void testOneWriterMultipleReaders() throws InterruptedException {
      lock = new ReentrantReadWriteLock();

      Writer writer=new Writer("writer");
      Reader reader1=new Reader("reader1");
      Reader reader2=new Reader("reader2");

      writer.start();
      reader1.start();
      reader2.start();

      writer.join();
      reader1.join();
      reader2.join();
   }

   class Writer extends Thread {

      public Writer(String name) {
         super(name);
      }

      public void run() {
         try {
            log("acquiring WL");
            lock.writeLock().lock();
            log("acquired WL successfully");
            sleep(1000);
         }
         catch(InterruptedException e) {
            ex=e;
         }
         finally {
            log("releasing WL");
            lock.writeLock().unlock();
         }
      }
   }


   class Reader extends Thread {

      public Reader(String name) {
         super(name);
      }


      public void run() {
         try {
            log("acquiring RL");
            lock.readLock().lock();
            log("acquired RL successfully");
            sleep(500);
         }
         catch(InterruptedException e) {
            ex=e;
         }
         finally {
            log("releasing RL");
            lock.readLock().unlock();
         }
      }
   }

   static void log(String msg) {
      System.out.println(Thread.currentThread().getName() + ": " +  msg);
   }

   public static Test suite() {
      return new TestSuite(ReadWriteLockTest.class);
   }

   public static void main(String[] args) {
      junit.textui.TestRunner.run(suite());
   }

}
