package org.jboss.cache.notifications;

import junit.framework.TestCase;
import org.jboss.cache.util.Util;

import java.util.HashMap;
import java.util.Map;

/**
 * Tests the diffs between maps.
 *
 * @author <a href="mailto:manik@jboss.org">Manik Surtani</a>
 * @since 2.0.0
 */
public class NodeMapDiffTest extends TestCase
{
   public void testDataAdded()
   {
      Util.MapModifications expected = new Util.MapModifications();
      expected.addedEntries.put("key", "value");
      expected.addedEntries.put("key1", "value1");

      Map pre = new HashMap();
      pre.put("oldKey", "oldValue");

      Map post = new HashMap();
      post.putAll(pre);
      post.put("key", "value");
      post.put("key1", "value1");

      assertEquals(expected, Util.diffNodeData(pre, post));
   }

   public void testDataRemoved()
   {
      Util.MapModifications expected = new Util.MapModifications();
      expected.removedEntries.put("key", "value");
      expected.removedEntries.put("key1", "value1");

      Map post = new HashMap();
      post.put("oldKey", "oldValue");

      Map pre = new HashMap();
      pre.putAll(post);
      pre.put("key", "value");
      pre.put("key1", "value1");

      assertEquals(expected, Util.diffNodeData(pre, post));
   }

   public void testDataChanged()
   {
      Util.MapModifications expected = new Util.MapModifications();
      expected.modifiedEntries.put("key", "value");
      expected.modifiedEntries.put("key1", "value1");

      Map pre = new HashMap();
      pre.put("oldKey", "oldValue");
      pre.put("key", "valueOLD");
      pre.put("key1", "value1OLD");

      Map post = new HashMap();
      post.putAll(pre);
      post.put("key", "value");
      post.put("key1", "value1");

      assertEquals(expected, Util.diffNodeData(pre, post));
   }

   public void testNullMaps()
   {
      try
      {
         Util.diffNodeData(null, null);
         fail("Expected NPE");
      }
      catch (NullPointerException npe)
      {
         // expected
      }

      try
      {
         Util.diffNodeData(new HashMap(), null);
         fail("Expected NPE");
      }
      catch (NullPointerException npe)
      {
         // expected
      }

      try
      {
         Util.diffNodeData(null, new HashMap());
         fail("Expected NPE");
      }
      catch (NullPointerException npe)
      {
         // expected
      }

   }

   public void testEmptyMaps()
   {
      Util.MapModifications expected = new Util.MapModifications();

      Map pre = new HashMap();
      Map post = new HashMap();

      assertEquals(expected, Util.diffNodeData(pre, post));
   }

   public void testNoChange()
   {
      Util.MapModifications expected = new Util.MapModifications();

      Map pre = new HashMap();
      pre.put("a", "b");
      pre.put("c", "d");
      pre.put("e", "f");

      Map post = new HashMap();
      post.put("a", "b");
      post.put("c", "d");
      post.put("e", "f");

      assertEquals(expected, Util.diffNodeData(pre, post));
   }

   public void testMultipleChanges()
   {
      Util.MapModifications expected = new Util.MapModifications();
      expected.modifiedEntries.put("key", "value");
      expected.modifiedEntries.put("key1", "value1");
      expected.addedEntries.put("key2", "value2");
      expected.addedEntries.put("key3", "value3");
      expected.removedEntries.put("key4", "value4");
      expected.removedEntries.put("key5", "value5");

      Map pre = new HashMap();
      pre.put("oldKey", "oldValue");
      pre.put("key", "valueOLD");
      pre.put("key1", "value1OLD");
      pre.put("key4", "value4");
      pre.put("key5", "value5");

      Map post = new HashMap();
      post.put("oldKey", "oldValue");
      post.put("key", "value");
      post.put("key1", "value1");
      post.put("key2", "value2");
      post.put("key3", "value3");

      assertEquals(expected, Util.diffNodeData(pre, post));
   }

}
