/*
 * Created on 17-Feb-2005
 *
 */
package org.jboss.cache.optimistic;

import org.jboss.cache.CacheImpl;
import org.jboss.cache.Fqn;
import org.jboss.cache.config.Configuration;
import org.jboss.cache.loader.SamplePojo;
import org.jboss.cache.misc.TestingUtil;
import org.jboss.cache.transaction.DummyTransactionManager;
import org.jboss.cache.transaction.GlobalTransaction;
import org.jboss.cache.transaction.TransactionTable;

import javax.transaction.Transaction;

public class AsyncCacheTest extends AbstractOptimisticTestCase
{

   CacheImpl cache, cache2;

   protected void setUp() throws Exception
   {
      cache = createReplicatedCache(Configuration.CacheMode.REPL_ASYNC);
      cache2 = createReplicatedCache(Configuration.CacheMode.REPL_ASYNC);
   }

   protected void tearDown()
   {
      super.tearDown();
      destroyCache(cache);
      destroyCache(cache2);
      cache = null;
      cache2 = null;
   }

   public AsyncCacheTest(String s)
   {
      super(s);
   }

   public void testRemoteCacheBroadcast() throws Exception
   {

      assertEquals(2, cache.getMembers().size());
      assertEquals(2, cache2.getMembers().size());

      DummyTransactionManager mgr = DummyTransactionManager.getInstance();

      //start local transaction
      mgr.begin();
      Transaction tx = mgr.getTransaction();

      //this sets
      GlobalTransaction gtx = cache.getCurrentTransaction(tx);

      SamplePojo pojo = new SamplePojo(21, "test");

      cache.put("/one/two", "key1", pojo);

      //GlobalTransaction gtx = cache.getCurrentTransaction(tx);
      TransactionTable table = cache.getTransactionTable();
      assertNotNull(mgr.getTransaction());
      mgr.commit();


      assertNull(mgr.getTransaction());

      //assert that the local cache is in the right state
      assertEquals(0, cache.getTransactionTable().getNumGlobalTransactions());
      assertEquals(0, cache.getTransactionTable().getNumLocalTransactions());

      assertTrue(cache.exists(Fqn.fromString("/one/two")));
      assertTrue(cache.exists(Fqn.fromString("/one")));
      assertEquals(pojo, cache.get(Fqn.fromString("/one/two"), "key1"));

      // allow changes to replicate since this is async
      TestingUtil.sleepThread((long) 1000);

      assertEquals(0, cache2.getTransactionTable().getNumGlobalTransactions());
      assertEquals(0, cache2.getTransactionTable().getNumLocalTransactions());

      assertTrue(cache2.exists(Fqn.fromString("/one/two")));
      assertTrue(cache2.exists(Fqn.fromString("/one")));
      assertEquals(pojo, cache2.get(Fqn.fromString("/one/two"), "key1"));
   }


   public void testTwoWayRemoteCacheBroadcast() throws Exception
   {
      assertEquals(2, cache.getMembers().size());
      assertEquals(2, cache2.getMembers().size());

      DummyTransactionManager mgr = DummyTransactionManager.getInstance();

      //start local transaction
      mgr.begin();
      Transaction tx = mgr.getTransaction();

      //this sets
      cache.getCurrentTransaction(tx);

      SamplePojo pojo = new SamplePojo(21, "test");

      cache.put("/one/two", "key1", pojo);

      GlobalTransaction gtx = cache.getCurrentTransaction(tx);
      TransactionTable table = cache.getTransactionTable();
      assertNotNull(mgr.getTransaction());
      mgr.commit();


      assertNull(mgr.getTransaction());

      //assert that the local cache is in the right state
      assertEquals(0, cache.getTransactionTable().getNumGlobalTransactions());
      assertEquals(0, cache.getTransactionTable().getNumLocalTransactions());

      assertTrue(cache.exists(Fqn.fromString("/one/two")));
      assertTrue(cache.exists(Fqn.fromString("/one")));
      assertEquals(pojo, cache.get(Fqn.fromString("/one/two"), "key1"));

      // let the async calls complete
      TestingUtil.sleepThread((long) 1000);

      assertEquals(0, cache2.getTransactionTable().getNumGlobalTransactions());
      assertEquals(0, cache2.getTransactionTable().getNumLocalTransactions());

      assertTrue(cache2.exists(Fqn.fromString("/one/two")));
      assertTrue(cache2.exists(Fqn.fromString("/one")));

      assertEquals(pojo, cache2.get(Fqn.fromString("/one/two"), "key1"));
   }
}
