package org.jboss.cache.optimistic;

import org.jboss.cache.CacheImpl;
import org.jboss.cache.Fqn;

import javax.transaction.Transaction;
import javax.transaction.TransactionManager;

/**
 * Tests the hasChild() API
 *
 * @author <a href="mailto:manik@jboss.org">Manik Surtani</a>
 * @since 2.0.0
 */
public class HasChildTest extends AbstractOptimisticTestCase
{
   private CacheImpl cache;
   private TransactionManager txMgr;
   private Fqn f = Fqn.fromString("/a");

   protected void setUp() throws Exception
   {
      cache = createCache();
      txMgr = cache.getTransactionManager();
   }

   protected void tearDown()
   {
      destroyCache(cache);
   }

   public HasChildTest(String name)
   {
      super(name);
   }

   public void testExists() throws Exception
   {
      cache.put(f, "k", "v");

      assertTrue(cache.getRoot().hasChild(f));

      cache.remove(f);

      assertFalse(cache.getRoot().hasChild(f));

      txMgr.begin();

      cache.put(f, "k", "v");
      assertTrue(cache.getRoot().hasChild(f));

      Transaction t = txMgr.suspend();
      assertFalse(cache.getRoot().hasChild(f));

      txMgr.resume(t);
      assertTrue(cache.getRoot().hasChild(f));
      txMgr.commit();

      assertTrue(cache.getRoot().hasChild(f));

      txMgr.begin();
      assertTrue(cache.getRoot().hasChild(f));
      cache.remove(f);
      assertFalse(cache.getRoot().hasChild(f));

      t = txMgr.suspend();
      assertTrue(cache.getRoot().hasChild(f));
      txMgr.resume(t);
      assertFalse(cache.getRoot().hasChild(f));
      txMgr.commit();

      assertFalse(cache.getRoot().hasChild(f));
   }
}
