package org.jboss.cache.pojo;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.cache.pojo.test.SerializedAddress;


/**
 * Test for Serializable POJOs
 *
 * @author Ben Wang
 */

public class NonAspectizedTest extends TestCase
{
   Log log_ = LogFactory.getLog(NonAspectizedTest.class);
   PojoCache cache_;

   public NonAspectizedTest(String name)
   {
      super(name);
   }

   protected void setUp() throws Exception
   {
      super.setUp();
      log_.info("setUp() ....");
      String configFile = "META-INF/local-service.xml";
      boolean toStart = false;
      cache_ = PojoCacheFactory.createCache(configFile, toStart);
      cache_.start();
   }

   protected void tearDown() throws Exception
   {
      super.tearDown();
      cache_.stop();
   }

//   public void testDummy() {}

   public void testPutPrimitive() throws Exception
   {
      log_.info("testPutPrimitive() ....");
      String test = "test";
      cache_.attach("/a", test);
      String result = (String) cache_.find("/a");
      assertEquals("test string ", "test", result);
      cache_.detach("/a");
      assertNull("Object should be null ", cache_.find("/a"));
   }

   public void testPutSerializable() throws Exception
   {
      log_.info("testPutSerializable() ....");
      SerializedAddress test = new SerializedAddress();
      test.setCity("Sunnyvale");
      test.setZip(94086);
      cache_.attach("/a", test);
      SerializedAddress result = (SerializedAddress) cache_.find("/a");
      assertEquals("test SerializedAddress ", test, result);
      cache_.detach("/a");
      assertNull("Object should be null ", cache_.find("/a"));
   }

   public static Test suite() throws Exception
   {
      return new TestSuite(NonAspectizedTest.class);
   }


   public static void main(String[] args) throws Exception
   {
      junit.textui.TestRunner.run(suite());
   }

}

