package org.jboss.cache.pojo.collection;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.cache.Fqn;
import org.jboss.cache.config.Configuration.CacheMode;
import org.jboss.cache.factories.UnitTestCacheConfigurationFactory;
import org.jboss.cache.pojo.PojoCache;
import org.jboss.cache.pojo.PojoCacheFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * List implementation testing.
 *
 * @author Ben Wang
 */

public class CachedListImplTest extends TestCase
{
   Log log = LogFactory.getLog(CachedListImplTest.class);
   PojoCache cache_, cache1_;

   public CachedListImplTest(String name)
   {
      super(name);
   }


   protected void setUp() throws Exception
   {
      super.setUp();
      log.info("setUp() ....");
      boolean toStart = false;
      cache_ = PojoCacheFactory.createCache(UnitTestCacheConfigurationFactory.createConfiguration(CacheMode.REPL_SYNC), toStart);
      cache_.getCache().getConfiguration().setFetchInMemoryState(false);
      cache1_ = PojoCacheFactory.createCache(UnitTestCacheConfigurationFactory.createConfiguration(CacheMode.REPL_SYNC), toStart);
      cache1_.getCache().getConfiguration().setFetchInMemoryState(false);
      cache_.start();
      cache1_.start();
   }

   protected void tearDown() throws Exception
   {
      super.tearDown();
      cache_.stop();
      cache1_.stop();
   }

   public void testSimpleRepl()
   {
      List list = new ArrayList();
      list.add("1");
      list.add("2");

      cache_.attach("list", list);

      // proxy now
      list = (List) cache_.find("list");

      // test repl
      try {
         cache_.getCache().put(Fqn.fromString("test"), "1", list);
         fail("Should have a non-serializable exception on list: " +list);
      } catch (Exception ex)
      {
         // OK. It works
      }

      // JBCACHE-975
      list = (List) cache_.detach("list");

      // test repl
      cache_.getCache().put(Fqn.fromString("test"), "1", list);

      ArrayList l1 = (ArrayList) cache1_.getCache().get(Fqn.fromString("test"), "1");
      System.out.println(" list : " + l1);
   }

   public static Test suite() throws Exception
   {
      return new TestSuite(CachedListImplTest.class);
   }

   public static void main(String[] args) throws Exception
   {
      junit.textui.TestRunner.run(suite());
   }

}

