/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2006, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.cache.pojo.collection;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.cache.Fqn;
import org.jboss.cache.config.Configuration.CacheMode;
import org.jboss.cache.factories.UnitTestCacheConfigurationFactory;
import org.jboss.cache.pojo.PojoCache;
import org.jboss.cache.pojo.PojoCacheFactory;

import java.util.HashMap;
import java.util.Map;

/**
 * List implementation testing.
 *
 * @author Ben Wang
 */

public class CachedMapImplTest extends TestCase
{
   Log log = LogFactory.getLog(CachedMapImplTest.class);
   PojoCache cache_, cache1_;

   public CachedMapImplTest(String name)
   {
      super(name);
   }


   protected void setUp() throws Exception
   {
      super.setUp();
      log.info("setUp() ....");
      boolean toStart = true;
      cache_ = PojoCacheFactory.createCache(UnitTestCacheConfigurationFactory.createConfiguration(CacheMode.REPL_SYNC), toStart);

      cache1_ = PojoCacheFactory.createCache(UnitTestCacheConfigurationFactory.createConfiguration(CacheMode.REPL_SYNC), toStart);
   }

   protected void tearDown() throws Exception
   {
      super.tearDown();
      cache_.stop();
      cache1_.stop();
   }

   public void testSimpleRepl()
   {
      Map map = new HashMap();
      map.put("1", "1");
      map.put("2", "2");

      cache_.attach("map", map);

      // Can't use proxy here. JBCACHE-975.
      map = (Map) cache_.detach("map");

      // test repl
      cache_.getCache().put(Fqn.fromString("test"), "1", map);

      Map m1 = (Map) cache1_.getCache().get(Fqn.fromString("test"), "1");
      System.out.println(" map : " + m1);
   }

   public static Test suite() throws Exception
   {
      return new TestSuite(CachedMapImplTest.class);
   }

   public static void main(String[] args) throws Exception
   {
      junit.textui.TestRunner.run(suite());
   }

}
