/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.cache.pojo.notification;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.jboss.cache.config.Configuration.CacheMode;
import org.jboss.cache.factories.UnitTestCacheConfigurationFactory;
import org.jboss.cache.pojo.PojoCache;
import org.jboss.cache.pojo.PojoCacheFactory;
import org.jboss.cache.pojo.notification.event.Event;

//$Id: ReplicatedObjectTest.java,v 1.2 2007/06/29 04:34:01 jgreene Exp $

/**
 * Tests replicated set notifications
 *
 * @author Jason T. Greene
 */
public class ReplicatedObjectTest extends ListTest
{
   private PojoCache listenerCache;

   public ReplicatedObjectTest(String name)
   {
      super(name);
   }

   protected void setUp() throws Exception
   {
      super.setUp();

      cache = PojoCacheFactory.createCache(UnitTestCacheConfigurationFactory.createConfiguration(CacheMode.REPL_SYNC), false);
      cache.start();
      listenerCache = PojoCacheFactory.createCache(UnitTestCacheConfigurationFactory.createConfiguration(CacheMode.REPL_SYNC), false);
      listenerCache.start();
      listenerCache.addListener(listener);
   }

   protected void tearDown() throws Exception
   {
      super.tearDown();
      cache.stop();
      listenerCache.stop();
   }

   protected void verifyNotification(Event notification)
   {
      assertSame(listenerCache, notification.getContext().getPojoCache());
      assertEquals(false, notification.isLocal());
   }

   public static Test suite() throws Exception
   {
      return new TestSuite(ReplicatedObjectTest.class);
   }

   public static void main(String[] args) throws Exception
   {
      junit.textui.TestRunner.run(ReplicatedObjectTest.suite());
   }
}
