/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.cache.pojo.rollback;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.aop.proxy.ClassProxy;
import org.jboss.cache.pojo.PojoCache;
import org.jboss.cache.pojo.PojoCacheFactory;
import org.jboss.cache.pojo.interceptors.PojoFailedTxMockupInterceptor;
import org.jboss.cache.pojo.test.Person;
import org.jboss.cache.transaction.DummyTransactionManager;

import javax.transaction.TransactionManager;
import java.util.ArrayList;

/**
 * Additional basic tests
 *
 * @author Ben Wang
 */

public class ListUndoTest extends TestCase
{
   Log log_ = LogFactory.getLog(ListUndoTest.class);
   PojoCache cache_;
   TransactionManager tx_mgr;

   public ListUndoTest(String name)
   {
      super(name);
   }

   protected void setUp() throws Exception
   {
      super.setUp();
      log_.info("setUp() ....");
      String configFile = "META-INF/local-service.xml";
      boolean toStart = false;
      cache_ = PojoCacheFactory.createCache(configFile, toStart);
      cache_.start();
      tx_mgr = DummyTransactionManager.getInstance();

   }

   protected void tearDown() throws Exception
   {
      super.tearDown();
      cache_.stop();
   }

//   public void testDummy() {}

   private void setTxRollback(boolean isTrue)
   {
      PojoFailedTxMockupInterceptor.TX_ROLLBACK = isTrue;
   }

   public void testSimple() throws Exception
   {
      ArrayList list = new ArrayList();
      list.add("test1");

      setTxRollback(true);
      cache_.attach("/a", list);
      assertFalse("Should not have cache interceptor ", isProxy(list));

      cache_.attach("/a", list);
   }

   public void testSimpleTxWithRollback1() throws Exception
   {
      log_.info("testSimpleTxWithRollback1() ....");
      Person test = new Person();
      test.setName("Ben");
      test.setAge(10);
      ArrayList list = new ArrayList();
      list.add("English");
      test.setLanguages(list);

      setTxRollback(true);
      cache_.attach("/a", test);
      assertFalse("Should not have cache interceptor ", isProxy(test.getLanguages()));

      cache_.attach("/a", test);
   }

   private boolean isProxy(Object pojo)
   {
      if (pojo instanceof ClassProxy) return true;
      return false;
   }

   public void testSimpleTxWithRollback2() throws Exception
   {
      log_.info("testSimpleTxWithRollback1() ....");
      Person test = new Person();
      test.setName("Ben");
      test.setAge(10);
      ArrayList list = new ArrayList();
      list.add("English");
      test.setLanguages(list);

      cache_.attach("/a", test);

      setTxRollback(true);
      cache_.detach("/a");

      assertTrue("Should still have cache interceptor ", isProxy(test.getLanguages()));
      cache_.detach("/a");
   }

   public static Test suite() throws Exception
   {
      return new TestSuite(ListUndoTest.class);
   }


   public static void main(String[] args) throws Exception
   {
      junit.textui.TestRunner.run(ListUndoTest.suite());
   }

}
