/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.cache.pojo.rollback;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.aop.proxy.ClassProxy;
import org.jboss.cache.pojo.PojoCache;
import org.jboss.cache.pojo.PojoCacheFactory;
import org.jboss.cache.pojo.test.Person;
import org.jboss.cache.transaction.DummyTransactionManager;

import javax.transaction.TransactionManager;
import java.util.HashSet;

/**
 * Additional basic tests
 *
 * @author Ben Wang
 */

public class SetTxUndoTest extends TestCase
{
   Log log_ = LogFactory.getLog(SetTxUndoTest.class);
   PojoCache cache_;
   TransactionManager tx_mgr;

   public SetTxUndoTest(String name)
   {
      super(name);
   }

   protected void setUp() throws Exception
   {
      super.setUp();
      log_.info("setUp() ....");
      String configFile = "META-INF/local-service.xml";
      boolean toStart = false;
      cache_ = PojoCacheFactory.createCache(configFile, toStart);
      cache_.start();
      tx_mgr = DummyTransactionManager.getInstance();

   }

   protected void tearDown() throws Exception
   {
      super.tearDown();
      cache_.stop();
   }

//   public void testDummy() {}

   public void testSimple() throws Exception
   {
      HashSet set = new HashSet();
      set.add("test1");

      tx_mgr.begin();
      cache_.attach("/a", set);
      tx_mgr.getTransaction().rollback();
      assertFalse("Should not have cache interceptor ", isProxy(set));

      cache_.attach("/a", set);
   }

   public void testSimpleTxWithRollback1() throws Exception
   {
      log_.info("testSimpleTxWithRollback1() ....");
      Person test = new Person();
      test.setName("Ben");
      test.setAge(10);
      HashSet set = new HashSet();
      set.add("English");
      test.setSkills(set);

      tx_mgr.begin();
      cache_.attach("/a", test);
      tx_mgr.getTransaction().rollback();
      assertFalse("Should not have cache interceptor ", isProxy(test.getSkills()));

      cache_.attach("/a", test);
   }

   private boolean isProxy(Object pojo)
   {
      if (pojo instanceof ClassProxy) return true;
      return false;
   }

   public void testSimpleTxWithRollback2() throws Exception
   {
      log_.info("testSimpleTxWithRollback1() ....");
      Person test = new Person();
      test.setName("Ben");
      test.setAge(10);
      HashSet set = new HashSet();
      set.add("English");
      test.setSkills(set);

      cache_.attach("/a", test);

      tx_mgr.begin();
      cache_.detach("/a");
      tx_mgr.getTransaction().rollback();

      assertTrue("Should still have cache interceptor ", isProxy(test.getSkills()));
      cache_.detach("/a");
   }

   public void testSimpleTxWithRollback3() throws Exception
   {
      log_.info("testSimpleTxWithRollback1() ....");
      Person test = new Person();
      test.setName("Ben");
      test.setAge(10);
      HashSet set = new HashSet();
      set.add("English");
      test.setSkills(set);
      tx_mgr.begin();
      cache_.attach("/a", test);
      cache_.detach("/a");
      tx_mgr.getTransaction().rollback();

      assertFalse("Should not have cache interceptor ", isProxy(test.getSkills()));
   }

   public static Test suite() throws Exception
   {
      return new TestSuite(SetTxUndoTest.class);
   }


   public static void main(String[] args) throws Exception
   {
      junit.textui.TestRunner.run(SetTxUndoTest.suite());
   }

}
