/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2006, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.cache.pojo.statetransfer;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.cache.CacheImpl;
import org.jboss.cache.config.Configuration.CacheMode;
import org.jboss.cache.factories.UnitTestCacheConfigurationFactory;
import org.jboss.cache.pojo.PojoCache;
import org.jboss.cache.pojo.PojoCacheFactory;
import org.jboss.cache.pojo.test.Person;
import org.jboss.cache.pojo.test.Student;

/**
 * Simple replicated test for state transfer
 *
 * @author Ben Wang
 */
public class ReplicatedTest extends TestCase
{
   Log log = LogFactory.getLog(ReplicatedTest.class);
   PojoCache cache, cache1;


   public ReplicatedTest(String name)
   {
      super(name);
   }

   protected void setUp() throws Exception
   {
      super.setUp();
      log.info("setUp() ....");
   }

   protected void tearDown() throws Exception
   {
      super.tearDown();
      cache.stop();
      cache1.stop();
   }

//   public void testDummy() {}

   private Person createPerson(String id, String name, int age)
   {
      Person p = new Person();
      p.setName(name);
      p.setAge(age);
      cache.attach(id, p);
      return p;
   }

   private Student createStudent(String id, String name, int age, String grade)
   {
      Student p = new Student();
      p.setName(name);
      p.setAge(age);
      p.setYear(grade);
      cache.attach(id, p);
      return p;
   }

   public void testSimple() throws Exception
   {
      boolean toStart = true;
      cache = PojoCacheFactory.createCache(UnitTestCacheConfigurationFactory.createConfiguration(CacheMode.REPL_SYNC), toStart);
      Person ben = createPerson("/person/test1", "Ben Wang", 40);

      System.out.println("\n*** I ***");
      System.out.println(((CacheImpl) cache.getCache()).printDetails());
      cache1 = PojoCacheFactory.createCache(UnitTestCacheConfigurationFactory.createConfiguration(CacheMode.REPL_SYNC), toStart);
      cache1.start();

      System.out.println("\n*** II ***");
      System.out.println(((CacheImpl) cache1.getCache()).printDetails());

      Person p = (Person) cache1.find("/person/test1");

      log.info("testSimple() ....");
      assertEquals("Ben Wang", ben.getName());
      assertEquals("Ben Wang", ((Person) cache1.find("/person/test1")).getName());
      cache.detach("/person/test1");
   }

   public static Test suite() throws Exception
   {
      return new TestSuite(ReplicatedTest.class);
   }


   public static void main(String[] args) throws Exception
   {
      junit.textui.TestRunner.run(suite());
   }

}
