package org.jboss.cache.transaction;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.jboss.cache.CacheImpl;
import org.jboss.cache.DefaultCacheFactory;
import org.jboss.cache.Fqn;
import org.jboss.cache.config.Configuration;
import org.jboss.cache.lock.IsolationLevel;

import javax.transaction.NotSupportedException;
import javax.transaction.SystemException;
import javax.transaction.Transaction;

/**
 * Tests whether modifications within callbacks (TreeCacheListener) are handled correctly
 *
 * @author Bela Ban
 * @version $Id: IsolationLevelNoneTest.java,v 1.8 2007/02/07 22:06:43 genman Exp $
 */
public class IsolationLevelNoneTest extends TestCase
{
   CacheImpl cache = null;
   final Fqn FQN = Fqn.fromString("/a/b/c");
   final String KEY = "key";
   final String VALUE = "value";
   Transaction tx;


   protected void setUp() throws Exception
   {
      super.setUp();
   }

   protected void tearDown() throws Exception
   {
      super.tearDown();
      if (cache != null)
      {
         cache.stop();
         cache.destroy();
         cache = null;
      }
   }


   public void testWithoutTransactions() throws Exception
   {
      cache = (CacheImpl) DefaultCacheFactory.getInstance().createCache(false);
      cache.getConfiguration().setCacheMode(Configuration.CacheMode.LOCAL);
      cache.getConfiguration().setIsolationLevel(IsolationLevel.NONE);
      cache.start();
      cache.put(FQN, KEY, VALUE);
      cache.put(FQN + "/d", KEY, VALUE);
      assertTrue(cache.exists(FQN, KEY));
      assertEquals(VALUE, cache.get(FQN, KEY));
      System.out.println("cache: " + cache.toString(true) + ", locks: " + cache.printLockInfo());
      assertEquals(0, cache.getNumberOfLocksHeld());
   }

   public void testWithTransactions() throws Exception
   {
      cache = (CacheImpl) DefaultCacheFactory.getInstance().createCache(false);
      cache.getConfiguration().setCacheMode(Configuration.CacheMode.LOCAL);
      cache.getConfiguration().setIsolationLevel(IsolationLevel.NONE);
      cache.setTransactionManagerLookup(new DummyTransactionManagerLookup());
      cache.start();
      tx = startTransaction();
      cache.put(FQN, KEY, VALUE);
      cache.put(FQN + "/d", KEY, VALUE);
      assertTrue(cache.exists(FQN, KEY));
      assertEquals(VALUE, cache.get(FQN, KEY));
      System.out.println("cache: " + cache.toString(true) + ", locks: " + cache.printLockInfo());
      assertEquals(0, cache.getNumberOfLocksHeld());
      tx.commit();
   }


   public void testWithTransactionsRepeatableRead() throws Exception
   {
      cache = (CacheImpl) DefaultCacheFactory.getInstance().createCache(false);
      cache.getConfiguration().setCacheMode(Configuration.CacheMode.LOCAL);
      cache.getConfiguration().setIsolationLevel(IsolationLevel.REPEATABLE_READ);
      cache.setTransactionManagerLookup(new DummyTransactionManagerLookup());
      cache.start();
      tx = startTransaction();
      cache.put(FQN, KEY, VALUE);
      cache.put(FQN + "/d", KEY, VALUE);
      assertTrue(cache.exists(FQN, KEY));
      assertEquals(VALUE, cache.get(FQN, KEY));
      System.out.println("cache: " + cache.toString(true) + ", locks: " + cache.printLockInfo());
      assertEquals(5, cache.getNumberOfLocksHeld());
      tx.commit();
   }

   private Transaction startTransaction() throws SystemException, NotSupportedException
   {
      DummyTransactionManager mgr = DummyTransactionManager.getInstance();
      mgr.begin();
      return mgr.getTransaction();
   }

   public static Test suite()
   {
      return new TestSuite(IsolationLevelNoneTest.class);
   }


}
