package org.jboss.cache.transaction;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.jboss.cache.CacheImpl;
import org.jboss.cache.DefaultCacheFactory;
import org.jboss.cache.Fqn;

import javax.transaction.Transaction;
import javax.transaction.TransactionManager;

/**
 * Based on a contribution by Owen Taylor
 *
 * @author otaylor@redhat.com
 * @author Manik Surtani (manik@jboss.org)
 */
public class SuspendTxTest extends TestCase
{
   CacheImpl cache;
   TransactionManager mgr;

   protected void setUp() throws Exception
   {
      super.setUp();
      cache = (CacheImpl) DefaultCacheFactory.getInstance().createCache(false);
      cache.getConfiguration().setCacheMode("local");
      cache.getConfiguration().setTransactionManagerLookupClass("org.jboss.cache.transaction.DummyTransactionManagerLookup");
      cache.start();
      mgr = cache.getTransactionManager();
   }

   protected void tearDown() throws Exception
   {
      super.tearDown();
      cache.stop();
      if (mgr.getTransaction() != null)
      {
         mgr.rollback();
      }
      mgr = null;
   }

   /**
    * Tests that locks created when a transaction is suspended are independent
    * from the transaction.
    */
   public void testSuspendedLocks() throws Exception
   {
      // create /one first
      cache.put("/one", null);
      cache.put("/a", null);
      mgr.begin();

      cache.put("/one/two", "key1", "val1");
      int numLocksBefore = cache.getNumberOfLocksHeld();

      Transaction tx = mgr.suspend();

      cache.put("/a/b", "key1", "val1");
      mgr.resume(tx);

      assertEquals(numLocksBefore, cache.getNumberOfLocksHeld());
   }

   /**
    * Tests that locks created when a transaction is suspended are independent
    * from the transaction.
    */
   public void testSuspendedUsingOptionsLocks() throws Exception
   {
      mgr.begin();

      cache.put("/one/two", "key1", "val1");
      int numLocksBefore = cache.getNumberOfLocksHeld();

      cache.getInvocationContext().getOptionOverrides().setFailSilently(true);// will cause any current txs to be suspended for the duration of this call.
      cache.put(Fqn.fromString("/a/b"), "key1", "val1");

      assertEquals(numLocksBefore, cache.getNumberOfLocksHeld());
   }


   public static Test suite()
   {
      return new TestSuite(SuspendTxTest.class);
   }

   public static void main(String[] args)
   {
      junit.textui.TestRunner.run(suite());
   }

}
