package org.jboss.cache.util;

import junit.framework.TestCase;
import org.jboss.util.stream.MarshalledValueInputStream;
import org.jboss.util.stream.MarshalledValueOutputStream;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.HashMap;
import java.util.Map;

public class MapCopyTest extends TestCase
{
   public void testSerializable() throws Exception
   {
      HashMap hm = new HashMap();
      hm.put(null, null);
      hm.put("y", "z");
      MapCopy mc = new MapCopy(hm);
      assertEquals(hm, mc);
      ByteArrayOutputStream os = new ByteArrayOutputStream();
      ObjectOutputStream oos = new ObjectOutputStream(os);
      oos.writeObject(mc);
      ByteArrayInputStream is = new ByteArrayInputStream(os.toByteArray());
      ObjectInputStream ois = new ObjectInputStream(is);
      Object o = ois.readObject();
      assertEquals(hm, o);
   }

   public void testSerializableWithMarshalledValueStream() throws Exception
   {
      HashMap hm = new HashMap();
      hm.put(null, null);
      hm.put("y", "z");
      MapCopy mc = new MapCopy(hm);
      assertEquals(hm, mc);
      ByteArrayOutputStream os = new ByteArrayOutputStream();
      ObjectOutputStream oos = new MarshalledValueOutputStream(os);
      oos.writeObject(mc);
      ByteArrayInputStream is = new ByteArrayInputStream(os.toByteArray());
      ObjectInputStream ois = new MarshalledValueInputStream(is);
      Object o = ois.readObject();
      assertEquals(hm, o);
   }

   public void testNull()
   {
      HashMap hm = new HashMap();
      hm.put(null, null);
      MapCopy mc = new MapCopy(hm);
      assertEquals(hm, mc);
      assertEquals(hm.toString(), mc.toString());

      hm.put(null, "x");
      hm.put("y", null);
      mc = new MapCopy(hm);
      mc.toString();
      assertEquals(true, mc.containsKey("y"));
   }

   public void testAll()
   {
      HashMap hm = new HashMap();
      hm.put("a", "b");
      hm.put("b", "c");
      MapCopy mc = new MapCopy(hm);
      assertEquals(hm, mc);
      assertEquals(hm.size(), mc.size());
      try
      {
         mc.clear();
         fail("read only");
      }
      catch (UnsupportedOperationException e)
      {
      }
      HashMap bhm = new HashMap(hm);
      hm.put("b", "d");
      assertEquals(bhm, mc);
      Map.Entry me = (Map.Entry) mc.entrySet().iterator().next();
      try
      {
         me.setValue("arg");
         fail("read only");
      }
      catch (UnsupportedOperationException e)
      {
      }
   }

   public void testModifications()
   {
      Map hm = new HashMap();
      hm.put("a", "b");
      Map mc = new MapCopy(hm);

      try
      {
         mc.put("x", "y");
         fail("should fail");
      }
      catch (UnsupportedOperationException uoe)
      {
         // ok
      }

      try
      {
         mc.remove("a");
         fail("should fail");
      }
      catch (UnsupportedOperationException uoe)
      {
         // ok
      }

      try
      {
         mc.keySet().iterator().remove();
         fail("should fail");
      }
      catch (UnsupportedOperationException uoe)
      {
         // ok
      }

      try
      {
         mc.entrySet().iterator().remove();
         fail("should fail");
      }
      catch (UnsupportedOperationException uoe)
      {
         // ok
      }

      try
      {
         mc.values().iterator().remove();
         fail("should fail");
      }
      catch (UnsupportedOperationException uoe)
      {
         // ok
      }


   }
}
