/*
  Copyright (C) 2002-2005 JAsCo Development Team
  http://ssel.vub.ac.be/jasco

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
 */
package jasco.runtime;
 
import jasco.options.Options;
import jasco.runtime.aspect.HookPropertyChangeEvent;
import jasco.runtime.cache.CombinedHooksFactory;
import jasco.runtime.connector.Connector;
import jasco.runtime.distribution.DistributedTrapExecutor;
import jasco.runtime.hotswap1.HotSwap;
import jasco.util.logging.Logger;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.TreeSet;
import java.util.Vector;

/*
 * 
 * @author Wim
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class ConnectorRegistry
{	
	private static Vector connectors = new Vector();
	
	private static Vector listeners = new Vector();
	
	private static Thread loadThread;
	
	private static Set threadContext = new HashSet();
	
	private static Vector combinations = new Vector();
	
	private static boolean noConnectors = true;
	
	private static ITrapExecutor trapexecutor = null;
	
	private ConnectorRegistry()  {
	}

	private static final Comparator CONNECTOR_COMPARATOR = new ConnectorComparator();
	
	public static void restartLoadThread()  {
	
		if(loadThread!=null)
			loadThread.stop();
		loadThread=new LoadThread();
		loadThread.start();
	}  
	
	public static void quit()  {
		
		try  {
		loadThread.stop();
		
		while(connectors.size()>0)
			unRegisterConnector((Connector)connectors.elementAt(0),false);
		}catch(Exception ex)  {
			Logger.getInstance().showDebug("Exception occured during killAll: "+ex.getMessage());
		}
	}

	protected static Enumeration allHooks()  {
		
		Vector res = new Vector();
		synchronized(connectors)  {
		
			Iterator i = connectors.iterator();
			while (i.hasNext())  {
				Connector c = (Connector) i.next();
				Enumeration enu = c.getHooks();
				while (enu.hasMoreElements())  {
					res.addElement(enu.nextElement());
				}
			}
		}
		return res.elements();
	}
		
	public static Vector findCutpointForMethod(String aMethod, Object anInstance,MethodJoinpoint aJascoMethod)  {
		return  findCutpointForMethod(aMethod,anInstance,aJascoMethod,false);
	}
				

	public static Vector findCutpointForMethod(String aMethod, Object anInstance,MethodJoinpoint aJascoMethod, boolean forJutta) 
	{	
	
		
		synchronized(connectors)  {
		
		Vector result = new Vector();
		if(connectors.size()==0)
			return result;
		
		Vector connectorsCopy = getConnectorsVector();
		Collections.sort(connectorsCopy,CONNECTOR_COMPARATOR);
		
		Enumeration i = combinations.elements();
		while (i.hasMoreElements())  {
			ConnectorCombinationStrategy c = (ConnectorCombinationStrategy) i.nextElement();	
			connectorsCopy=c.validateCombinations(connectorsCopy,aJascoMethod);
		}
		
		int startID = 0;
		i = connectorsCopy.elements();
		while (i.hasMoreElements()) 	
		{   
			Connector connector = (Connector) i.nextElement();	
			//System.out.println(connector.getName());
			Vector tmp = connector.findCutpointForMethod(anInstance,aJascoMethod,startID,forJutta);	
			//System.out.println(connector.getName()+tmp.size());
			if(tmp!=null)
				addAll(result,tmp);
			startID+=jasco.options.Options.HOOKS_MAX_VALUE+1;	
		}
		
		return result;
		}
	}
	
	protected static void addAll(Vector v, Vector toAdd)  {
	
		Enumeration e = toAdd.elements();
		while(e.hasMoreElements())
			v.addElement(e.nextElement());
	}	
	
	public static void addConnectorCombinationStrategy(ConnectorCombinationStrategy c)  {
	
		combinations.addElement(c);
		fireConnectorRegistryChanged(new CRChangeEvent(CRChangeEvent.COMBSTRAT_ADDED,c));
		
		if(c instanceof DoNotCache) {
			Options.setIsJuttaEnabled(false);
			Logger.getInstance().showOutput("JAsCo Warning: non cachable connector combination strategy detected, Jutta system is disabled...");
		}
	}
	
	public static void CombinationStrategy(ConnectorCombinationStrategy c)  {
	
		combinations.removeElement(c);
		fireConnectorRegistryChanged(new CRChangeEvent(CRChangeEvent.COMBSTRAT_REMOVED,c));
	}
		
	
	public static void addConnectorRegistryListener(ConnectorRegistryListener l)  {
		synchronized(listeners)  {
		listeners.addElement(l);
		}
	}
	
	public static void RegistryListener(ConnectorRegistryListener l)  {
	    synchronized(listeners)  {
		listeners.removeElement(l); 
	    }
	}
	
	protected static void fireConnectorRegistryChanged(CRChangeEvent event)  {
		
		synchronized(listeners)  {
	
		Enumeration i = listeners.elements();
		while (i.hasMoreElements())  {
		
			ConnectorRegistryListener l = (ConnectorRegistryListener) i.nextElement();
			l.registryChanged(event);
		}
		}
	}
			
	
	public static void registerConnector(Connector c) 
	{	
		synchronized(connectors)  {
		if (!connectors.contains(c))  {
			
			connectors.add(c);
			noConnectors=false;
			fireConnectorRegistryChanged(new CRChangeEvent(CRChangeEvent.CONNECTOR_ADDED,c));
			if(Options.doHotSwapJDI())
				HotSwap.hotswaptargetClasses(c);
		}	
		}
	}
	
	public static void unRegisterConnector(Connector c)  {
	
		unRegisterConnector(c,true);
	}
	
	public static void unRegisterConnector(Connector c, boolean notifyAspects) 
	{	
		synchronized(connectors)  {
		connectors.remove(c);	
		fireConnectorRegistryChanged(new CRChangeEvent(CRChangeEvent.CONNECTOR_REMOVED,c));
		if(connectors.size()==0)
			noConnectors=true;
		if(Options.doHotSwapJDI())
				HotSwap.unhotswaptargetClasses(c);	
		
		if(notifyAspects)
			c.firePropertyChangeEvent(new HookPropertyChangeEvent(HookPropertyChangeEvent.IS_REMOVED));
		}
	}
	
	public static void start()  {
	}
	
	private static final File[] classpathArray ;
	
	private static File[] initClasspath()  {
		
		Set res = new TreeSet();
		StringTokenizer tokenizer = new StringTokenizer(Options.getConnectorLoadPath(),Options.PATH_SEPARATOR);
		while (tokenizer.hasMoreTokens()) 
		{
			String token = tokenizer.nextToken();
			File file = new File(token);
			if (file.isDirectory())   {
				//System.out.println("adding ... "+file.getPath());
				try  {
				res.add(file.getCanonicalFile());
				}catch(IOException ex)  {
				}
			}
		}
		File[] f = new File[res.size()];
		res.toArray(f);
		return f;
	}
	
	public static void forceConnectorLoad()  {
	
		if(loadThread!=null)
			loadThread.interrupt();
		else refresh();	
	}	
	
	static 
	{	
		if(Options.isDistributed())
			trapexecutor=new DistributedTrapExecutor();
		else trapexecutor=new DefaultTrapExecutor();
		
		classpathArray=initClasspath();
		if (Options.isJuttaEnabled())  {
			
			if(!Options.doHotSwapJDI())
				Logger.getInstance().showOutput("--- JAsCo Jutta system activated ---");
		}
		
		else if(!Options.doHotSwapJDI()) 
			Logger.getInstance().showOutput("--- JAsCo Jutta system NOT activated ---");
		
		if (jasco.options.Options.getConnectorLoadInterval()>-1)  {
			ConnectorRegistry.load();
			ConnectorRegistry.checkPersistance();
		}
		
		if (jasco.options.Options.getConnectorLoadInterval()>0)  {
			loadThread=new LoadThread();
			loadThread.start();
		}
		else Logger.getInstance().showWarning("Automatic Connector loading disabled");
		
		if (jasco.options.Options.showDebugOutput())  {
			Logger.getInstance().showOutput("JASCO DEBUG: System initialized");
		}
		
		
	}
	
	public static void pauseRefreshing()  {
		
		loadThread.suspend();
	}
	
	public static void resumeRefreshing()  {
		
		loadThread.resume();
	}
	
	protected static HashMap getConnectorsCopy()  {
		
		synchronized(connectors)  {
		HashMap v = new HashMap();
		Iterator i = connectors.iterator();
		while (i.hasNext())  {
			Connector c = (Connector) i.next();
			
			/*if(Options.showDebugOutput())
				System.out.println("JASCO DEBUG: loaded connector: "+c.getName()+";");*/
			v.put(c.getName(),c);
		}
		//System.out.println("---");
		return v;
		}
	}
	
	public synchronized static Iterator getConnectors()  {
	
		return connectors.iterator();
	}
	
	public synchronized static Vector getConnectorsVector()  {
		
			Vector v =  (Vector) connectors.clone();
			
			return v;
		}
	
	protected static void debug()  {	
		
	}
	
	public static  boolean isAdapted(JascoMethod method)  {
		
		return  trapexecutor.isAdapted(method);	
	}
	
	public static Object executeTrap(String sig, Object o, JascoMethod method)  throws Exception {
		
		return  trapexecutor.executeTrap(method);	
	}
	
	public static  boolean isAdaptedAndThreadFlag(JascoMethod method)  {
		
		boolean b =   !noConnectors && method.getCombinedHooks()!=CombinedHooksFactory.EMPTY_CONBINED_HOOKS&&!threadflag;
		
		return b; 
	}
	
	public static  void setThreadFlag()  {
		
		threadflag=true;
		
		//threadContext.add(Thread.currentThread());
	}
	static boolean threadflag = false;
	
	public static  void releaseThreadFlag()  {
		
	
		threadflag=false;
		//threadContext.remove(Thread.currentThread().getName());
		
		//System.out.println("RELEASING");
	}
	
	protected static void load()  {
		
		//debug();
		//System.out.println("number of traps occured: ");
		
		Map connectorsCopy = getConnectorsCopy();
		Class[] tmp = new Class[]  {"".getClass()};
		
		for(int ji=0;ji<classpathArray.length;++ji)  {
			File file = classpathArray[ji];
			File connectorpath = new File(file,"Connector");
			if (connectorpath.exists() && connectorpath.isDirectory()) 
			{
				String[] files = connectorpath.list();
				for(int i=0; i<files.length; ++i)
					try
					{
					File tfile = new File(connectorpath,files[i]);
					String classname = tfile.getName();
					if(!classname.endsWith(".class"))
						continue;
					int j = classname.indexOf('.');
					if(j!=-1)
						classname=classname.substring(0,j);
					String cClass = "Connector."+classname;	
					
					
					if(connectorsCopy.remove(classname)==null)  { //not yet loaded
						Class clz = jasco.options.Options.loadClass(cClass);
						if(!Connector.class.isAssignableFrom(clz))
							continue;
						if (Options.showDebugOutput())  
							System.out.println("JASCO DEBUG: new connector found! "+classname+";");
						clz.getMethod("load",tmp).invoke(null,new Object[]  {tfile.getAbsolutePath()});
					}
					}
					catch(ExceptionInInitializerError e)  {
						
						Logger.getInstance().showDebug(e);
					}
					catch(NoSuchMethodException ex)  {
						
						Logger.getInstance().showDebug(ex);
					}
					catch(ClassFormatError error)  {
						Logger.getInstance().showDebug(error);
					}
					catch(Throwable ex) 
					{
						Logger.getInstance().showDebug(ex);
					}
			}
		}
		
		//all connectors still present in connector have been removed
		Iterator j = connectorsCopy.values().iterator();
		while (j.hasNext())  {
			Connector c = (Connector) j.next();
			unRegisterConnector(c);
		}
		
		synchronized(listeners)  {
	
		Enumeration i = listeners.elements();
		while (i.hasMoreElements())  {
		
			ConnectorRegistryListener l = (ConnectorRegistryListener) i.nextElement();
			l.registryUpdated();
		}
		}
	}
	
	public static void refresh() {
		
		ConnectorRegistry.load();
		ConnectorRegistry.checkPersistance();
	}
	
	public static void checkPersistance()  {
		
		synchronized(connectors)  {
		Iterator i = connectors.iterator();
		boolean change = false;
		while (i.hasNext()) 	
		{
			Connector connector = (Connector) i.next();	
			if(ConnectorPersistance.loadConnector(connector))
				change=true;
			
			if(change) 
				fireConnectorRegistryChanged(new CRChangeEvent(CRChangeEvent.PROPERTY_CHANGE,connector));	
		}
		
		}	
	}
	
	protected static void notifyAspects(Connector c, HookPropertyChangeEvent event)  {
	}
	
		
	public static void reOrderConnectors() {
		
		fireConnectorRegistryChanged(new CRChangeEvent(CRChangeEvent.ALLCHANGE));	
	}

	public static boolean areConnectorsCachable() {
		Iterator i = ConnectorRegistry.getConnectors();
		while(i.hasNext()) {
			Connector c = (Connector) i.next();
			if(!c.adaptOnClasses())
				return false;
		}
		
		return true;
	}
			
}

class ConnectorComparator implements Comparator {
	
	public int compare(Object o1, Object o2) {
		
		if(o1 instanceof Connector && o2 instanceof Connector) {
			
			Connector c1 = (Connector) o1;
			Connector c2 = (Connector) o2;
			
			if(c1.getPriority()<c2.getPriority())
				return 1;
			else if(c2.getPriority()<c1.getPriority())
				return -1;
			else return 0;
		}
		else return -1;
	}
}

class LoadThread extends Thread  {
	
	ThreadGroup parent;
	
	LoadThread()  {
		ThreadGroup group = getThreadGroup();
		while(group.getParent()!=null)
			group=group.getParent();
		parent=group;
		setDaemon(true);	
	}
	
	public void run()  {
		
		if (jasco.options.Options.showDebugOutput())  {
			System.out.println("JASCO DEBUG: jasco aspect framework operational");
		}
		setPriority(Options.getConnectorLoadingPriority());
		int sleeptime = jasco.options.Options.getConnectorLoadInterval();
		
		while (true)  {
			
			try {
			sleep(sleeptime);
			}catch(Exception ex)  {
			}
			ConnectorRegistry.load();
			ConnectorRegistry.checkPersistance();
			
			/*
			System.out.println(parent.activeCount());
			Thread[] t = new Thread[parent.activeCount()];
			parent.enumerate(t);
			for(int i=0;i<t.length;++i)
				System.out.println(t[i]);
			if(parent.activeCount()==1)
				break;*/
		}
	}

}
