package jasco.runtime;

import jasco.options.Options;
import jasco.runtime.aspect.IHook;
import jasco.runtime.aspect.StatefulAspect;
import jasco.runtime.cache.CombinedHooksFactory;
import jasco.runtime.connector.Connector;
import jasco.runtime.connector.DynamicHookElement;
import jasco.runtime.connector.HookElement;
import jasco.util.logging.Logger;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

public class DefaultTrapExecutor implements ITrapExecutor, ConnectorRegistryListener {
	
	private Vector cache;
	private Hashtable cachedItems=new Hashtable();
	private boolean doNotCache = false;
	
	public DefaultTrapExecutor() {
		
		if (Options.isJuttaEnabled())
			cache = new Vector(Options.getJuttaCacheSize());
		
		ConnectorRegistry.addConnectorRegistryListener(this);
		
		initDoNotCache();
	}
	
	protected void addToCachedItems(Vector result,CombinedHooks cached)  {
		
		cachedItems.put(result,cached);
	}
	
	protected void invalidateCache()  {
		
		if(Options.showDebugOutput())
			Logger.getInstance().showOutput("JASCO DEBUG: invalidating cache...");		
		
		//synchronized(cache) {
		Enumeration enu = cache.elements();
		while (enu.hasMoreElements())  {
			JascoMethod m = (JascoMethod) enu.nextElement();
			m.setDoNotCache(false);
			m.setCombinedHooks(null);
		}
		cache.removeAllElements();
		
		Iterator i = ConnectorRegistry.getConnectors();
		while (i.hasNext())  {
			Connector m = (Connector) i.next();
			compCachable(m);
		}
		
	}
	
	protected void compCachable(Connector c)  {
		
		if (!c.adaptOnClasses())  {
			Logger.getInstance().showWarning("connector "+c.getName()+" is not cachable (non-static), Jutta system is not activated");
			c.setDoNotCache(true);
			return;
		}
		
		
		Enumeration hooks = c.getHooks();
		while (hooks.hasMoreElements())  {
			HookElement cpe = (HookElement) hooks.nextElement();
			IHook cp = cpe.getCutpoint();
			if(cp instanceof StatefulAspect||cp instanceof DoNotCache||cpe instanceof DynamicHookElement)
				if (c.getCombinationStrategies().hasMoreElements())  {
					Logger.getInstance().showWarning("hook "+cp.getClass().getName()+" is not cachable (dynamic condition in combination with combination strategy), Jutta system is not activated for connector "+c.getName());
					c.setDoNotCache(true);
					return;
				}
		}
		
		c.setDoNotCache(false);
	} 
	
	protected boolean isCachable(Vector hooks)  {
		Enumeration enu = hooks.elements();
		while (enu.hasMoreElements())  {
			HookElement cpe = (HookElement) enu.nextElement();
			IHook cp = cpe.getCutpoint();
			if(cp._Jasco_getConnector().doNotCache())
				return false;
		}
		return true;
	}
	
	protected CombinedHooks getCachedItem(Vector result)  {
		
		java.util.Enumeration hasherI = cachedItems.keys();
		while(hasherI.hasMoreElements()) {
			Vector res = (Vector) hasherI.nextElement();
			boolean iss = true;
			if(result.size()!=res.size())
				continue;
				
			Enumeration enu = res.elements();
			Enumeration enum2 = result.elements();
			while (enu.hasMoreElements())  {
				if (enu.nextElement()!=enum2.nextElement())  {
					iss=false;
					break;
				}
			}
			if(iss)
				return (CombinedHooks) cachedItems.get(result);
		}
		return null;
	}
	
	public Object executeTrap(JascoMethod aJascoMethod) throws Exception {
		
		if (Options.isJuttaEnabled()&&!doNotCache)  {
			
				//CombinedHooks combinedAspects = (CombinedHooks) cache.get(aMethod);
				CombinedHooks combinedAspects = aJascoMethod.getCombinedHooks();
				if (combinedAspects==null)  {
					
					if (aJascoMethod.doNotCache())  {
						Vector result = ConnectorRegistry.findCutpointForMethod(aJascoMethod.getSignature(),aJascoMethod.getCalledObject(),aJascoMethod,false);
						logJP(aJascoMethod.getSignature(),result);
						return JAsCoJPInterpreter.executeHooks(result,aJascoMethod);
					}
					Vector result = ConnectorRegistry.findCutpointForMethod(aJascoMethod.getSignature(),aJascoMethod.getCalledObject(),aJascoMethod,true);
					logJP(aJascoMethod.getSignature(),result);
					if (!isCachable(result))  {
					
						cache.add(aJascoMethod); 
						aJascoMethod.setDoNotCache(true);
						result=ConnectorRegistry.findCutpointForMethod(aJascoMethod.getSignature(),aJascoMethod.getCalledObject(),aJascoMethod,false);
						return JAsCoJPInterpreter.executeHooks(result,aJascoMethod);
					} 		
					else if (result.size()==0)  {
						combinedAspects=CombinedHooksFactory.EMPTY_CONBINED_HOOKS;
						cache.add(aJascoMethod); 
						aJascoMethod.setCombinedHooks(combinedAspects);
					}
					else  {	
							combinedAspects=getCachedItem(result);
							if (combinedAspects==null)  {
							    try {
								  
							      combinedAspects= CombinedHooksFactory.getDefault().generateCachedAspectJavassist(result,aJascoMethod); 
								  addToCachedItems(result,combinedAspects);
						    	}catch(Exception ex)  {
									System.err.println(ex.getMessage());
									ex.printStackTrace();
								    Logger.getInstance().showError("JAsCo SYSTEM ERROR: Caching aspects failed, executing the old fashioned way...");
								    return JAsCoJPInterpreter.executeHooks(result,aJascoMethod);
							    }
							}
							cache.add(aJascoMethod); 
							aJascoMethod.setCombinedHooks(combinedAspects);
					}
				}
				
			    return combinedAspects.invoke(aJascoMethod);
			}
			else  {
				Vector result = ConnectorRegistry.findCutpointForMethod(aJascoMethod.getSignature(),aJascoMethod.getCalledObject(),aJascoMethod);
				logJP(aJascoMethod.getSignature(),result);
				return JAsCoJPInterpreter.executeHooks(result,aJascoMethod);
			}
	}
	
	/**
	 * @param result
	 */
	private void logJP(String aMethod, Vector result) {
		if (jasco.options.Options.showDebugOutput())  {
			System.out.println("JASCO DEBUG: joinpoint encountered "+aMethod);
			String test = "JASCO DEBUG: matching hooks = ";
			Enumeration j = result.elements();
			if(!j.hasMoreElements())
				test+= "NONE";
			while (j.hasMoreElements())   {
				Object next = j.nextElement();
				if (next instanceof HookElement)  {
					HookElement cp = (HookElement) next;
					test+=cp.getCutpoint().getClass().getName()+cp.getBefore().getExecutionNumber()+" ";
				}
				else test+= j.nextElement().getClass().getName().toString()+" ";
			}
			System.out.println(test);	
		}
	}

	public boolean isAdapted(JascoMethod method) {
		
		return method.getCombinedHooks()!=CombinedHooksFactory.EMPTY_CONBINED_HOOKS;
	}

	public void registryChanged(CRChangeEvent event) {
		if(Options.isJuttaEnabled()) {
			invalidateCache();	
			initDoNotCache();
		}
	}

	private void initDoNotCache() {
		this.doNotCache=!ConnectorRegistry.areConnectorsCachable();
	}

	public void registryUpdated() {
	}

}
