/**
 * 
 */
package jasco.runtime.aspect;

import jasco.runtime.MethodJoinpoint;
import jasco.runtime.distribution.DistributedJascoMethod;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.Vector;

import org.jgroups.protocols.TransportedVectorTime;

/**
 * @author dnielben
 * 27-11-2007. This class Implements a Sequence pointcut with causal relation between
 * the events of the sequence. It asumes that the message constains the 
 * information about the vector clock. If the messages delivered to it
 * are causally ordered, it will match all the seuences found in the trace
 * if the mesages are not causally ordered it will match only the messages
 * that are causally conected and that arrive in the correct order.  It assumes
 * that ordering of messages is done before it receives the mesages.
 * 
 * To see the definition of causal relation check the information about vector clocks.
 */
public class PCutpointConstructorCausalSequence extends
		PCutpointConstructorRegular {

	
	// Only the vector time of the last matched vector time is stored
	// only causal dependent joinpoints are going to be evaluated 
	public org.jgroups.protocols.TransportedVectorTime vectorTimeOfLastMatchedJoinPoint = null;
	jasco.util.logging.Logger log = jasco.util.logging.Logger.getInstance();
	
	/**
	 * 
	 */
	public PCutpointConstructorCausalSequence() {
		super();
		// TODO Auto-generated constructor stub
	}
	
	protected String generateClassCreateCode(String argname) {
		return argname+" = new PCutpointConstructorCausalSequence();"+NEWLINE;
	}
	
	public String generateCreateCode(String argname)  {
	
		String code="";
		
		String newArgName = this.getNextArgName();
		
		code+="PCutpointConstructorCausalSequence "+newArgName+" = null;"+NEWLINE;
		code+=generateClassCreateCode(newArgName);
		code+=argname+" = "+newArgName+";"+NEWLINE;
		
		Iterator i = states.iterator();
		while (i.hasNext())  {
			State state = (State) i.next();
			code+="State "+state.getLabel()+" = new State(\""+state.getLabel()+"\", "+state.getID()+");"+NEWLINE;
			if(state==current)
				code+=newArgName+".setStartState("+state.getLabel()+");"+NEWLINE;
			else code+=newArgName+".addState("+state.getLabel()+");"+NEWLINE;
		}
		TreeMap transitionMap = new TreeMap();
		
		Iterator j = allTransitions().iterator();
		while (j.hasNext())  {
			Transition trans = (Transition) j.next();
			String param =  (String) transitionMap.get(trans.getLabel()+"");
			if (param==null)  {
				param = getNextArgName();
				transitionMap.put(trans.getLabel()+"",param);
			
				code+="PCutpointConstructorApplicationDesignator "+param+" = null;"+NEWLINE;
				code+=trans.getParam().generateCreateCode(param);
			}
			State start = trans.getStartState();
			State dest = trans.getDestState();
			code+=newArgName+".addTransition("+start.getLabel()+", "+trans.getLabel()+", "+param+", "+dest.getLabel()+");"+NEWLINE;
			
		}
		
		return code;
	}
	
	public boolean evalGuards(MethodJoinpoint jp, Vector guards)  {
		//For now we only consider one guard and we hard copy 
		// how to porceed here. This is a point of extendsion and flexibility and could
		// be used cleanly later.
		
		//If there aren't modifiers all events should be accepted
		if(guards.size()==0) return true;
		
		String modifier = (String) guards.iterator().next();
		
		// If there is no clock information I return false, I can't say anything
		// This is an ad-hoc desition it could be changed. I have some doubt if it
		// will affect jutta.
		org.jgroups.protocols.TransportedVectorTime joinPointVectorTime = null;
		if(jp instanceof DistributedJascoMethod)
			joinPointVectorTime = ((DistributedJascoMethod) jp).vectorTime;
		if(joinPointVectorTime == null) return false;
		
		//If is the first event or no vectorclocks are being handled 
		//it returns true for both modifiers either causal or conc 
		if(vectorTimeOfLastMatchedJoinPoint == null) return true;
		
		//Test causality as defined by by Mattern
		if(modifier.equals("causal")){
			//This should be only less no leass or equal. isn't it?
			//equal means parallel also.
			boolean causallyRelated = vectorTimeOfLastMatchedJoinPoint.lessThanOrEqual(joinPointVectorTime);
			return causallyRelated;
		}
		
		//Test concurrency as defined by Mattern
		if(modifier.equals("causal")){
			//This should be only less no less or equal. isn't it?
			//equal means parallel also?
			boolean concurrent= !vectorTimeOfLastMatchedJoinPoint.lessThanOrEqual(joinPointVectorTime) &&
			                      !joinPointVectorTime.lessThanOrEqual(vectorTimeOfLastMatchedJoinPoint);
			return concurrent;
		}
        return false;
	}
	
	public boolean eval(MethodJoinpoint jp,Hashtable mappings,boolean forJutta)  {
		
		if(!forJutta) {
			CachedRegularResult cachedStuff = current.getCachedResult(jp);
			if(cachedStuff!=null) {
				moveState(cachedStuff);
				return true;
			}
		}
		//System.out.println("CURRENT : "+current.getLabel() + "(" + forJutta+")");
		//System.out.println(jp);
		Iterator transitions = current.getTransitions();
		
		if(forJutta)
			transitions =allTransitions().iterator();
		
		while (transitions.hasNext())  {
			Transition trans = (Transition) transitions.next();
			PCutpointConstructorApplicationDesignator next = trans.getParam();
			
			//System.out.println("EVAKIG : "+trans.getLabel());
			// We evaluate guards . Guards eveluation do not change state they are functional.
			// The semantics of this construct updates vector clocks always when an event is  accepted
			if (evalGuards(jp, trans.getGuards()) && next.eval(jp,mappings,forJutta))  {
				if (!forJutta)  {
					if(!next.doNotCache())
						current.storeResult(jp,new CachedRegularResult(trans,trans.getDestState()));
					moveState(trans);
					org.jgroups.protocols.TransportedVectorTime joinPointVectorTime = null;
					if(jp instanceof DistributedJascoMethod)
						joinPointVectorTime = ((DistributedJascoMethod) jp).vectorTime;
					max(joinPointVectorTime);
				}
				return true;
			}
		}
		return false;
	}
	
    public synchronized void max(TransportedVectorTime vector) {
        int otherTimeVector[]=vector.getValues();
        int timeVector[]=vectorTimeOfLastMatchedJoinPoint.getValues();
        if (otherTimeVector.length!=timeVector.length) {
            log.showWarning("max: got message with wrong time vector length: "+otherTimeVector.length+", expected: "+timeVector.length);
            return;
        }
        
        for(int i=0;i<timeVector.length;i++) {
            if (otherTimeVector[i]>timeVector[i]) timeVector[i]=otherTimeVector[i];
        }
    }	
	
	
}
