/**
 * 
 */
package jasco.runtime.aspect;

import jasco.runtime.MethodJoinpoint;
import jasco.runtime.distribution.DistributedJascoMethod;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.TreeMap;

import org.jgroups.Event;
import org.jgroups.Message;
import org.jgroups.protocols.TransportedVectorTime;

/**
 * @author dnielben 10-03-2007. This class Implements a Sequence pointcut with
 *         causal relation between the events of the sequence. It receives the
 *         events and if they are not causally ordered it orders them before
 *         consumming them. It assumes that the message constains the
 *         information about the vector clock.
 * 
 * To see the definition of causal relation check the information about vector
 * clocks.
 */
public class PCutpointConstructorOrderedCausalSequence extends
			PCutpointConstructorRegular {
	

	
	// Only the vector time of the last matched vector time is stored
	// only causal dependent joinpoints are going to be evaluated 
	public org.jgroups.protocols.TransportedVectorTime vectorTimeOfLastMatchedJoinPoint = null;

	/**
	 * Queue containing upward messages waiting for delivery i.e causal order.
	 */
	private LinkedList waitingQueue = new LinkedList();

	private Object lock = new Object();

	/**
	 * 
	 */
	public PCutpointConstructorOrderedCausalSequence() {
		super();
		// TODO Auto-generated constructor stub
	}

	protected String generateClassCreateCode(String argname) {
		return argname + " = new PCutpointConstructorOrderedCausalSequence();"
				+ NEWLINE;
	}

	public String generateCreateCode(String argname) {

		String code = "";

		String newArgName = this.getNextArgName();

		code += "PCutpointConstructorOrderedCausalSequence " + newArgName
				+ " = null;" + NEWLINE;
		code += generateClassCreateCode(newArgName);
		code += argname + " = " + newArgName + ";" + NEWLINE;

		Iterator i = states.iterator();
		while (i.hasNext()) {
			State state = (State) i.next();
			code += "State " + state.getLabel() + " = new State(\""
					+ state.getLabel() + "\", " + state.getID() + ");"
					+ NEWLINE;
			if (state == current)
				code += newArgName + ".setStartState(" + state.getLabel()
						+ ");" + NEWLINE;
			else
				code += newArgName + ".addState(" + state.getLabel() + ");"
						+ NEWLINE;
		}
		TreeMap transitionMap = new TreeMap();

		Iterator j = allTransitions().iterator();
		while (j.hasNext()) {
			Transition trans = (Transition) j.next();
			String param = (String) transitionMap.get(trans.getLabel() + "");
			if (param == null) {
				param = getNextArgName();
				transitionMap.put(trans.getLabel() + "", param);

				code += "PCutpointConstructorApplicationDesignator " + param
						+ " = null;" + NEWLINE;
				code += trans.getParam().generateCreateCode(param);
			}
			State start = trans.getStartState();
			State dest = trans.getDestState();
			code += newArgName + ".addTransition(" + start.getLabel() + ", "
					+ trans.getLabel() + ", " + param + ", " + dest.getLabel()
					+ ");" + NEWLINE;

		}

		return code;
	}

	public boolean eval(MethodJoinpoint jp, Hashtable mappings, boolean forJutta) {
		org.jgroups.protocols.TransportedVectorTime joinPointVectorTime = null;
		boolean tmpEval = false;
		boolean tmpQueuedEval=false;
		
		if (jp instanceof DistributedJascoMethod)
			joinPointVectorTime = ((DistributedJascoMethod) jp).vectorTime;
		boolean causallyNext = (vectorTimeOfLastMatchedJoinPoint != null
				&& joinPointVectorTime != null &&
		        // This should be only less no leass or equal. isn't it?
				// equal means parallel also.
				isCausallyNext(joinPointVectorTime));

		synchronized (lock) {

			// If the vector time of the join point is null it means that
			// is a local event, so it causally should be processed. It can also
			// mean that the causal layer in JGroups is not active and then it
			// should
			// be processed too.
			// If vectorTimeOfLastMatchedJoinPoint is null it could means that
			// either causal layer is inactive or it is the first message in
			// the sequence. In either case the message should be processed.
			// Of course if it is causally dependant it should be processed.
			if (vectorTimeOfLastMatchedJoinPoint == null
					|| joinPointVectorTime == null || causallyNext)
				tmpEval = super.eval(jp, mappings, forJutta);
			else if(joinPointVectorTime != null) addToDelayQueue(((DistributedJascoMethod) jp));

			// if it was matched by the sequence it is now the last matched
			// event.
			if (tmpEval) {
				System.out.println(this.getClass() + " Causally dependent, old event: "
						+ vectorTimeOfLastMatchedJoinPoint + " new event: "
						+ joinPointVectorTime);
				vectorTimeOfLastMatchedJoinPoint = joinPointVectorTime;
			}
			
            TransportedVectorTime queuedVector = null;
            
            while ((!waitingQueue.isEmpty()) &&
                    isCausallyNext((queuedVector = ((DistributedJascoMethod) waitingQueue.getFirst()).vectorTime))) {
            	DistributedJascoMethod dmsg = (DistributedJascoMethod) waitingQueue.getFirst();
                waitingQueue.remove(dmsg);
                tmpQueuedEval = super.eval(dmsg, mappings, forJutta);
                
    			// if it was matched by the sequence it is now the last matched
    			// event.
    			if (tmpQueuedEval) {
    				System.out.println(this.getClass() + " Causally dependent, queued event: "
    						+ vectorTimeOfLastMatchedJoinPoint + " new event: "
    						+ joinPointVectorTime);
    				vectorTimeOfLastMatchedJoinPoint = joinPointVectorTime;
    			}

            }
		}
		// ldbn 12-03-2008. I decide to return the conjuction of both boolean values
		// I have to check if several advices will be triggered or skiped.
		return tmpEval || tmpQueuedEval;
	}

	/**
	 * Adds a the message to the queue
	 * 
	 * @param djp
	 *            A distributed JoinPoint
	 */
	private void addToDelayQueue(DistributedJascoMethod djp) {
		ListIterator i = waitingQueue.listIterator(0);
		DistributedJascoMethod current = null;
		while (i.hasNext()) {
			current = (DistributedJascoMethod) i.next();
			if (djp.vectorTime.lessThanOrEqual(current.vectorTime)) {
				waitingQueue.add(i.previousIndex(), djp);
				return;
			}
		}
		waitingQueue.add(djp);
	}

	public synchronized boolean isCausallyNext(TransportedVectorTime vector) {
		
		//if the currentVectorTime haven't be fixed any message is causally next
		if(vectorTimeOfLastMatchedJoinPoint == null) return true;
		int senderIndex = vector.getSenderIndex();
		int[] timeVector = vectorTimeOfLastMatchedJoinPoint.getValues();
		// if (senderIndex == view.getLocalIndex()) return true;

		int[] otherTimeVector = vector.getValues();

		if (otherTimeVector.length != timeVector.length) {
			System.out
					.println("isCausallyNext: got message with wrong time vector length: "
							+ otherTimeVector.length
							+ ", expected: "
							+ timeVector.length);
			return true;
		}

		boolean nextCausalFromSender = false;
		boolean nextCausal = true;

		for (int i = 0; i < timeVector.length; i++) {
			if ((i == senderIndex) && (otherTimeVector[i] == timeVector[i] + 1)) {
				nextCausalFromSender = true;
				continue;
			}
			// if (i == view.getLocalIndex()) continue;
			if (otherTimeVector[i] > timeVector[i])
				nextCausal = false;
		}

		return (nextCausalFromSender && nextCausal);
	}

}
