/*
  Copyright (C) 2002-2005 JAsCo Development Team
  http://ssel.vub.ac.be/jasco

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
 */
package jasco.runtime.aspect;

import jasco.runtime.MethodJoinpoint;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.Vector;

public class PCutpointConstructorRegular extends PCutpointConstructorApplicationDesignator  
{	
	protected State current;
	protected State startState;
	protected Vector states = new Vector();
	protected int labelToExecute = -1;
	protected TreeMap statementMap = null;
	protected State prevState = null;
	
	public PCutpointConstructorRegular()  {
		super(null);
	}
	
	public void setStatementMap(TreeMap map)  {
	
		statementMap=map;
	}
	
	public TreeMap getStatementMap()  {
	
		return statementMap;
	}
	
	protected String generateClassCreateCode(String argname) {
		return argname+" = new PCutpointConstructorRegular();"+NEWLINE;
	}
	
	public String getPreviousStateID() {
		
		return prevState.getLabel();
	}
	
	public String generateCreateCode(String argname)  {
	
		String code="";
		
		String newArgName = this.getNextArgName();
		
		code+="PCutpointConstructorRegular "+newArgName+" = null;"+NEWLINE;
		code+=generateClassCreateCode(newArgName);
		code+=argname+" = "+newArgName+";"+NEWLINE;
		
		Iterator i = states.iterator();
		while (i.hasNext())  {
			State state = (State) i.next();
			code+="State "+state.getLabel()+" = new State(\""+state.getLabel()+"\", "+state.getID()+");"+NEWLINE;
			if(state==current)
				code+=newArgName+".setStartState("+state.getLabel()+");"+NEWLINE;
			else code+=newArgName+".addState("+state.getLabel()+");"+NEWLINE;
		}
		TreeMap transitionMap = new TreeMap();
		
		Iterator j = allTransitions().iterator();
		while (j.hasNext())  {
			Transition trans = (Transition) j.next();
			String param =  (String) transitionMap.get(trans.getLabel()+"");
			if (param==null)  {
				param = getNextArgName();
				transitionMap.put(trans.getLabel()+"",param);
			
				code+="PCutpointConstructorApplicationDesignator "+param+" = null;"+NEWLINE;
				code+=trans.getParam().generateCreateCode(param);
			}
			State start = trans.getStartState();
			State dest = trans.getDestState();
			code+=newArgName+".addTransition("+start.getLabel()+", "+trans.getLabel()+", "+param+", "+dest.getLabel()+");"+NEWLINE;
			
		}
		
		return code;
	}
	
	public boolean eval(MethodJoinpoint jp,Hashtable mappings,boolean forJutta)  {
		
		if(!forJutta) {
			CachedRegularResult cachedStuff = current.getCachedResult(jp);
			if(cachedStuff!=null) {
				moveState(cachedStuff);
				return true;
			}
		}
		//System.out.println("CURRENT : "+current.getLabel() + "(" + forJutta+")");
		//System.out.println(jp);
		Iterator transitions = current.getTransitions();
		
		if(forJutta)
			transitions =allTransitions().iterator();
		
		while (transitions.hasNext())  {
			Transition trans = (Transition) transitions.next();
			PCutpointConstructorApplicationDesignator next = trans.getParam();
			
			//System.out.println("EVAKIG : "+trans.getLabel());
			if (next.eval(jp,mappings,forJutta))  {
				if (!forJutta)  {
					if(!next.doNotCache())
						current.storeResult(jp,new CachedRegularResult(trans,trans.getDestState()));
					moveState(trans);
				}
				return true;
			}
		}
		return false;
	}
	
	/**
	 * @param cachedStuff
	 */
	protected void moveState(CachedRegularResult cachedStuff) {
		
		moveState(cachedStuff.getTransition(),cachedStuff.getDestState());
	}

	/**
	 * @param transition
	 * @param destState
	 */
	protected void moveState(Transition transition, State destState) {
		
		prevState=current;
		current=destState;
		labelToExecute=transition.getLabel();
	}

	protected void moveState(Transition trans) {
		
		moveState(trans,trans.getDestState());
	}
	
	public Iterator getNextStateIDs() {
		
		Vector v = new Vector();
		Iterator transitions = current.getTransitions();
		while (transitions.hasNext())  {
			Transition trans = (Transition) transitions.next();
			v.add(trans.getDestState().getID()+"");
		}
		return v.iterator();
	}


	public boolean doNotCache()  {
		return true;
	}
	
	
	public void addState(State state)  {
		
			states.add(state);
	}
	
	public void addTransition(State start, int label, PCutpointConstructorApplicationDesignator des, State dest)  {
		
		Transition trans = new Transition(start,label,des);
		start.addTransition(trans,dest);
	}
	
	/**
	 * ldbn 13-03-12007. This method adds a transition with an additional  vector containing gards.
	 * In principle it was added to support selective causality, but I decided to added in this class and not in
	 * the specialized version because
	 * guards are a general concept that could be used later in other 
	 * contexts. However, the behavior and use of guards will be implemented 
	 * on the specialized classes in the eval method.
	 *   
	 * @param start
	 * @param label
	 * @param des
	 * @param dest
	 * @param guards
	 */
	public void addTransition(State start, int label, PCutpointConstructorApplicationDesignator des, State dest, Vector guards)  {
		
		Transition trans = new Transition(start,label,des,guards);
		start.addTransition(trans,dest);
	}	
	
	public void setStartState(State state)  {
		
		if(!states.contains(state))
			states.add(state);
		
		current=state;
		startState=state;
	}
	
	public State getStartState() {
		
		return startState;
	}
	
	
	public Iterator getStates()  {
		return states.iterator();
	}
	
	public String toDescString()  {
	
		return "Stateful pointcut";
	}
	
	public boolean definesName(String name) {
		
		return getStatementMap().get(name)!=null;
	}
	
	public Vector allPointcuts()  {
	
		Vector trans = new Vector();
		Iterator statesI = getStates();
		while (statesI.hasNext())  {
			State s = (State) statesI.next();
			Iterator transI = s.getTransitions();
			while (transI.hasNext())  {
				Transition t = (Transition) transI.next();
				trans.add(t.getParam());
			}
		}
		return trans;
	}
	public Vector allTransitions()  {
	
		Vector trans = new Vector();
		Iterator statesI = getStates();
		while (statesI.hasNext())  {
			State s = (State) statesI.next();
			Iterator transI = s.getTransitions();
			while (transI.hasNext())  {
				Transition t = (Transition) transI.next();
				trans.add(t);
			}
		}
		return trans;
	}
	
	public int getCurrentStateID()  {
		
		return current.getID(); 
	}

	/**
	 * @param name
	 * @return
	 */
	public int getStateIDForName(String name) {
		if(name.equals(PCutpointConstructorComplement.COMPLEMENT_NAME))
			return State.COMPLEMENT;
		else {
			Iterator i = states.iterator();
			while(i.hasNext()) {
				State state = (State) i.next();
				if(state.getLabel().equals(name))
					return state.getID();
			}
		}
		
		throw new IllegalArgumentException("id not found for stateful name: "+name);
	}
		
		
}
	
	