/*
  Copyright (C) 2002-2005 JAsCo Development Team
  http://ssel.vub.ac.be/jasco

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
 */
package jasco.runtime.aspect;

import jasco.runtime.MethodJoinpoint;

import java.util.HashMap;
import java.util.Iterator;
import java.util.TreeMap;

public class State implements Comparable  {  
		
	private String label = "";
	private TreeMap map = new TreeMap();
	private java.util.Vector transitions = new java.util.Vector ();
	private int id;
	private HashMap joinpointCache = new HashMap();
	
	public static final int COMPLEMENT = 1;
	
	private static int idGenerator = 10;
	private Transition firstTransition;
	
	public State(String label)  {
		this(label,idGenerator++);
		
	}
	
	public State(String label, int id)  {
		this.label=label;
		this.id=id;
	}
	
	public void storeResult(MethodJoinpoint jp, CachedRegularResult trans) {
		
		joinpointCache.put(jp,trans);
	}
	
	public CachedRegularResult getCachedResult(MethodJoinpoint jp) {
		
		return (CachedRegularResult)joinpointCache.get(jp);
	}
	
	public State()  {
	
		this("state"+idGenerator++,idGenerator); 
	}
	
	public Transition getTransition() {
		if(firstTransition==null) {
			firstTransition=(Transition) transitions.elementAt(0);
		}
		return firstTransition;
	}
	
	public void addTransition(Transition trans, State dest)  {
		
		map.put(trans,dest);
		transitions.add(trans);
	}
	
	public Iterator getTransitions()  {
	
		return transitions.iterator();
	}
	
	public State getDestState(Transition trans)  {
	
		return (State)map.get(trans);
	}
	
	public String getLabel()  {
	
		return label;
	}
	
	public int getID() {
		
		return id;
	}
	
	public int compareTo(Object o)  {
		
		if (o instanceof State)  {
			State state = (State) o;
			return label.compareTo(state.getLabel()); 
		}
		else return -1;
	}
}