/*
 * Created on 4-apr-2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package jasco.runtime.aspect.factory;

import jasco.runtime.MethodJoinpoint;
import jasco.runtime.aspect.IHook;
import jasco.util.logging.Logger;

import java.util.Iterator;
import java.util.Map;

/* 
 Copyright (C) 2002-2005 JAsCo Development Team
 http://ssel.vub.ac.be/jasco
  
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as
 published by the Free Software Foundation; either version 2 of the
 License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 USA 
 */

/**
 * convenience class. Remmebers hook instances for given keys.
 * 
 * @author Wim
 */

public abstract class AbstractAspectFactory implements IAspectFactory{
	
	private Map table;
	public AbstractAspectFactory(Map myTable) {
		super();
		this.table=myTable;
	}

	/* (non-Javadoc)
	 * @see jasco.runtime.aspect.factory.IAspectFactory#getAspectInstance(jasco.runtime.MethodJoinpoint, jasco.runtime.aspect.factory.IAspectInstanceCreator)
	 */
	public IHook getAspectInstance(MethodJoinpoint jp,
			IAspectInstanceCreator creator) {
		if(jp==null)
			return null;
		Object key = getKey(jp);
		IHook currentHook =  this.getAspectInstance(key,creator);
		return currentHook;
	}
	
	public IHook getAspectInstance(Object key, IAspectInstanceCreator creator) {
		
		if(key==null)
			return null;
		
		IHook currentHook = fetchHook(key);
		if (currentHook==null)  {
		
			try {
				currentHook = creator.createNewInstance();
				storeHook(key,currentHook);
			} catch (HookInstantiationFailed e) {
			Logger.getInstance().showError(e);
			}
		}
		return currentHook;
	}
	
	/**
	 * Returns the key for looking up hooks given this joinpoint. Key may be null, default hook is used in that case.
	 * 
	 * @param jp joinpoint
	 * @return a key for looking up hooks
	 */
	public abstract Object getKey(MethodJoinpoint jp);
	
	/**
	 * fetches a hook for a given key
	 * 
	 * @param key a key for looking up hooks
	 * @return a hook instance
	 */
	public IHook fetchHook(Object key) {
		return (IHook) table.get(key);
	}
	
	/**
	 * stores a hook for a given key
	 * 
	 * @param key 
	 * @param aHook a hook
	 */
	public void storeHook(Object key, IHook aHook) {
		table.put(key,aHook);
	}
	
	public Iterator getAll() {
		
		return table.values().iterator();
	}

}
