package jasco.runtime.carma;

import jasco.options.Options;
import jasco.runtime.JascoMethod;
import jasco.runtime.MethodJoinpoint;
import jasco.util.logging.Logger;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import javassist.CtClass;
import jpl.Atom;
import jpl.Query;
import jpl.Term;
import jpl.fli.Prolog;
import wim.factgen.FactGenFacade;

public class CarmaEvaluator {
	private static CarmaEvaluator instance = new CarmaEvaluator();
	public static CarmaEvaluator getInstance() {
		return instance;
	}
	
	private FactGenFacade factgen = FactGenFacade.getInstance();
	private long id = 0;
	
	public CarmaEvaluator() {
		 Prolog.set_default_init_args(
                 new String[] {
                         "libpl.dll",
                         "-f", "none",
                         "-g", "set_prolog_flag(debug_on_error,false)",
                         "-q"
                 }
         );

	}
	 
	public void registerClass(CtClass clz) {
		
		factgen.addClass(clz);
	}
	
	public boolean eval(String pc, MethodJoinpoint jp) {
		
		if(!Options.isCarmaEnabled()) {
			Logger.getInstance().showError("Carma not enabled at VM startup, eval pointcut failed");
			return true;
		}
		consultChangedDB();
		
		if(!(jp instanceof JascoMethod)) {
			Logger.getInstance().showWarning("Carma pointcut eval failed because thisJoinPoint is not a JascoMethod: "+jp.getClass().getName());
			return true;
		}
		
		JascoMethod mjp = (JascoMethod) jp;
		String jpname = factgen.getJPForMethod(mjp.getJavassistMethod());
		
		if(jpname==null) {
			Logger.getInstance().showError("jp not found in Carma: "+jp.toString());
			return true;
		}
		
		pc=pc.replace("thisJoinPoint",jpname);
		
		String toEval=pc;
	
		Query q1 = new Query(pc);
	
		return q1.hasSolution();
	} 

	private void consultChangedDB() {
		String prologResult = factgen.getIncrementalPrologFacts();
		if(prologResult==null) return;
		
		File file = newDBFile();
		try {
		FileWriter writer = new FileWriter(file);
		writer.write(prologResult);
		writer.close();
		
		
		Query q1 = 
		    new Query( 
		        "consult", 
		        new Term[] {new Atom(file.getAbsolutePath())} 
		    );
		
		if(!q1.hasSolution())
			Logger.getInstance().showError("db file "+file.getAbsolutePath()+" not consulted successfully!");
		}catch(IOException ex) {
			Logger.getInstance().showError(ex);
		}
		
		
		
	}

	private File newDBFile() {
		File dir = Options.getLogDir();
		return new File(dir,"clzbg"+id+++".pl");
	}
}
