/**
 * 
 */
package jasco.runtime.distribution;

// JGroups imports
import jasco.runtime.ConnectorRegistry;
import jasco.runtime.JascoMethod;
import jasco.util.distribution.JascoThreadCflow;
import jasco.util.logging.Logger;

import org.jgroups.Channel;
import org.jgroups.JChannel;
import org.jgroups.Message;
import org.jgroups.blocks.GroupRequest;
import org.jgroups.blocks.MessageDispatcher;
import org.jgroups.blocks.RequestHandler;
import org.jgroups.stack.IpAddress;
import org.jgroups.util.Rsp;
import org.jgroups.util.RspList;

/**
 * This singleton class manages the comunication of the Jasco
 * joinpoint(methodsInvocations) events with the group of instances that belongs
 * to the same group.
 * 
 * @author lbenavid
 */
public class DistributePointcutHandler implements RequestHandler {
	private static DistributePointcutHandler _managerInstance;

	Channel channel;

	MessageDispatcher disp;

	RspList rsp_list;

	String props = DistributedOptions.getInstance().getStack("AspectsGroup");

	/**
	 * This class is a singleton class
	 */
	private DistributePointcutHandler() {
		try {
			start();
		} catch (Exception e) {
			Logger.getInstance().showDebug(e);
		}
	}

	public Channel getChannel() {

		return channel;
	}

	public void start() throws Exception {
		channel = new JChannel(props);
		channel.setOpt(Channel.LOCAL, false);// Not self delivery
		disp = new MessageDispatcher(channel, null, null, this);
		channel.connect(DistributedOptions.getInstance().getGroupName("MessageDispatcherTestGroup"));
		Logger.getInstance().showDebug(channel.getView().toString());
	}

	public static DistributePointcutHandler getInstance() {
		return _managerInstance;
	}

	/** Broad cast a message and do not care bout the answers */
	public void broadcastMessageAsync(DistributedStandardMessage jm) {
		RspList rsp_list = disp.castMessage(null, new Message(null, null, jm),
				GroupRequest.GET_NONE, 0);
	}

	/** Broad cast a message and waits for all the answers */
	public RspList broadcastMessageSync(DistributedStandardMessage jm) {
		RspList rsp_list = disp.castMessage(null, new Message(null, null, jm),
				GroupRequest.GET_ALL, 0);
		return rsp_list;
	}

	/** Broad cast a message and waits for the first answer */
	public RspList broadcastMessageFirstAnswer(DistributedStandardMessage jm) {
		RspList rsp_list = disp.castMessage(null, new Message(null, null, jm),
				GroupRequest.GET_FIRST, 0);
		return rsp_list;
	}


	/** Send a synchronous Message to an specific host */
	public Object sendSyncMessage(DistributedStandardMessage jm, Host host) {
		RspList rsp_list = disp.castMessage(null, new Message(
				host.getAddress(), null, jm), GroupRequest.GET_FIRST, 0);
		if (rsp_list.size() <= 0)
			throw new HostNotFoundException(host);
		else
			return ((Rsp) rsp_list.elementAt(0)).getValue();
	}

	public void sendNotification(DistributedJascoMethod jm) {
		//String thr= Thread.currentThread().getName();
		//System.out.println("In send Notification " + thr);
		if (!jm.isRemoteJP()) {
			//System.out.println("In send Notification is remote " + thr);
			if (jm.getHost() == null)
				jm.setHost(new Host((IpAddress) channel.getLocalAddress()));
			try {
				//System.out.println("Ready to prepare " + thr);
				DistributedJascoMethod djm = jm.clone();
				djm.prepareArguments();
				djm.setRemoteJP(true);
				//System.out.println("prepared " + thr);
				broadcastMessageAsync(new DistributedStandardMessage(djm,
						DistributedStandardMessage.REMOTE_JP));
			} catch (Exception e) {
				//System.out.println("Error in prepare " + thr);
				e.printStackTrace();
			}
		}	
	}
	
	public Object invokeRemoteSynchronous(DistributedAdviceExecution adviceExecution) {
		Host host = adviceExecution.getHost();
		return invokeRemoteOn(host,adviceExecution,DistributedStandardMessage.REMOTE_SYNC_MESSAGE);
	}
	
	public Object getRemoteAsyncResult(Host host, long taskid) {
		
		Object resp = sendSyncMessage(new DistributedStandardMessage(taskid,DistributedStandardMessage.REMOTE_FETCH_ASYNC_RESULT),host);
		return resp;
	}
	
	
	public Object invokeRemoteOn(Host host, DistributedAdviceExecution adviceExecution, int message) {
		
		//Modifies the Jp information to only send serializable information
		//Parameters are send by copy only when they are serializable
		//This solution is using the same object, this can be problematic for
		//concurrency, so in the future a clone of the jp can be used ldbn 22-9-2004
		DistributedJascoMethod jm = adviceExecution.getJP();
		Object[] args = jm.getArgumentsArray();
		Object tmpObject = jm.getCalledObject();
		Object[] argscopy = new Object[args.length]; 
		for (int i = 0; i < args.length; ++i){
			argscopy[i] = args[i];
		} 
		/*
		 * Object[] argscopy = new Object[args.length]; for (int i = 0; i <
		 * args.length; ++i) { argscopy[i] = args[i]; if (!(args[i] instanceof
		 * Serializable)) args[i] = null; } jm.setArgumentsArray(args); if
		 * (!(tmpObject instanceof Serializable)) jm.setCalledObject(null);
		 */
		// This is just to cactch an undesired exception...
		// after the try-catch the ancien values are restored in
		// jm so the impact should be minimum
		try {
			jm.prepareArguments();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		//Make the remote call
		
		Object resp = sendSyncMessage(new DistributedStandardMessage(adviceExecution
				,message), host);
		
		//restore jp values
	    jm.setCalledObject(tmpObject);
	    jm.setArgumentsArray(argscopy);

		return resp;
	}

	/**
	 * This methods execute the trap to the remote pointcut. If the trap sends
	 * an error a false is answered, if the traps executes ok a true is
	 * answered. This can be changed to send objects, but for now there is no
	 * need for that.
	 */
	public Object receiveNotification(JascoMethod jm) {
		Object result;
		try {
			ConnectorRegistry.executeTrap(jm.getName(), jm.getCalledObject(),
					jm);
			result = true;
		} catch (Exception e) {
			Logger.getInstance().showDebug(e);
			result = false;
		}
		return result;
	}

	
	public Object handle(Message msg) {
		Logger.getInstance().showDebug(msg.toString());
		
		
		
		DistributedStandardMessage dsmsg = (DistributedStandardMessage) msg.getObject();
		
		//dsmsg.vectorTime = msg.getHeader(org.jgroups.protocols.)
		
		/* Disabled because it gives raise to problems!!!!
		
		try {
			((DistributedJascoMethod) dsmsg.getMsg()).createRemoteReferences();
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}*/
		
		//Comment by ldbn 26-06-2006. This line was added to support remote cflow
		JascoThreadCflow.setRemoteInfo(dsmsg.getRemoteInfo()); 
		if(dsmsg.getType() == DistributedStandardMessage.REMOTE_JP){
			
			// ldbn 27-11-2007. Add the vector time information to this message.
			// This is used to determine the causal relation.
			
			DistributedJascoMethod djm = (DistributedJascoMethod) dsmsg.getMsg();
			djm.vectorTime = ((org.jgroups.protocols.CAUSALTAGS.CausalHeader) msg.getHeader("CAUSAL")).getVectorTime();
			//System.out.println(this.getClass() + " received a message with vector clock: " + djm.vectorTime.toString());
		
			return receiveNotification((JascoMethod) djm);
		}	
		if(dsmsg.getType() == DistributedStandardMessage.REMOTE_SYNC_MESSAGE)
			return DistributedInvoker.getInstance().invokeSynchronous(
					(DistributedAdviceExecution) dsmsg.getMsg());
		if(dsmsg.getType() == DistributedStandardMessage.REMOTE_ASYNC_MESSAGE)

			return DistributedInvoker.getInstance().invokeASynchronous(
					(DistributedAdviceExecution) dsmsg.getMsg());
		if(dsmsg.getType() == DistributedStandardMessage.REMOTE_FETCH_ASYNC_RESULT) {
			Object result =  FutureManager.getInstance().retrieveFutureResult(
					(Long) dsmsg.getMsg());
			return result;
		}
		return null; 
	}

	/**
	 * initializes this manager instance
	 * 
	 */
	public static void init() {
		if (!initialized())
			_managerInstance = new DistributePointcutHandler();
		else
			throw new IllegalArgumentException(
					"Cannot initialize distribution manager more than once");
	}

	public static boolean initialized() {
		return _managerInstance != null;
	}

	public ObjectFutureWrapper invokeRemoteASynchronous(DistributedAdviceExecution adviceExecution) {
		Host host = adviceExecution.getHost();
		return (ObjectFutureWrapper) invokeRemoteOn(host,adviceExecution,DistributedStandardMessage.REMOTE_ASYNC_MESSAGE);
	}

}
