/**
 * 
 */
package jasco.runtime.distribution;

// JGroups imports
import jasco.runtime.connector.Connector;
import jasco.util.logging.Logger;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.util.HashSet;

import javassist.bytecode.ClassFile;

import org.jgroups.Channel;
import org.jgroups.JChannel;
import org.jgroups.Message;
import org.jgroups.blocks.GroupRequest;
import org.jgroups.blocks.MessageDispatcher;
import org.jgroups.blocks.RequestHandler;
import org.jgroups.util.RspList;

/**
 * This singleton class manages the comunication of the Jasco
 * connector events with the group of instances that belongs 
 * to the same group.
 * 
 * @author lbenavid
 */
public class DistributedConnectorHandler implements RequestHandler {
	private static DistributedConnectorHandler _handlerInstance =
		new DistributedConnectorHandler();

	private Channel channel;

	private MessageDispatcher disp;

	private RspList rsp_list;

	private String 
	props = DistributedOptions.getInstance().getStack("ConnectorsGroup"); 

	private HashSet connSet = new HashSet();

	/**
	 * This class is a singleton class
	 */
	private DistributedConnectorHandler() {
		try {
			start();
			// new model: no jp distr anymore
			//ConnectorRegistry.addConnectorRegistryListener(
			//		new DistributedConnectorListener());
			// 4-06-2007 LDBN: It seems that conector distributions is deisabled since long time ago
			// I disable it completly. We can reactivate it in the DistributedTrapExecutor!
			//
		} catch (Exception e) {
			Logger.getInstance().showDebug(e);
		}
	}

	public Channel getChannel() {

		return channel;
	}

	public void start() throws Exception {
		channel = new JChannel(props);
		channel.setOpt(Channel.LOCAL, false);// Not self delivery
		disp = new MessageDispatcher(channel, null, null, this);
		channel.connect(DistributedOptions.getInstance().getGroupName(
				"ConnectorsGroup"));
		Logger.getInstance().showDebug(channel.getView().toString());
	}

	public static DistributedConnectorHandler getInstance() {
		return _handlerInstance;
	}

	public void sendNotification(DistributedJascoMessage dm) {
		rsp_list = disp.castMessage(null, new Message(null, null, dm),
				GroupRequest.GET_NONE, 0);
	}

	/**
	 * This methods is executed when a connector notification is received
	 */
	public Object receiveNotification(DistributedJascoMessage aMessage) {
		Object result;
		try {
			if (aMessage.getType() == DistributedConnectorMessage.ADD_CONNECTOR) {
				ByteArrayInputStream in = 
					new ByteArrayInputStream((byte[]) aMessage.getMsg());
				ClassFile cf = new ClassFile(new DataInputStream(in));
				(new File("Connector")).mkdir();
				File f = new File("Connector/" +
						cf.getSourceFile().replaceFirst(".java",".class"));
				FileOutputStream out = new FileOutputStream(f);
				cf.write(new DataOutputStream(out));
				f.deleteOnExit();
				out.close();
				connSet.add(cf.getName());
			}
			if (aMessage.getType() == DistributedConnectorMessage.REMOVE_CONNECTOR) {
				String fileName = aMessage.getMsg().toString().replace('.', '/') + ".class";
				System.out.println(fileName);
				File f = new File(fileName);
				f.delete();
			}			
			result = true;
		} catch (Exception e) {
			Logger.getInstance().showDebug(e);
			result = false;
		}
		return result;
	}

	public Object handle(Message msg) {
		Logger.getInstance().showDebug(msg.toString());
		return receiveNotification((DistributedJascoMessage) msg.getObject());
	}

	/**
	 * @return Returns the connSet.
	 */
	public HashSet getConnSet() {
		return connSet;
	}

	/**
	 * @param connSet
	 *            The connSet to set.
	 */
	public void setConnSet(HashSet connSet) {
		this.connSet = connSet;
	}

	/**
	 * Return true if the connector is loaded in the local jasco runtime
	 */
	public boolean isLocal(Connector c) {
		return !connSet.contains(c.getClass().getName());
	}
}
