package jasco.runtime.distribution;

import jasco.runtime.JAsCoBeanCallback;
import jasco.runtime.JascoMethod;
import jasco.runtime.ReflectJascoMethod;
import jasco.runtime.distribution.remoteref.RemoteAWEDReference;
import jasco.runtime.distribution.remoteref.RemoteAWEDReferenceImpl;
import jasco.runtime.transform.TransformerConstants;

import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Modifier;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Map;
import java.util.TreeMap;

import javassist.CtMethod;
import javassist.NotFoundException;

public class DistributedJascoMethod extends ReflectJascoMethod {
	
	//	LDBN 6-6-2005 introduce the host parameter, in principle will identify 
	// this node in the group created by jgroups
	private Host host;
	private boolean remoteJP = false;

	// LDBN 24-05-2006 This structures stores the stubs and types that
	// reference remotely the objects passed by reference 
	private Map remoteStubs = new TreeMap();
	private Map remoteTypes = new TreeMap();
	
	// LDBN 27-11-2007. Store the vector time of the message
	// null means local message, or causal layer in Jgroups not activated
	// so no causal relation can be derived
	public org.jgroups.protocols.TransportedVectorTime vectorTime = null;
	
	public DistributedJascoMethod(String fullname, String aName,
			String aClassName, int methodid, JAsCoBeanCallback callbackObject,
			int modifiers, boolean isExecution) {
		super(fullname, aName, aClassName, methodid, callbackObject, modifiers,
				isExecution);
	}
	
	public Object invokeOriginal(Object o, Object[] args) throws Exception
	{	
		if(o==null&&!Modifier.isStatic(getModifiers()))
			return null;
		//LDBN 19-6-2007 This code was added to fix a bug that appears when the
		// adviced method throws an exception and the arround avice with procedd or
		// the arround throwing advice were used in the aspect. Basically the expected
		// exeption was the throwed one but the one that was being received was
		// InvocationTargetException. This behavior is only present in DJASCO, 
		// Normal Jasco execution returned the expected exception. So, we now
		// preprocees the exception taking out this wrapping exception and throwing the
		// original one. It seems that the ancient code:
		//   return super.invokeOriginal(o, args)
		// is not the correct one and JAsCo (non distributed) uses maybe another method
		// TODO: Check that invokeOriginal is the correct method to call.
		else {
			Object returnObject = new Object();
			try{ 
				returnObject = super.invokeOriginal(o,args);
			}catch(InvocationTargetException ex)  {				
				Throwable source = ex.getCause();
				if(source instanceof Exception)
					throw (Exception) source;
				else throw ex;
			}
			return returnObject;
		}
		/*
		if(host.equals(InetAddress.getLocalHost()))
			
		else return DistributedTrapExecutor.invokeProceed(this,o,args);
		*/
	}
	
	public Host getHost() {
		
		return host;
	}
	
	public void setHost(Host host) {
		
		this.host=host;
	}

	/**
	 * @return Returns the remoteJP.
	 */
	public boolean isRemoteJP() {
		return remoteJP;
	}

	/**
	 * @param remoteJP The remoteJP to set.
	 */
	public void setRemoteJP(boolean remoteJP) {
		this.remoteJP = remoteJP;
	}
	 
    public static JascoMethod buildFrom(CtMethod method,ClassLoader loader, int id) throws NotFoundException {
		
		String fullName = TransformerConstants.getFullName(method);
		
		JascoMethod theMethod = new DistributedJascoMethod(
				fullName,  
				method.getName(), 
				method.getDeclaringClass().getName(), 
				id, 
				null, 
				method.getModifiers(), 
				true);
		
		initFrom(theMethod,method,loader,id);
		
		return theMethod;
	}

	public void setAtributesStubs(Map atributesStubs) {
		this.remoteStubs = atributesStubs;
	}

	public Map getAtributesStubs() {
		return remoteStubs;
	}
	
	//ldbn 29-05-2006 creates the remote references before distributing the message
	public void prepareArguments() throws Exception {
			  remoteStubs.put("target", getStub(this.getCalledObject()));
			  remoteTypes.put("target", this.getCalledObject().getClass());
			  Object[] args = this.getArgumentsArray();
			  for(int i=0;i<args.length;++i) {
				  if(!(args[i] instanceof Serializable)){
					  remoteStubs.put((new Integer(i)).toString(), getStub(args[i]));
					  remoteTypes.put((new Integer(i)).toString(), args[i].getClass());
					  args[i]=null;
				 }
			  }
			  if(!((this.getCalledObject()) instanceof Serializable)) this.setCalledObject(null);
	}
	
	public DistributedJascoMethod clone() throws CloneNotSupportedException{
		DistributedJascoMethod djm= (DistributedJascoMethod) super.clone();
		djm.setArgumentsArray(djm.getArgumentsArray().clone());
		return djm;
	}
	
	
	/** This method is called once the message arrives to the host.
	 * There the remote reference objects are created. First using
	 * the type information is used to create an object. Then using aspects
	 * all the methods of such object are instrumented to create a
	 * proxy object that will redirect the method calls to the
	 * remote reference.ldbn 27-10-2006.
	 * @throws IllegalAccessException 
	 * @throws InstantiationException 
	*/
	public void createRemoteReferences() throws InstantiationException, IllegalAccessException{
		//-- Estracting the target Object ---
	      //1. Extract the type
	      //	Class targetType = (Class) remoteTypes.get("target");
		  //2. Crate the proxy. Remember that the target is always remote(as specified in AWED)		
	      //	this.setCalledObject(targetType.newInstance());
		  //3. Set the instance in the ReferenceManager
	      //	ReferenceManager.getInstance().addDummyInstance(this.getCalledObject());
	      //Note: that the previous step does creates an load the corresponding aspect
	     
	    //---Now extracting the arguments using the sames steps
		// The references are creted in the actual arguments array
		Object[] args = this.getArgumentsArray();
		for(int i=0; i < args.length;i++){
			  if(false && args[i]==null){
				try {
				// extract the types and creates the dummy object	
					args[i]=((Class) remoteStubs.get((new Integer(i)).toString())).newInstance();
				// Set the instance in the reference manager. This step creates 
				// automatically the aspects and connectors.
					ReferenceManager.getInstance().addDummyInstance(args[i]);
				} catch (InstantiationException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (IllegalAccessException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			  }	
		}
	}
	
	
	// ldbn 29-05-2006 Returns a genreal remote reference with a field pointing
	// to the object parameter. 
	private RemoteAWEDReference getStub(Object m) throws RemoteException {
		RemoteAWEDReferenceImpl refimpl = new RemoteAWEDReferenceImpl(m);
		RemoteAWEDReference stub = (RemoteAWEDReference) UnicastRemoteObject.exportObject(refimpl, 0);
		return stub;
	}
}
