package jasco.runtime.distribution;

import java.io.Serializable;
import java.net.UnknownHostException;

import org.jgroups.stack.IpAddress;

public class Host implements Serializable {

	private static final int NO_PORT = Integer.MAX_VALUE;

	private static final String LOCALHOST = "localhost";

	private IpAddress address;

	public Host(IpAddress address) {
		super();
		this.address = address;
	}

	public boolean equals(Object o) {

		if (o instanceof Host)
			return equals((Host) o);
		else
			return super.equals(o);
	}

	public boolean equals(Host host) {

		if (host == null) {
			return isLocalHost();
		} else if (host.address.getPort() == NO_PORT
				|| address.getPort() == NO_PORT) // no prt spec, only ips
			return host.address.getIpAddress().equals(address.getIpAddress());
		else
			return address.equals(host.address);
	}

	public String toString() {

		if (this.equals(getLocalHost()))
			return LOCALHOST;
		else
			return address.getIpAddress().getHostAddress() + ":"
					+ address.getPort();
	}

	public boolean isLocalHost() {

		return this.equals(getLocalHost());
	}

	public static Host getLocalHost() {

		if (DistributePointcutHandler.initialized())
			return new Host((IpAddress) DistributePointcutHandler.getInstance()
					.getChannel().getLocalAddress());
		//LDBN:21-11-2007: It seems that this second return is used at compilation time
		//I leave it because if replced by null the compilers shows a nullpointer exception
		// It was necessary to introduce the try catch because the IpAddress was changed
		// in Jgroups and now is throwing a UnknowHostException
		try {
			return new Host(new IpAddress("127.0.0.1", NO_PORT));
		} catch (UnknownHostException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	public static Host buildFrom(String desc) {
		Host host = null;

		if (desc.equals(LOCALHOST))
			return getLocalHost();

		int index = desc.indexOf(':');
		if (index < 0) {// no port spec
			try {
				host = new Host(new IpAddress(desc, NO_PORT));
			} catch (UnknownHostException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} else {

			String ip = desc.substring(0, index);
			String portS = desc.substring(index + 1);
			Integer port = new Integer(portS);

			try {
				return new Host(new IpAddress(ip, port.intValue()));
			} catch (UnknownHostException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		host = new Host(new IpAddress());
		return host;
	}

	/**
	 * @return Returns the address.
	 */
	public IpAddress getAddress() {
		return address;
	}
}
