/*
  Copyright (C) 2002-2005 JAsCo Development Team
  http://ssel.vub.ac.be/jasco

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
 */
package jasco.runtime.hotswap;

import jasco.options.Options;
import jasco.runtime.CRChangeEvent;
import jasco.runtime.ConnectorRegistry;
import jasco.runtime.ConnectorRegistryListener;
import jasco.runtime.carma.CarmaEvaluator;
import jasco.runtime.connector.Connector;
import jasco.runtime.inline.InlineCompiler;
import jasco.runtime.transform.JAsCoClassTransformer;
import jasco.util.logging.ErrorOutputLogger;
import jasco.util.logging.Logger;

import java.lang.instrument.ClassDefinition;
import java.lang.instrument.ClassFileTransformer;
import java.lang.instrument.Instrumentation;
import java.security.ProtectionDomain;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;

import javassist.ClassPool;
import javassist.CtClass;

public class JAsCoTransformer implements ClassFileTransformer,ConnectorRegistryListener {
	
	private Logger logger = new ErrorOutputLogger();
	private Set loadedClasses = new TreeSet();
	private Vector listeners = new Vector();
	
	
	public JAsCoTransformer()  {
	
		
		ConnectorRegistry.addConnectorRegistryListener(this);
	}
	
	public JAsCoHotSwap getHotSwap()  {
	
		return JAsCoHotSwap.getSingleton();
	}
	
	
	public TransformClassChecker getTransformChecker()  {
	
		return TransformClassChecker.getSingleton();
	}

	public void registryChanged(CRChangeEvent event)  {
	
		switch (event.getType())  {
		
			case CRChangeEvent.CONNECTOR_ADDED:
				hotswaptargetClasses((Connector)event.getParam());
				break;
			case CRChangeEvent.CONNECTOR_REMOVED:
				unhotswaptargetClasses((Connector)event.getParam());
				break;
			case CRChangeEvent.PROPERTY_CHANGE:
				cpropertyChange((Connector) event.getParam());
				break;
			case CRChangeEvent.ALLCHANGE:
			case CRChangeEvent.COMBSTRAT_ADDED:
			case CRChangeEvent.COMBSTRAT_REMOVED:
				connectorCombStratChange();
				break;
			
		}
	}
	
	public void registryUpdated()  {
	}
	
	protected void connectorCombStratChange()  {
	
		Iterator i = ConnectorRegistry.getConnectors();
		while (i.hasNext())  {
			refreshInlineCompiledMethods((Connector)i.next());
		}
	}
	
	protected void cpropertyChange(Connector c)  {
		
		boolean enabled = c.isEnabled();
		boolean sstatic = c.adaptOnClasses();
		c.setEnabledImpl(true);
		c.setAdaptOnClassesImpl(true);
		Map toRemoveMethods = removeInlineCompiledMethods(c);
		c.setEnabledImpl(enabled);
		c.setAdaptOnClassesImpl(sstatic);
		if(toRemoveMethods!=null)
			checkWhetherConnectorsAppy(toRemoveMethods); 
	}

	protected void doSingleMethodUnhotswap(Hashtable removalMethods) throws Exception {
		
		Vector swaptable = new Vector();
		Iterator i = removalMethods.entrySet().iterator();
		while (i.hasNext())  {
			Map.Entry entry = (Map.Entry) i.next();
			String classname = (String) entry.getKey();
			Vector methods = (Vector) entry.getValue();
			
			byte[] code = removeTraps(classname,methods);
			
			if(code!=null)
				swaptable.add(new ClassDefinition(Class.forName(classname),code));
		}
		
		ClassDefinition[] clzs = new ClassDefinition[swaptable.size()];
		swaptable.copyInto(clzs);
		
		getHotSwap().redefineClasses(clzs);
	}
	
	
	protected byte[] removeTraps(String classname, Vector methods) throws Exception {
		
		CtClass clazz = JAsCoClassLoader.getSingleton().loadClass(classname);
		JAsCoClassTransformer transformer = new JAsCoClassTransformer(clazz);
			
		byte[] code = transformer.removeTrapsAtRuntimeByName(methods);
		if(code==null)
			return null;
		
		Vector v = (Vector) getAdaptedMethods().get(classname);
		if(v!=null)
			v.removeAll(methods);
		
		InlineCompiler.getDefault().removeInlineCompiledMethods(classname,methods);
		
		return transformer.getResultingByteCode();
	}
	
	public Hashtable getAdaptedMethods() {
		
		return JAsCoWeaver.getInstance().getAdaptedMethods();
	}
		
	public void unhotswaptargetClasses(Connector unload)  {
		
		this.refreshInlineCompiledMethods(unload);
	try {
		Hashtable toremoveMethods = new Hashtable();
		Hashtable removalMethods = getAdaptedMethods();
		Enumeration adaptedMethodsEnum = removalMethods.keys();
		while (adaptedMethodsEnum.hasMoreElements())  {
			String matchingclassname = (String)adaptedMethodsEnum.nextElement();
			Vector matchingmethods = (Vector) removalMethods.get(matchingclassname);
		
			Set resultset = AspectApplicationComputer.getInstance().getMethodsWhereHooksAreApplicable(matchingclassname);

			Vector toremove = new Vector();
			Iterator matchingmethodsI = matchingmethods.iterator();
			while (matchingmethodsI.hasNext())  {
				
				String matchingmethod = (String) matchingmethodsI.next();
				if (!resultset.contains(matchingmethod))  {
					toremove.add(matchingmethod);
				}
			}	
			toremoveMethods.put(matchingclassname,toremove);
		} 
		
		doSingleMethodUnhotswap(toremoveMethods);
	}
	catch(Exception ex)  {
			Logger.getInstance().showError("Unloading failed: ");
			Logger.getInstance().showError(ex);		
	}
	}
	
	public void refreshInlineCompiledMethods(Connector theconnector) {
		
		Map toremoveMethods = removeInlineCompiledMethods(theconnector);
		if(toremoveMethods==null)
			return;
		checkWhetherConnectorsAppy(toremoveMethods);
	}
	
	public Map removeInlineCompiledMethods(Connector theconnector)  {
		
		if(!InlineCompiler.getDefault().isEnabled())
			return null;
		
		try {
		Hashtable toremoveMethods = new Hashtable();
		Hashtable inlineCompiled = InlineCompiler.getDefault().getInlineCompiledMethods();
		
		Enumeration adaptedMethodsEnum = inlineCompiled.keys();
		while (adaptedMethodsEnum.hasMoreElements())  {
			String matchingclassname = (String)adaptedMethodsEnum.nextElement();
			Set matchedmethods = AspectApplicationComputer.getInstance().getMethodsWhereHooksAreApplicable(matchingclassname,theconnector);
			
			Vector result = new Vector();
			result.addAll(matchedmethods);
			toremoveMethods.put(matchingclassname,result);
		}
		doSingleMethodUnhotswap(toremoveMethods);
		return toremoveMethods;
		}
		catch(Exception ex)  {
			Logger.getInstance().showError("Unloading inline compiled methods failed: ");
			Logger.getInstance().showError(ex);	
			return null;
		}
	}
	
	//TODO possible to optimize...
	public void checkWhetherConnectorsAppy(Map removalMethods)  {
		
		try {
			hotswaptargetClasses();
		} catch (Exception e) {
			Logger.getInstance().showError(e);
		}
	}
	
	public void hotswaptargetClasses() throws Exception {
		
		try  {
			
				Vector swaptable = new Vector();
				
				Class[] classes = getHotSwap().getAllClasses();
				for(int i=0;i<classes.length;++i)  {
					Class nextClass = classes[i];
					try {
						if(getTransformChecker().checkValidType(nextClass.getName())==null)
							continue;
						
						CtClass clz = JAsCoClassLoader.getSingleton().loadClass(nextClass.getName());
						byte[] code = JAsCoWeaver.getInstance().transformClass(clz,nextClass.getClassLoader());
						
						
						if(code!=null) {
							Logger.getInstance().showDebug("Hotswapping: "+nextClass.getName());	
							swaptable.add(new ClassDefinition(nextClass,code));
						}
						
						
						
					}
					catch(Exception ex)  {
						Logger.getInstance().showError("Transforming failed: "+nextClass.getName());
						Logger.getInstance().showError(ex);
						//ex.printStackTrace();
					}
				}
				
				ClassDefinition[] clzs = new ClassDefinition[swaptable.size()];
				swaptable.copyInto(clzs);
				
				getHotSwap().redefineClasses(clzs);
				
			}
			catch(Exception ex)  {
					Logger.getInstance().showError("Hotswapping failed for connector:");
					Logger.getInstance().showError(ex);
				}
	}
		
	
	public void hotswaptargetClasses(Connector c)  {
		
		try  {
			Vector swaptable = new Vector();
			
			Class[] classes = getHotSwap().getAllClasses();
			for(int i=0;i<classes.length;++i)  {
				Class nextClass = classes[i];
				try {
					if(getTransformChecker().checkValidType(nextClass.getName())==null)
						continue;
					
					
					byte[] code = JAsCoWeaver.getInstance().transformClass(nextClass.getName(),nextClass.getClassLoader());
					if(code!=null)
						swaptable.add(new ClassDefinition(nextClass,code));
					/*
					try {
						
						if(code!=null) {
					ClassDefinition[] clzs = new ClassDefinition[1];
					clzs[0]=new ClassDefinition(nextClass,code);
					getHotSwap().redefineClasses(clzs);
						} 
					}catch(Throwable ex)  {
						System.out.println("11111111111111111111111111111111111111111");
						FileOutputStream writer = new FileOutputStream("/home/lbenavid/boe.class");
						writer.write(code);
						writer.close();
						ex.printStackTrace();
						System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@2");
						Logger.getInstance().showError("!!!!!!!!BUG!!!!!!!: "+nextClass.getName());
						Logger.getInstance().showError(ex);
					} */
				}
				catch(Exception ex)  {
					Logger.getInstance().showError("Transforming failed: "+nextClass.getName());
					Logger.getInstance().showError(ex);
				}
			}
			
			ClassDefinition[] clzs = new ClassDefinition[swaptable.size()];
			swaptable.copyInto(clzs);
			getHotSwap().redefineClasses(clzs);
			
			
		}
		catch(Exception ex)  {
				Logger.getInstance().showError("Hotswapping failed for connector: "+c.getName());
				Logger.getInstance().showError(ex);
			}
	}
	
	
	
	

	protected ClassPool getClassPool()  {
	
		return JAsCoClassLoader.getSingleton().getClassPool();
	}
	
	public Instrumentation getIntrumentation()  {
	
		return JAsCoHotSwap.getSingleton().getInstrumentation();
	}
	
	public void addClassLoadingListener(ClassLoadingListener l) {
		
		listeners.add(l);
	}
	public void removeClassLoadingListener(ClassLoadingListener l) {
		
		listeners.remove(l);
	}	
	
	protected void addPathsToClassPath(ClassLoader loader) {
		
		JAsCoClassLoader.getSingleton().appendPathsToClassPath(loader);
	}
	long totalTime = 0;
	
	public byte[] transform(ClassLoader loader, String className, Class classBeingRedefined, ProtectionDomain protectionDomain, byte[] classfileBuffer)  {
		
		
		if(classBeingRedefined!=null)
			return null;
		
		try  {
	
		String clazzname = className.replace('/','.') ;
	
		if(loader!=null)
			addPathsToClassPath(loader);
	
		CtClass clz = getTransformChecker().checkValidType(clazzname);
		if(clz==null)
			return null;
		
		if(Options.isCarmaEnabled())
			CarmaEvaluator.getInstance().registerClass(clz);
		
		JAsCoClassTransformer preTransformer = new JAsCoClassTransformer(clz);
		Logger.getInstance().showDebug("preprocessing: "+className);
		boolean result =  preTransformer.preTransform();
		
		if(!result)
			return null;
		
		byte[] code =  JAsCoWeaver.getInstance().transformClass(clz,loader,true);
		if(code==null) { //no aspects applicable, load preprocessed instead
			code=clz.toBytecode();
			clz.defrost();
		}
		
		fireClassLoadingEvent(clz,className);

		return code;
		}
		catch(Throwable ex)  {
			Logger.getInstance().showError("class transformation failed for "+className+" ("+ex.getClass().toString()+" -> "+ex.getMessage()+")");
			Logger.getInstance().showDebug(ex);
		} 
		return null;
	}
	
	protected void fireClassLoadingEvent(CtClass clz, String classname) {
		
		Iterator i = listeners.iterator();
		while(i.hasNext()) {
			ClassLoadingListener l = (ClassLoadingListener) i.next();
			l.classLoaded(clz,classname);
		}
	}
}
	