/*
 * Created on 5-apr-2005
 *
 * 
 * 
 */
package jasco.runtime.hotswap;

import jasco.runtime.JascoMethod;
import jasco.runtime.connector.HookElement;
import jasco.runtime.inline.InlineCompiler;
import jasco.runtime.inline.InlineJutta;
import jasco.runtime.transform.JAsCoClassTransformer;
import jasco.util.logging.Logger;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;

import javassist.CannotCompileException;
import javassist.CtClass;
import javassist.NotFoundException;

/* 
 Copyright (C) 2002-2005 JAsCo Development Team
 http://ssel.vub.ac.be/jasco
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as
 published by the Free Software Foundation; either version 2 of the
 License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 USA 
 */

/**
 * @author Wim
 */

public class JAsCoWeaver {
	
	private static JAsCoWeaver instance = new JAsCoWeaver();
	
	private Hashtable adaptedMethods = new Hashtable();
	/**
	 * 
	 */
	private JAsCoWeaver() {
		super();
	}
	
	public static JAsCoWeaver getInstance() {
		
		return instance;
	}
	
	protected byte[] transformClass(CtClass cls, ClassLoader loader) throws IOException, CannotCompileException {
		
		
		return transformClass(cls, loader,false);
	}
	
	
	protected byte[] transformClass(CtClass cls, ClassLoader loader,boolean schemaChanges) throws IOException, CannotCompileException {
		
		boolean adapted = false;
		JascoMethod[] jps = AspectApplicationComputer.getInstance().computeJoinPoints(cls,loader);
		Set matchingMethods = new TreeSet();
		InlineJutta ijutta = new InlineJutta(cls);
		for(int i=0;i<jps.length;++i) {
			
			Vector hooks =  AspectApplicationComputer.getInstance().computeApplicableHooks(jps[i]);
			if(hooks.isEmpty())
				continue;
			
			adapted=true;
			if(!transformJoinpoint(jps[i],cls,loader,ijutta,schemaChanges,hooks)) 
				matchingMethods.add(jps[i].getName());
		}

		ijutta.finalizeWeaving();
		
		AspectApplicationComputer.getInstance().releaseDoNotLoadFlag(jps);
		
		if(!adapted)
			return null;
		
		if(!matchingMethods.isEmpty())
			return insertRegularTrap(cls,cls.getName(),loader,matchingMethods);
		
		byte[] code = cls.toBytecode();
		cls.defrost();
		
		return code;
	}
	
	protected boolean transformJoinpoint(JascoMethod jp, CtClass cls, ClassLoader loader, InlineJutta ijutta,boolean schemaChanges, Vector hooks) {
		
		
		if(isNotCachable(hooks)) {
			return false;
		}
		else {
			
			boolean result = InlineCompiler.getDefault().inlineCompile(cls,hooks,jp,ijutta,schemaChanges);
			return result;
		}
	}
	
	/**
	 * @param hooks
	 * @return
	 */
	private boolean isNotCachable(Vector hooks) {
		Iterator  i = hooks.iterator();
		while(i.hasNext()) {
			HookElement next = (HookElement) i.next();
			if(next.getCutpoint()._Jasco_getConnector().doNotCache()) 
				return true;
		}
		return false;
	}

	protected byte[] insertRegularTrap(CtClass cls, String classname, ClassLoader loader,Set matchingMethods)  {
		
		
		try  {
			JAsCoClassTransformer transformer = new JAsCoClassTransformer(cls); 
			Vector adaptedMethodsV = (Vector) getAdaptedMethods().get(classname);
			if(adaptedMethodsV==null) {
				adaptedMethodsV=new Vector();
			}
			
			
			Logger.getInstance().showDebug("hotswapping class: "+cls.getName());
			byte[] result = transformer.insertTrapsAtRuntime(adaptedMethodsV,matchingMethods,loader);
			if(result==null||transformer.getAdaptedMethods().size()==0)
				return null;
			
			addAdaptedMethods(classname,transformer.getAdaptedMethods());
			
			return result;
		}
		
		catch(Throwable x)  {
			Logger.getInstance().showError("Transforming failed :"+classname);
			Logger.getInstance().showError(x.getMessage());
			x.printStackTrace();
		}
		
		return null;
	}
	
	protected void addAdaptedMethods(String classname, Vector methods)  {
		
		Vector adapted = (Vector) getAdaptedMethods().get(classname);
		if (adapted==null)  {
			adapted=new Vector();
			getAdaptedMethods().put(classname,adapted);
		}
		
		adapted.addAll(methods);
		
		InlineCompiler.getDefault().removeInlineCompiledMethods(classname,methods);
	}
	
	public Hashtable getAdaptedMethods()  {
		
		return adaptedMethods;
	}
	
	
	public Hashtable getAdaptedMethodsForRemoval()  {
		
		return join(getAdaptedMethods(),jasco.runtime.inline.InlineCompiler.getDefault().getInlineCompiledMethods());
	}
	
	protected Hashtable join(Hashtable table1, Hashtable table2)  {
		
		Hashtable res= new Hashtable();
		addAll(res,table1);
		addAll(res,table2);
		return res;
	}
	
	protected void addAll(Hashtable res, Hashtable table)  {
		
		Iterator i = table.entrySet().iterator();
		while (i.hasNext())  {
			
			Map.Entry entry = (Map.Entry) i.next();
			String s = (String) entry.getKey();
			Vector v = (Vector) entry.getValue();
			Vector resV = (Vector) res.get(s);
			if (resV==null)  {
				resV=new Vector();
				res.put(s,resV);
			}
			resV.addAll(v);
		}
	}
	
	public boolean trapRemoved(String classname, String methodname)  {
		
		Vector v = (Vector)getAdaptedMethods().get(classname);
		if(v==null)
			return false;
		
		return v.remove(methodname);
	}
	
	public byte[] transformClass(String name, ClassLoader classLoader) throws IOException, CannotCompileException {
		CtClass clz=null;
		try {
			clz = JAsCoClassLoader.getSingleton().loadClass(name);
		} catch (NotFoundException e) {
			Logger.getInstance().showError(e);
		}
		
		return transformClass(clz,classLoader);
	}
			
	
}
