/*
  Copyright (C) 2002-2005 JAsCo Development Team
  http://ssel.vub.ac.be/jasco

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
 */
package jasco.runtime.hotswap1;

import jasco.options.Options;
import jasco.runtime.ConnectorRegistry;
import jasco.runtime.connector.Connector;
import jasco.runtime.connector.ISignatureMatcher;
import jasco.runtime.connector.MethodSignature;
import jasco.util.logging.ErrorOutputLogger;
import jasco.util.logging.Logger;

import java.net.InetAddress;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.TreeSet;
import java.util.Vector;

import javassist.ByteArrayClassPath;
import javassist.ClassPool;
import javassist.CtClass;
 

public class HotSwapInVM  {
	
	private static final RuntimeBeanTransformerJavassist beantransformer = new RuntimeBeanTransformerJavassist();
	
	
	public static final int SERVER_PORT = Options.juttaserverport();
	
	public static final String TOTRANSFORM = "TOTRANSFORM"; 
	public static final String ADAPTEDMETHODS = "ADAPTEDMETHODS"; 
	public static final String DONE = "DONEMOAJDNSNSOSJSJSJS;"; 
	public static final String sepChar = ";";
	
	private static final Vector EMPTY_VECTOR = new Vector();
	private static final byte[] EMPTY_BYTES = new byte[0];
	
	private static InetAddress netaddress = null;
	private static ClassPool classPool = ClassPool.getDefault();
	private static Set loaded = new TreeSet();
	
	private static VMTCPNetwork network;
	private static final Logger logger = new ErrorOutputLogger();
	
	private static boolean doContinue;
	
	static  {
		try {
		netaddress = InetAddress.getLocalHost();
		}
		catch(Exception ex)  {
			Logger.getInstance().showError(ex);
		}
		if (Options.doHotSwapJDI())  {
		
			network = new VMTCPNetwork(SERVER_PORT,netaddress,true);
			network.addNetworkListener(new NetworkListenerClass());
		}
		else  {
		
			network = new VMTCPNetwork(SERVER_PORT,netaddress,false);
		
			//network.addNetworkListener(new NetworkListenerClass2());
		}
	}
	
	public static byte[] transformLoadedClassInsideVM(String classname, byte[] bytes) throws ClassNotFoundException {
		
		
		long time = System.currentTimeMillis();		
		if(loaded.contains(classname))
			return bytes;
		
		
		Logger 	tmpLogger = Logger.getInstance();
		String outputBackup = Options.getOutputDir();
		try  {
		
		if(!HotSwap.checkValidType(classname))
			return bytes;
		
		network.sendPacket(TOTRANSFORM+sepChar+classname);
		classPool.insertClassPath(new ByteArrayClassPath(classname, bytes));
		
		beantransformer.setClassPool(classPool);
		
		
		
		Options.setOutputDir(Options.getTempDir().getCanonicalPath());
		long ttime=0;
		Vector adaptedMethods = new Vector();
		Iterator connectors = ConnectorRegistry.getConnectors();
		while (connectors.hasNext())  {
			Connector theconnector = (Connector) connectors.next();
			Enumeration enumer = theconnector.getMethods();
			while (enumer.hasMoreElements())  {
				ISignatureMatcher sig = (ISignatureMatcher)enumer.nextElement();

				
					if (!Options.showDebugOutput())  
						Logger.setInstance(logger);
					
				    ttime = System.currentTimeMillis();	
				    
				    Iterator contents = sig.getSignatures();
				    boolean res=false;
				    while(contents.hasNext()) {
				    	MethodSignature signature = (MethodSignature) contents.next();
				    	if(!signature.matchesClass(classname, classPool))
				    		continue;
				    	Logger.getInstance().showDebug("hotswapping "+classname);
				    	res =res|| beantransformer.transformBean(classname,adaptedMethods,true,signature.getMethodname());
				    }
				    
					
					ttime = System.currentTimeMillis()-ttime;
					Logger.setInstance(tmpLogger);
					
					if(!res)
						continue;
				 	adaptedMethods.addAll(beantransformer.getAdaptedMethods());	
				
			}
		}
		
		String tosend = ADAPTEDMETHODS+sepChar+classname;
		
		Iterator i = adaptedMethods.iterator();
		while (i.hasNext())  {
			String next = (String) i.next();
			tosend+=sepChar+next;
		}
		doContinue=false;
		loaded.add(classname);
		network.sendPacket(tosend);
		//waitForAnswer();
		
		ClassPool cl = new ClassPool(null);
		cl.appendSystemPath();
		CtClass clz = cl.get(classname);	
		byte[] code = beantransformer.getDecompiledClassBytecode();
		
		time = System.currentTimeMillis()-time;
		Options.setOutputDir(outputBackup);
		//System.out.println("hotswapping "+classname+" took:"+time+"(transformbean took"+ttime+")");
		
		
		return code;
		}
		
		catch(Throwable x)  {
			Logger.getInstance().showError("Transforming failed :"+classname);
			Logger.getInstance().showError(x.getMessage());
			x.printStackTrace();
		}
		
		Options.setOutputDir(outputBackup);
		return EMPTY_BYTES;
		
		
		
		
	}
	
	public HotSwapInVM()  {
	}
	
	protected static class NetworkListenerClass implements NetworkListener  {
	
		public void receive(String message)  {
			//System.out.println("received:"+message);
			if (message.startsWith(TOTRANSFORM))
				totransformMessage(message);
			else if (message.startsWith(ADAPTEDMETHODS))
				adaptedMethodsMessage(message);	
				
		}
	}
	
	protected static void waitForAnswer()  {
		
		while (!doContinue)  {
			try {
			Thread.sleep(100);
			}catch(Exception ex)  {
			}
		}
	}
	
	protected static class NetworkListenerClass2 implements NetworkListener  {
		
		public void receive(String message)  {
			//System.out.println("received:"+message);
			try {
				}catch(Exception ex)  {
			}
			if(message.equals(DONE))
				doContinue=	true;
		}
	}
	
	protected static void totransformMessage(String message)  {
	
		StringTokenizer tokenizer = new StringTokenizer(message,sepChar);
		tokenizer.nextElement();
		String clname = (String) tokenizer.nextElement();
		
		HotSwap.saveOriginal(clname);
	}
	
	protected static void adaptedMethodsMessage(String message)  {
		
		StringTokenizer tokenizer = new StringTokenizer(message,sepChar);
		tokenizer.nextElement();
		String clname = (String) tokenizer.nextElement();
		Vector adaptedMethods = new Vector();
		while (tokenizer.hasMoreElements())  {
			adaptedMethods.add(tokenizer.nextElement());
		}
		
		HotSwap.saveAdaptedMethods(clname,adaptedMethods);
		
		//network.sendPacket(DONE);
	}
	
}