/*
 * Created on 11-apr-2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package jasco.runtime.inline;

import jasco.runtime.MethodJoinpoint;
import jasco.runtime.aspect.IHook;
import jasco.runtime.aspect.factory.PerObjectAspectFactory;
import jasco.runtime.connector.DynamicHookElement;
import jasco.runtime.connector.HookElement;
import jasco.runtime.transform.TransformerConstants;
import jasco.util.generators.JavaGenerator;
import jasco.util.logging.Logger;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import javassist.CannotCompileException;
import javassist.CtClass;
import javassist.CtField;

/* 
 Copyright (C) 2002-2005 JAsCo Development Team
 http://ssel.vub.ac.be/jasco
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as
 published by the Free Software Foundation; either version 2 of the
 License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 USA 
 */

/**
 * @author Wim
 */

public class PerInstanceOptimizer {

	private static PerInstanceOptimizer instance = new PerInstanceOptimizer();
	private static final String HOOKID = "_JAsCo_hookInstance";
	
	private Map nameHookElementMap = new HashMap();
	private Map hookElementNameMap = new HashMap();
	
	private int currentId;
	
	private PerInstanceOptimizer() {
		super();
	}
	
	public void processClass(CtClass clz, Vector applicableHookElements) {
		
		Vector perInstanceHooks = getNewPerInstanceHookElements(applicableHookElements);
		if(perInstanceHooks.isEmpty())
			return;
		
		Iterator res = perInstanceHooks.iterator();
		while(res.hasNext()) {
			DynamicHookElement next = (DynamicHookElement) res.next();
			processClass(clz,next);
		}
	}
	
	private String getNewFieldName() {
		
		return HOOKID+currentId++;
	}

	/**
	 * @param clz
	 * @param next
	 */
	private void processClass(CtClass clz, DynamicHookElement next) {
		
		String fieldName = getNewFieldName();
		String fieldSource = "protected "+next.getCutpoint().getClass().getName()+" "+fieldName+";";
//		//fieldSource+="("+next.getCutpoint().getClass().getName()+")"; //NOT SUPPORTED By JAVASSIST ($0)
//		fieldSource+=this.getClass().getName()+"#";
//		fieldSource+="getInstance().getAspectInstance(\""+fieldName+"\", $0);"+JavaGenerator.NEWLINE;
		try {
			CtField field = CtField.make(fieldSource,clz);
			TransformerConstants.makeSynthetic(field);
			clz.addField(field);
			
			nameHookElementMap.put(fieldName,next);
			hookElementNameMap.put(next,fieldName);
		} catch (CannotCompileException e) {
			Logger.getInstance().showError(e);
		}
	}
	
	public boolean isOptmized(HookElement element) {
		
		
		boolean b =  hookElementNameMap.containsKey(element);
		return b;
	}
	
	public String getInstanceFetchString(HookElement next, MethodJoinpoint jp) {
		
		if(java.lang.reflect.Modifier.isStatic(jp.getModifiers()))
			return null;
		
		String name = (String) hookElementNameMap.get(next);
		if(name==null) throw new IllegalArgumentException("error in perinstance optimzier, name should not be null!");
		return "$0."+name;
	}
	
	public IHook getAspectInstance(String fieldName, Object instance) {
		
		DynamicHookElement hElement = (DynamicHookElement) nameHookElementMap.get(fieldName);
		if(hElement == null) throw new IllegalArgumentException("error in perinstance optimzier, helement should not be null!");
		
		PerObjectAspectFactory factory = (PerObjectAspectFactory)hElement.getAspectFactory();
		return factory.getAspectInstance(instance,hElement);
		
	}

	/**
	 * @return
	 */
	public static PerInstanceOptimizer getInstance() {
		return instance;
	}

	/**
	 * @param applicableHookElements
	 * @return
	 */
	private Vector getNewPerInstanceHookElements(Vector applicableHookElements) {
		
		Vector res = new Vector();
		Iterator j =applicableHookElements.iterator();
		while(j.hasNext()) {
			HookElement element = (HookElement) j.next();
			if(element instanceof DynamicHookElement) {
				DynamicHookElement dynelement = (DynamicHookElement) element;
				if(dynelement.getAspectFactory() instanceof PerObjectAspectFactory && !hookElementNameMap.containsKey(dynelement))
					res.add(dynelement);
			}
		}
		
		return res;
	}

	/**
	 * @param el
	 * @param facInvokeCode
	 * @return
	 */
	public String generateOptimziedFactoryInit(DynamicHookElement el, String facInvokeCode, String assignStartCode, MethodJoinpoint jp) {
		if(java.lang.reflect.Modifier.isStatic(jp.getModifiers())) //static is default to use
			return "";

		String code = "";
		String fetchHookInstance = getInstanceFetchString(el,jp);
		code+="if("+fetchHookInstance+"==jasco.runtime.NULL#instance)";
		code+=fetchHookInstance+"= "+facInvokeCode;
		code+=assignStartCode+fetchHookInstance+";"+JavaGenerator.NEWLINE;
		
		return code;
	}


}
