/*
 * Created on 12-mrt-2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package jasco.runtime.mixin;

import jasco.runtime.CRChangeEvent;
import jasco.runtime.ConnectorRegistry;
import jasco.runtime.ConnectorRegistryListener;
import jasco.runtime.aspect.IHook;
import jasco.runtime.connector.HookElement;

import java.util.Iterator;
import java.util.Map;
import java.util.Vector;
import java.util.WeakHashMap;

/* 
 Copyright (C) 2002-2005 JAsCo Development Team
 http://ssel.vub.ac.be/jasco
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as
 published by the Free Software Foundation; either version 2 of the
 License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 USA 
 */

/**
 * @author Wim
 */

public class MixinManager implements ConnectorRegistryListener {
	
	private static final MixinManager instance = new MixinManager();
	
	private Map mixinCache = new WeakHashMap(); 
	
	
	
	/**
	 * 
	 */
	private MixinManager() {
		super();
		
		
		// TODO Auto-generated constructor stub
	}
	
	public static final MixinManager getInstance() {
		
		return instance;
	}
	
	public Object mixinOf(Object target, Class toImplement) throws NoMixinFound {
		
		if(!toImplement.isInterface())
			throw new IllegalArgumentException("Virtual mixin should be an interface!");
		
		if(toImplement.isAssignableFrom(target.getClass())) //already subclass of mixin, return this directly
			return target;
		
		
		Vector result = getApplicableHooks(target);
		Iterator i = result.iterator();
		while(i.hasNext()) {
			IHook o = (IHook) i.next();
			if(isTargetFor(o,toImplement))
				return o;
		}
		
		throw new NoMixinFound(toImplement.getName()+" for "+target);
	}

	/** 
	 * @param o
	 * @param toImplement
	 * @return
	 */
	private boolean isTargetFor(IHook o, Class toImplement) {
		
		return toImplement.isAssignableFrom(o.getClass());
	}

	/**
	 * @param target
	 * @return
	 */
	private Vector getApplicableHooks(Object target) {
		
		if(mixinCache.containsKey(target))
			return (Vector) mixinCache.get(target);
		
		DummyMixinJP dummyJP = new DummyMixinJP(target);
		Vector result = ConnectorRegistry.findCutpointForMethod(dummyJP.getSignature(),target,dummyJP);
		Vector cpResult = new Vector();
		Iterator i = result.iterator();
		while(i.hasNext()) {
			HookElement o = (HookElement) i.next();
			IHook cp = o.getCutpoint();
			cpResult.add(cp);
		}
		
		mixinCache.put(target,cpResult);
		
		return cpResult;	
	}

	/* (non-Javadoc)
	 * @see jasco.runtime.ConnectorRegistryListener#registryChanged(jasco.runtime.CRChangeEvent)
	 */
	public void registryChanged(CRChangeEvent event) {
		invalidateCache();
	}

	/**
	 * 
	 */
	private void invalidateCache() {
		mixinCache.clear();
		
	}

	/* (non-Javadoc)
	 * @see jasco.runtime.ConnectorRegistryListener#registryUpdated()
	 */
	public void registryUpdated() {
	
	}
}
