/*
  Copyright (C) 2002-2005 JAsCo Development Team
  http://ssel.vub.ac.be/jasco

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
 */
package jasco.runtime.transform;

import jasco.options.Options;

import java.io.File;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import javassist.CtClass;
import javassist.CtMethod;

public class JAsCoClassTransformer  {
		
	private CtClass clazz;
	private Vector adaptedMethods;
	
	
	public JAsCoClassTransformer(CtClass clz)  {
		this.clazz=clz;
	}
	
	public boolean preTransform() throws Exception{
		
		MethodAccumulator methodAcc = new MethodAccumulator();
		methodAcc.setDebugString("Preprocessing");
		ClassPreTransformer preprocessor = new ClassPreTransformer();
		 
		methodAcc.process(clazz,clazz.getName());
		preprocessor.setProcessMethods(methodAcc.getResultingMethods());
	
		boolean result = preprocessor.process(clazz,clazz.getName());
		
		return result;
	}
	
	/**
	 * when loader is null, static init is generated that inits all jasco methods directly, otherwise the loader is used 
	 * 
	 * @param excludedmethods
	 * @param loader
	 * @return
	 * @throws Exception
	 */
	public byte[] insertTrapsAtRuntime(Vector excludedmethods, Set matchingmethods, ClassLoader loader) throws Exception {
		

		MethodAccumulator methodAcc = new MethodAccumulator(excludedmethods,matchingmethods);
		boolean result = methodAcc.process(clazz,clazz.getName());
		
		if(!result)
			return null;
		
		String classname = clazz.getName();
		String classFileName=classname.replace('.','/');
		File classFile = new File(Options.getTempDir()+File.separator + classFileName + ".class");
		new File(classFile.getParent()).mkdirs();	
		
		Vector methods = methodAcc.getResultingMethods();
		adaptedMethods=new Vector();
		Iterator methodsI = methods.iterator();
		while (methodsI.hasNext())  {
			CtMethod method = (CtMethod) methodsI.next();
			adaptedMethods.add(TransformerConstants.getFullName(method));
		}
		
		TrapInserter trapinserter = new TrapInserter();
		trapinserter.setProcessMethods(methods);
		trapinserter.setUseFieldClass(clazz,true);
		
		FieldGenerator fields = null;
		
		if(Options.isDistributed())
			fields=new DistributedFieldGenerator(trapinserter,loader);
		else if(Options.useReflectionForProceed()) {
			fields=new ReflectFieldGenerator(trapinserter,loader);
		}
		else fields=new FieldGenerator(trapinserter,loader);
		
		CallbackGenerator callback = new CallbackGenerator(trapinserter);
		
		callback.process(clazz,clazz.getName());
		callback.generateClass(Options.getTempDir());
		 
		fields.process(clazz,clazz.getName());
		fields.generateClass(Options.getTempDir());
		
		trapinserter.process(clazz,clazz.getName());
		
		if(jasco.options.Options.doTransformEvents()) {
			EventTransformer transformer = new EventTransformer(trapinserter);
			transformer.process(clazz,clazz.getName());
		}
			
		return trapinserter.getTargetByteCode();
	}
	
	
	
	protected void debug(Vector debugTEST)   {
		
		try  {
		Class testCLASS = Class.forName(clazz.getName());
		//Vector debugTEST2 = trapinserter.debug();
		Vector debugTEST2 = new Vector();
		java.lang.reflect.Method[] debugMETHODS = testCLASS.getDeclaredMethods();
		for(int i=0;i<debugMETHODS.length;++i)
			debugTEST2.add(debugMETHODS[i].getName());
		debug(debugTEST,debugTEST2);
		}catch(Throwable ex)  {
		System.err.println(ex.getMessage());
		ex.printStackTrace();
		}
	}
	
	protected Vector debug()  {
		try  {
		Class testCLASS = Class.forName(clazz.getName());
		//Vector debugTEST2 = trapinserter.debug();
		Vector debugTEST2 = new Vector();
		java.lang.reflect.Method[] debugMETHODS = testCLASS.getDeclaredMethods();
		for(int i=0;i<debugMETHODS.length;++i)
			debugTEST2.add(debugMETHODS[i].getName());
		return debugTEST2;
		}catch(Throwable ex)  {System.err.println(ex.getMessage());ex.printStackTrace();
		}
		
		return new Vector();
	}
		
	
	protected void debug(Vector v1, Vector v2)  {
		
		System.out.println("XXXXX checking for new methods....");
		Iterator i=v2.iterator();
		while (i.hasNext())  {
			Object o = i.next();
			jasco.util.logging.Logger.getInstance().showWarning(o+"");
			if(!v1.contains(o))
				jasco.util.logging.Logger.getInstance().showWarning("new method added:"+o);
		}
	}
	
	public byte[] getResultingByteCode() throws Exception  {
		
		clazz.stopPruning(true);
		byte[] code = clazz.toBytecode();
		clazz.defrost();
		return code;
	}
	
	public Vector getAdaptedMethods()  {
	
		return adaptedMethods;
	}
	
	public byte[] removeTrapsAtRuntimeByName(Vector themethods) throws Exception {
		
		MethodFetcher methodAcc = new MethodFetcher(themethods);
		boolean result = methodAcc.process(clazz,clazz.getName());
		if(!result)
			return null;
		
		Vector methods = methodAcc.getResultingMethods();
		
		return removeTrapsAtRuntimeByConcreteMethods(methods);
	}
		
	public byte[] removeTrapsAtRuntimeByConcreteMethods(Vector themethods) throws Exception {
			
		TrapRemover remover=new TrapRemover();
		remover.setProcessMethods(themethods);
			
		remover.process(clazz,clazz.getName());
			
		return remover.getTargetByteCode();
	}
}