/*
  Copyright (C) 2002-2005 JAsCo Development Team
  http://ssel.vub.ac.be/jasco

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
 */
package jasco.runtime.transform;

import jasco.options.Options;
import jasco.util.generators.JavaGenerator;

import java.io.File;
import java.io.FileWriter;

import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtMethod;
import javassist.Modifier;

public class TrapInserter extends MethodProcessor  {
		
	
	public TrapInserter()  {
		super();
	}
	
	public TrapInserter(ClassPool p)  {
		super(p);
	}
	
	public TrapInserter(ClassProcessor g)  {
	
		super(g);
		
	}
	
	public boolean processMethod(CtMethod themethod, int methodid) throws Exception {
		
		boolean forCall = false;
		forCall=TransformerConstants.isForCall(themethod.getName());
		
		themethod.setBody(createNewBody(themethod,methodid));
		
		return true;
	}
	
	protected String createInitCode(CtMethod method, int methodid) throws Exception {
		
		CtMethod oldMethod = method;
		
		String jascoMethodField = TransformerConstants.getMethodField(methodid);
		String jascoTrapIdField = TransformerConstants.getTrapIDField(methodid);
		
		//String callMethodStringImpl = "(";
		CtClass[] params = oldMethod.getParameterTypes();
		StringBuffer argsArrayCreateCode= new StringBuffer("Object[] argsArray = new Object[");
		argsArrayCreateCode.append(params.length);
		argsArrayCreateCode.append("];");
		
		for(int j=0; j<params.length; ++j)  {
			String classRp = getSourceName(params[j]);
			//callMethodStringImpl+="var"+j;
			if (params[j].isPrimitive())   {
				argsArrayCreateCode.append("argsArray[");
				argsArrayCreateCode.append(j);
				argsArrayCreateCode.append("]=new ");
				argsArrayCreateCode.append(makePrimitiveObject(params[j]));
				argsArrayCreateCode.append("($");
				argsArrayCreateCode.append((j+1));
				argsArrayCreateCode.append(");");
			}
			else   {
				argsArrayCreateCode.append("argsArray[");
				argsArrayCreateCode.append((j));
				argsArrayCreateCode.append("]=$");
				argsArrayCreateCode.append((j+1));
				argsArrayCreateCode.append(";");
			}
		}
		argsArrayCreateCode.append(JavaGenerator.NEWLINE);
		StringBuffer codeBuffer=new StringBuffer();
		
		codeBuffer.append(argsArrayCreateCode.toString());
		codeBuffer.append(accessField(jascoMethodField));
		codeBuffer.append(".setArgumentsArray(argsArray);");
		codeBuffer.append(JavaGenerator.NEWLINE);
		String code = codeBuffer.toString();
			
		String thisInstance="$0";
		if(Modifier.isStatic(oldMethod.getModifiers()))
			thisInstance="jasco.runtime.NULL#instance";
		
		code+=accessField(jascoMethodField)+".setCalledObject("+thisInstance+");"+JavaGenerator.NEWLINE;
		
		return code;
	}
	

	
	protected String createNewBody(CtMethod method, int methodid) throws Exception {
		
		String origname = getNewMethodName(method);
		
		CtMethod oldMethod = method;
		
		String jascoMethodField = TransformerConstants.getMethodField(methodid);
		String jascoTrapIdField = TransformerConstants.getTrapIDField(methodid);
		
		//String callMethodStringImpl = "(";
		CtClass[] params = oldMethod.getParameterTypes();
			
		String thisInstance="$0";
		if(Modifier.isStatic(oldMethod.getModifiers()))
			thisInstance="jasco.runtime.NULL#instance";
		
		
		StringBuffer codeBuffer = new StringBuffer();
		codeBuffer.append("{if(jasco.runtime.ConnectorRegistry#isAdapted(");
		codeBuffer.append(accessField(jascoMethodField));
		codeBuffer.append(")) {");
		
		codeBuffer.append("try {");
		codeBuffer.append(JavaGenerator.NEWLINE);
		String code = codeBuffer.toString();
		
		code+= createInitCode(method,methodid);
		//code+="System.out.println(\"in trap: \"+Class.forName(\""+getTargetClass().getName()+"\").getClassLoader().toString());";
		 
		String startCode="";
		String returnCode=""; 
		if (oldMethod.getReturnType()!=CtClass.voidType)    {
			returnCode+="return ";
			
			if (oldMethod.getReturnType().isPrimitive())  {
				String type = makePrimitiveObject(oldMethod.getReturnType());
				startCode+=type+" res = "+"("+type+")";
				returnCode+="res."+oldMethod.getReturnType().getName()+"Value();";
			}
			else  {
				String type = oldMethod.getReturnType().getName();
				startCode+=type+" res = "+" ("+type+") ";
				returnCode+=" res;";
			}
		}
		else returnCode="return;"+JavaGenerator.NEWLINE;
		
		code+=startCode;
		code+="jasco.runtime.ConnectorRegistry#executeTrap("+accessField(jascoTrapIdField)+", "+thisInstance+","+accessField(jascoMethodField)+")"+";"+JavaGenerator.NEWLINE;	
		code+=returnCode;
		
		
		CtClass[] exceptions = method.getExceptionTypes();
		code += "} catch(java.lang.Exception b) {"+ JavaGenerator.NEWLINE;
		for(int i=0; i<exceptions.length; ++i)  {
			code+=JavaGenerator.TAB+"if(b instanceof "+exceptions[i].getName()+")";
			code+=" throw ("+exceptions[i].getName()+") b;"+JavaGenerator.NEWLINE;
		}
		code+=JavaGenerator.TAB+"if(b instanceof RuntimeException)";
		code+=" throw (RuntimeException) b;"+JavaGenerator.NEWLINE;
		
		String subCode2 ="";
		subCode2+= JavaGenerator.TAB+"jasco.options.Options#doDebugException(b);"+JavaGenerator.NEWLINE;
		code+=subCode2;
		
		code+="}}"+JavaGenerator.NEWLINE;
		
		String invokeCode = origname+"("+"$$"+");"+JavaGenerator.NEWLINE;
		
		if (oldMethod.getReturnType()!=CtClass.voidType)    
			invokeCode="return "+invokeCode;
		
		code+=invokeCode+"}";
		
		if (Options.showDebugOutput())  {
			FileWriter writer = new FileWriter(Options.getLogDir()+File.separator+"transform"+getFieldClass().getSimpleName()+methodid+".txt");
			writer.write(code);
			writer.close();
		}
		
		return code;
	}
}