/*
  Copyright (C) 2002-2005 JAsCo Development Team
  http://ssel.vub.ac.be/jasco

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
 */
package jasco.tools.connectorparser;


import jasco.tools.jascoparser.PJAsCoParseElement;
import jasco.tools.jascoparser.PJavaCode;
import jasco.util.logging.Logger;

import java.util.Iterator;
import java.util.Vector;

public class PCutpointDeclaration extends PJAsCoParseElement
{
	private String cutpointtype = null;
	private String aspecttype = null;
	private String aspectPackage = "";
	private String cutpointdeclarationname = null;
	private PCutpointDeclarationParameters cutpointdeclarationparameters = new PCutpointDeclarationParameters();
	private PJavaCode localjavacode = new PJavaCode();
	private String aspectFactory = null;
	private Vector abstractimplementations = new Vector();
	private String aspectFactoryArg=null;
	
	PCutpointDeclaration()
	{
	}
	
	public PCutpointDeclaration copy() {
		
		try {
			return (PCutpointDeclaration)clone();
		} catch (CloneNotSupportedException e) {
			Logger.getInstance().showError(e);
		}
		
		return null;
	}
	

//	!!!!!!!!!!!!!!!!!!!!new CODE!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
	public void addAbstractImplementation(PAbstractImplementation aAbstractImplementation) {
		abstractimplementations.add(aAbstractImplementation);
	}
	//!!!!!!!!!!!!!!!!!!!!new CODE!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//

	//!!!!!!!!!!!!!!!!!!!!new CODE!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
	public Iterator getAbstractImplementations() {
		return abstractimplementations.iterator();
	}
	//!!!!!!!!!!!!!!!!!!!!new CODE!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//

	
	public void setAspectFactory(String s)  {
	
		aspectFactory=s;
	}
	
	public String getAspectFactory()  {
	
		return aspectFactory;
	}
	
	public boolean hasAspectFactory() {
		
		return aspectFactory!=null;
	}
	
	public String getFullClassName()  {
		
		String res = aspecttype + "_" + cutpointtype;
		if(!getAspectPackage().equals(""))
			res=getAspectPackage()+"."+res;
		return res;
	}
	
	public String getFullAspectBeanClassName()  {
		
		String res = aspecttype;
		if(!getAspectPackage().equals(""))
			res=getAspectPackage()+"."+res;
		return res;
	}
	
	public String getClassName()  {
	
		return aspecttype + "_" + cutpointtype;
	}
	
	public void setAspectPackage(String aspectPackage)
	{
		this.aspectPackage = aspectPackage;
	}
	
	public String getAspectPackage()  {
	
		return aspectPackage;
	}
	
	public void setCutpointType(String aCutpointType)
	{
		cutpointtype = aCutpointType;
	}
	
	public String getCutpointType()
	{
		return cutpointtype;
	}
	
	public void setAspectType(String aAspectType)
	{
		aspecttype = aAspectType;
	}
	
	public String getAspectType()
	{
		return aspecttype;
	}
	
	public String getAspectClasssName()
	{
		return aspecttype;
	}
	
	public void setCutpointDeclarationName(String aCutpointDeclarationName)
	{	
		cutpointdeclarationname = aCutpointDeclarationName;
	}
	
	public String getCutpointDeclarationName()
	{
		return cutpointdeclarationname;
	}
	
	public void addCutpointDeclarationParameter(PCutpointDeclarationParameter aCutpointDeclarationParameter)
	{
		cutpointdeclarationparameters.addCutpointDeclarationParameter(aCutpointDeclarationParameter);
	}
	
	public PCutpointDeclarationParameters getCutpointDeclarationParameters() 
	{
		return cutpointdeclarationparameters;
	}
	/*
	public void addLocalJavaCode(String aJavaCode)
	{
		localjavacode.addJavaCode(aJavaCode);
	}
	
	public PJavaCode getLocalJavaCode()
	{
		return localjavacode;
	}*/


	/**
	 * @param fullType
	 */
	public void setFullName(String fullType) {
		
		int index = fullType.lastIndexOf("_");
		this.setCutpointType(fullType.substring(index+1));
		String aspecttype = fullType.substring(0,index);
		
		int dotIndex = aspecttype.lastIndexOf(".");
		if(dotIndex<0)
			this.setAspectType(aspecttype);
		else {
			this.setAspectType(aspecttype.substring(dotIndex+1));
			this.setAspectPackage(aspecttype.substring(0,dotIndex));
		}
		
	}

	public void setAspectFactoryArg(String aspectFactoryArg) {
		this.aspectFactoryArg=aspectFactoryArg;
	}

	public String getAspectFactoryInit() {
		if(!this.hasAspectFactory())
			return null;
		String args = "(";
		if(aspectFactoryArg!=null)
			args+="\""+aspectFactoryArg+"\"";
		args+=")";
		return aspectFactory+args;	
	}
}
