/*
  Copyright (C) 2002-2005 JAsCo Development Team
  http://ssel.vub.ac.be/jasco

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
 */
package jasco.tools.gui;

import jasco.options.Options;
import jasco.util.AbstractDialog;
import jasco.util.BrowseField;
import jasco.util.MyFileChooser;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

class CompileAspectDialog  extends AbstractDialog{
	
	private BrowseField output;
	private BrowseField aspect;
	private MyFileChooser fileChooser;
	public final static String NEWLINE = System.getProperty("line.separator");
	
	CompileAspectDialog(JFrame parent)  {
	
		super(parent,"Compile an aspect bean",640,480,true);
		fileChooser= new MyFileChooser(parent);
		fileChooser.selectDir(new File(System.getProperty("user.dir")));
		output=new BrowseField(fileChooser,true);
		output.selectFile(System.getProperty("user.dir"));
		aspect=new BrowseField(fileChooser,false);
		aspect.selectFile(System.getProperty("user.dir"));
		setupGUI();
		pack();
		//show();
	}
	
	protected void setupGUI()  {
	
		getContentPane().setLayout(new BorderLayout());
		final JTextArea text = new JTextArea();
		JPanel top = new JPanel();
		top.setLayout(new GridLayout(2,2));
		JLabel outputL = new JLabel("Select the output directory:");
		JLabel connectorL = new JLabel("Select the aspect bean source file:");
		//text.setPreferredSize(new Dimension(100,100));
		text.setRows(15);
		text.setColumns(50);
		aspect.addActionListener(new ActionListener()  {
			public void actionPerformed(ActionEvent ev)  {
				
				try {
				text.setText("");
				String f = aspect.getSelectedFile();
				BufferedReader buffer = new BufferedReader(new FileReader(f));
				
				String tmp = "";
				while ((tmp=buffer.readLine()) != null) 
					text.append(tmp+NEWLINE);
				
				buffer.close();
				}
				catch(Exception ex)  {
				}
			}
		});
					
		//output.selectFile(new File(jasco.options.Options.getOutputDir()).getAbsolutePath());
		top.add(connectorL);
		top.add(aspect);
		
		top.add(outputL);
		top.add(output);
		
		getContentPane().add(top,"North");
		text.setLineWrap(true);
		text.setEditable(false);
		getContentPane().add(new JScrollPane(text));
		
		JButton comp = new JButton("COMPILE");
		JButton cancel = new JButton("CANCEL");
		
		cancel.addActionListener(new ActionListener()  {
			public void actionPerformed(ActionEvent ev)  {
				
				cancelDialog();
			}
		});
		
		comp.addActionListener(new ActionListener()  {
			public void actionPerformed(ActionEvent ev)  {
				
				File f = new File(output.getSelectedFile());
				boolean b = true;
				b=b&&f.exists();
				f=new File(aspect.getSelectedFile());
				b=b&&f.exists();
				
				if (!b) {
					showErrorDialog("Selected Files are not valid!!");
					return;
				}
				approveDialog();
			}
		});
		
		JPanel boe = new JPanel();
		GridLayout g = new GridLayout(1,2,10,10);
		boe.setLayout(g);
		
		boe.add(cancel);
		boe.add(comp);
		
		getContentPane().add(boe,"South");
		
	}
	
	public void approveDialog()  {
		
		Options.setOutputDir(getOutputDir());
		try {
			jasco.Jasco.compileAspect(getAspectFile());
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		setVisible(false);
	}
	
	public String getAspectFile()  {
	
		return aspect.getSelectedFile();
	}
	
	public String getOutputDir()  {
	
		return output.getSelectedFile();
	}
	
	protected void dialogCanceled() {
		setVisible(false);
		
	}
}