package jasco.tools.jascoparser;

import jasco.util.generators.EnumGenerator;
import jasco.util.generators.MethodGenerator;
import jasco.util.generators.VariableGenerator;
import jasco.util.javacompiler.CompileError;
import jasco.util.logging.Logger;

import java.io.File;
import java.util.Iterator;
import java.util.Vector;

public abstract class JavaParser {
	
	private String filename;
	private int previousline;
	private JTokenManager tokenmanager;
	private Token currenttoken;
	private boolean isCheckingClasses =true;
	
	public JavaParser(JTokenManager tokenmanager, String aFileName) {
		
		previousline = 0; 
		this.tokenmanager = tokenmanager;
		filename = aFileName;
		currenttoken = tokenmanager.getNextToken();
	}
	
	protected abstract int tokenRPAREN();
	protected abstract int tokenANNOTATION();
	protected abstract int tokenLPAREN();
	protected abstract int tokenRBRACE();
	protected abstract int tokenLBRACE();
	protected abstract int tokenRBRACKET();
	protected abstract int tokenLBRACKET();
	protected abstract int tokenIMPLEMENTS();
	protected abstract int tokenEXTENDS();
	protected abstract int tokenCOMMA();
	protected abstract int tokenIDENTIFIER();
	protected abstract int tokenTHROWS();
	protected abstract int tokenSEMICOLON();
	protected abstract int tokenASSIGN();
	protected abstract int tokenCLASS();
	protected abstract int tokenGT();
	protected abstract int tokenLT();
	protected abstract int tokenDOT();
	protected abstract int tokenIMPORT();
	protected abstract int tokenEOF();
	protected abstract int tokenPACKAGE();
	protected abstract int tokenVARARGS();
	protected abstract int tokenSTATIC();
	protected abstract int tokenINTERFACE();
	protected abstract int tokenENUM();
	protected abstract int tokenATINTERFACE();
	protected abstract int tokenFINAL();
	
	protected abstract boolean isExistingClassName(String name);
	
	protected abstract int parseModifiers(StringBuffer annotations, PImports imports) throws JascoParseException;
	
	
	
	protected boolean isAbstractModifier(int modifiers) {
		
		return java.lang.reflect.Modifier.isAbstract(modifiers);
	}
	
	protected String parseGenerics() {
		
		String parsed = "";
		int numberof = 1;
		if(getCurrentId()!= tokenLT())
			return parsed;
		
		parsed+=getCurrentImage()+" ";
		getNextToken();
		
		while(true) {
			if(getCurrentId()== tokenLT())
				numberof++;
			else if(getCurrentId()== tokenGT())
				numberof--;
			parsed+=getCurrentImage()+" ";
			getNextToken();
			if(numberof==0)
				return parsed;
		}
	}
	
	
	protected String getFileName()  {
		
		return filename;
	}
	
	protected void checkFileName(String name) throws JascoParseException {
		
		File file = new File(getFileName());
		String toCheck=file.getName();
		
		
		int index = toCheck.indexOf(".");
		if(index!=-1)
			toCheck=toCheck.substring(0,index);
		
		if(!toCheck.equals(name))
			throw new JascoParseException(this.newError("Name ("+name+") different than file name ("+toCheck+")!"));
	}
	
	public void setIsCheckingClasses(boolean b) {
		
		isCheckingClasses=b;
	}
	
	public boolean isCheckingClasses() {
		
		return isCheckingClasses;
	}
	
	protected int getPreviousLine()  {
		
		return previousline;
	}
	
	protected JTokenManager getTokenManager()  {
		
		return tokenmanager;
	}
	
	protected int getLine()
	{
		return tokenmanager.getLine();
	}
	
	protected void getNextToken() throws JascoParseException
	{
		try {
			previousline = getLine();
			currenttoken = tokenmanager.getNextToken();
		} catch(Throwable t) {throw new JascoParseException(newError(t.getMessage()));}
	}
	
	protected int getCurrentId()
	{
		return currenttoken.kind;
	}
	
	protected String getCurrentImage()
	{
		return currenttoken.image;
	}
	
	protected void assertToken(int givenToken, int expectedToken, CompileError error) throws JascoParseException
	{
		if (givenToken != expectedToken) 
		{
			throw new JascoParseException(error);
		}
	}
	
	protected void compareNotToken(String givenToken, String expectedToken, CompileError error) throws JascoParseException
	{
		if (givenToken.equals(expectedToken)) 
		{
			throw new JascoParseException(error);
		}
	}
	
	protected void assertOrToken(int givenToken, int expectedToken1, int expectedToken2, CompileError error) throws JascoParseException
	{
		if (givenToken != expectedToken1 && givenToken != expectedToken2) 
		{
			throw new JascoParseException(error);
		}
	}
	
	protected void assertNotToken(int givenToken, int expectedToken, CompileError error) throws JascoParseException
	{
		if (givenToken == expectedToken) 
		{
			throw new JascoParseException(error);
		}
	}
	
	protected void checkSEMICOLON(String aErrorString) throws JascoParseException
	{
		String errorstring = " ';' expected" + "\n\t" + aErrorString + "\n\t" + generatePointer(1 + aErrorString.length());
		CompileError error = new CompileError(getFileName(), getPreviousLine(), errorstring);
		assertToken(getCurrentId(), tokenSEMICOLON() , error);
	}
	
	
	protected void checkIDENTIFIER(String aErrorString) throws JascoParseException
	{
		String errorstring = " <identifier> expected" + "\n\t" + aErrorString + "\n\t" + generatePointer(1 + aErrorString.length());
		CompileError error = new CompileError(getFileName(), getPreviousLine(), errorstring);
		
		assertToken(getCurrentId(), tokenIDENTIFIER() , error);
	}
	
	protected void checkLBRACE(String aErrorString) throws JascoParseException
	{
		String errorstring = " '{' expected" + "\n\t" + aErrorString + "\n\t" + generatePointer(1 + aErrorString.length());
		CompileError error = new CompileError(getFileName(), getPreviousLine(), errorstring);
		assertToken(getCurrentId(), tokenLBRACE() , error);
	}
	
	protected void checkGT(String aErrorString) throws JascoParseException
	{
		String errorstring = " '>' expected" + "\n\t" + aErrorString + "\n\t" + generatePointer(1 + aErrorString.length());
		CompileError error = new CompileError(getFileName(), getPreviousLine(), errorstring);
		assertToken(getCurrentId(), tokenGT() , error);
	}
	
	protected void checkRBRACKET(String aErrorString) throws JascoParseException
	{
		String errorstring = " ']' expected" + "\n\t" + aErrorString + "\n\t" + generatePointer(1 + aErrorString.length());
		CompileError error = new CompileError(getFileName(), getPreviousLine(), errorstring);
		assertToken(getCurrentId(), tokenRBRACKET(), error); 
	}
	
	
	protected CompileError newError(String desc)  {
		
		return new CompileError(getFileName(), getPreviousLine(), desc);
	}
	
	protected String generatePointer(int numberOfWhiteSpaces)
	{
		String result = "";
		for(int i = 0; i < numberOfWhiteSpaces; i++) 
		{
			result = result + " ";
		}
		return result + "^";
	}
	
	protected void parseMethod(PJavaClass parent, MethodGenerator method, PImports imports) throws JascoParseException {
		
		getNextToken();
		while (getCurrentId()!=tokenRPAREN())  {
			boolean finalVar = false;
			
			if(getCurrentId()==tokenFINAL()) {
				finalVar=true;
				getNextToken();
			}
			
			
			String vtype = accumulateType(imports);
			boolean varargs = false;
			if(getCurrentId()==tokenVARARGS()) {
				varargs=true;
				getNextToken();
			}
			
			checkIDENTIFIER(getCurrentImage());
			
			String vname = getCurrentImage(); 
			getNextToken();
			
			
			String typearrayaddendum=this.parseArrayAddendum();
			vtype+=typearrayaddendum;
			
			if(varargs)
				vtype+="...";
			method.addVariable(vtype,vname,finalVar);
			
			
			
			assertOrToken(getCurrentId(),tokenCOMMA(),tokenRPAREN(),newError(", or ) expected"));
			if(getCurrentId()==tokenCOMMA())
				getNextToken();
		}
		
		getNextToken();
		if (getCurrentId()==tokenTHROWS())  {
			getNextToken();
			
			while (true)  {
				String etype = accumulateType(imports);
				method.addException(etype);
				if(getCurrentId()!=tokenCOMMA())
					break;
				else getNextToken(); 
			} 
		}
		
		if (getCurrentId()==tokenSEMICOLON()&&(isAbstractModifier(method.getModifiers())||!parent.getType().equals(PJavaClass.Type.CLASS))) { //abstract method
			getNextToken();
		}
		else  {
			
			checkLBRACE(method.getName());
			getNextToken();
			PJavaCode code = parseMethodBody();
			
			method.setImplementation(code.toString());
		}
	}
	
protected void parseDeclarationPreamble(PJavaClass clz) throws JascoParseException {
		
		StringBuffer annotations = new StringBuffer();
		int modifiers = parseModifiers(annotations,clz.getImports());
		
		clz.setModifiers(modifiers);
		clz.setAnnotations(annotations.toString());
	}

	
	
	protected void parseField(PJavaClass parent, VariableGenerator field) throws JascoParseException {
		
		String typearrayaddendum = parseArrayAddendum();
		
		if(!typearrayaddendum.equals(""))
			field.setType(field.getType()+typearrayaddendum);
		
		if(getCurrentId()==tokenASSIGN()) {
			getNextToken();
			String code = parseFieldBody();
			
			if(!code.equals(""))
				field.setInitialiser(code.toString());
		}	
		
		if(getCurrentId()==tokenCOMMA()) {
			getNextToken();
			
			checkIDENTIFIER("");
			String name = getCurrentImage();
			VariableGenerator newField = field.copy(name);
			
			parent.addField(new PField(newField,getPreviousLine()));
			getNextToken();
			parseField(parent,field);
		}
		else if(getCurrentId()==tokenSEMICOLON())
			getNextToken();
		
	}
	
	private String parseArrayAddendum() {
		String typearrayaddendum="";
		while(getCurrentId()==tokenLBRACKET()) {
			getNextToken();
			assertToken(getCurrentId(),tokenRBRACKET(),newError("] expected"));
			typearrayaddendum+="[]";
			getNextToken();
		}	
		return typearrayaddendum;
	}

	protected void parseJavaCode(PJavaClass parent, PImports imports) throws JascoParseException {	
		StringBuffer annotations = new StringBuffer();
		int modifiers = parseModifiers(annotations, imports);
		
		parseJavaCode(parent,imports,annotations,modifiers);
	}
	
	protected void parseSuperDeclarations(PJavaClass target, PImports imports) throws JascoParseException {
		
		if(getCurrentId()==tokenEXTENDS()) {
			getNextToken();
			checkIDENTIFIER("super class name expected for"+target.getName());
			String type = accumulateType(imports);
			target.setSuperClass(type);
		}
		if(getCurrentId()==tokenIMPLEMENTS()) {
			getNextToken();
			checkIDENTIFIER("super interface name expected for"+target.getName());
			String type = accumulateType(imports);
			target.addInterface(type);
			while(getCurrentId()==tokenCOMMA()) {
				getNextToken();
				checkIDENTIFIER("super interface name expected for"+target.getName());
				String itype = accumulateType(imports);
				target.addInterface(itype);
			}
		}
	}
	
	protected void parseClass(PJavaClass parent, PImports imports, StringBuffer annotations, int modifiers, PJavaClass.Type type) throws JascoParseException {
		
		checkIDENTIFIER("class name expected");
		
		String classname = getCurrentImage();
		PJavaClass innerClass = new PJavaClass(classname);
		innerClass.setType(type);
		innerClass.setModifiers(modifiers);
		innerClass.setAnnotations(annotations.toString());
		getNextToken();
		
		String generics = parseGenerics();
		innerClass.setGenerics(generics);
		
		parseSuperDeclarations(innerClass,imports);
		
		checkLBRACE("opening brace expected for class");
		getNextToken();
		
		while(getCurrentId()!=tokenRBRACE()) {
			parseJavaCode(innerClass,imports);
		}
		
		parent.addInnerClass(innerClass);
		
		getNextToken();
		
	}
	
	protected void parseJavaCode(PJavaClass parent, PImports imports, StringBuffer annotations, int modifiers) throws JascoParseException {	
		
		if(getCurrentId()==tokenCLASS()) {
			
			getNextToken();
			parseClass(parent,imports,annotations,modifiers,PJavaClass.Type.CLASS);
			return;
		}
		else if(getCurrentId()==tokenINTERFACE()) {
			
			getNextToken();
			parseClass(parent,imports,annotations,modifiers,PJavaClass.Type.INTERFACE);
			return;
		}
		else if(getCurrentId()==tokenATINTERFACE()) {
			
			getNextToken();
			parseClass(parent,imports,annotations,modifiers,PJavaClass.Type.ANNOTATION);
			return;
		}
		else if(getCurrentId()==tokenENUM()) {
			
			getNextToken();
			parseEnum(parent,imports,annotations,modifiers);
			return;
		}
		else if (getCurrentId()==tokenLBRACE())   { //static init
			getNextToken();
			PJavaCode code = parseMethodBody();
			parent.addStaticInitializer(code);
			return; 
		}
		else if (getCurrentId()==tokenSEMICOLON())   { //simple semi COLON
			getNextToken();
			return; 
		}
		
		String generics = parseGenerics();
		String type= accumulateType(imports);
		String name = getCurrentImage();
		
		String simpleType = getSimpleName(type);
	
		if(simpleType.equals(parent.getName())&&getCurrentId()==tokenLPAREN()) {//constructor
			name=simpleType;
			type="";
		}
		else {
			checkIDENTIFIER(getCurrentImage());
			getNextToken();
		}
		
		
		
		if(getCurrentId()==tokenLPAREN())  { //method
			MethodGenerator method = new MethodGenerator(name,type);
			method.addAnnotation(annotations.toString());
			PMethod pm = new PMethod(method,getPreviousLine());
			method.setModifiers(modifiers);
			method.setGenerics(generics); 
			pm.setLine(getPreviousLine());
			parent.addMethod(pm);
			parseMethod(parent,method,imports);
		}
		else if(type.equals(""))
			throw new JascoParseException(newError("( expected for constructor"));
		else  { 
			VariableGenerator field = new VariableGenerator(type,name);
			PField pf = new PField(field,getPreviousLine());
			field.addAnnotation(annotations.toString());
			field.setModifiers(modifiers);
			pf.setLine(getPreviousLine());
			parent.addField(pf);
			parseField(parent,field);
		}
	}
	
	private String getSimpleName(String type) {
		int index = type.lastIndexOf(".");
		if(index<0)
			return type;
		
		else return type.substring(index+1);
	}

	private void parseEnum(PJavaClass parent, PImports imports, StringBuffer annotations, int modifiers) {
		
		checkIDENTIFIER("enum name expected");
		
		String classname = getCurrentImage();
		EnumGenerator java = new EnumGenerator(classname);
		java.setModifiers(modifiers);
		java.addAnnotation(annotations.toString());
		
		getNextToken();
		
		String values = "";
		
		this.checkLBRACE("");
		getNextToken();
		while(getCurrentId()!=tokenRBRACE()) {
			values+=getCurrentImage()+" ";
			getNextToken();
		}
		getNextToken();
		
		java.setValues(values);
		
		PEnum innerClass = new PEnum(java,getPreviousLine());
		parent.addEnum(innerClass);  
	}

	protected String accumulateType(PImports imports)  throws JascoParseException  {
		
		return accumulateType("",imports,TYPE_CLASS);
	}
	
	protected String accumulateType(String type, PImports imports)  throws JascoParseException  {
		
		return accumulateType(type,imports,TYPE_CLASS);
	}
	
	protected String accumulateType(PImports imports,int typeToParse)  throws JascoParseException  {
		
		return accumulateType("",imports,typeToParse);
	}
	
	public static final int TYPE_CLASS = 0;
	public static final int TYPE_METHOD = 1;
	public static final int TYPE_ASPECT = 2;
	
	/**
	 * 
	 * @param parsedType
	 * @param imports
	 * @param typeToParse is either CLASS, METHOD or ASPECT
	 * @return
	 * @throws JascoParseException
	 */
	protected String accumulateType(String parsedType, PImports imports, int typeToParse)  throws JascoParseException
	{	
		
		if (parsedType.equals(""))   { //start at first token
			parsedType += getCurrentImage();
			getNextToken();
		}
		else { 
			if (getCurrentId()==tokenDOT())  {
				parsedType += getCurrentImage();
				getNextToken();
				parsedType += getCurrentImage();
				getNextToken();
			}
			
			else return parsedType;
		} 
		
		boolean checked = false;
		
		while (true)
		{	
			
			if (getCurrentId()==tokenLT())  { //generics
				
				parsedType=checkAnyType(parsedType,imports,typeToParse);
				checked=true;
				parsedType+=getCurrentImage();
				getNextToken();
				
				while (true)  {
					parsedType+=accumulateType(imports);
					
					if (getCurrentId()==tokenCOMMA())  {
						parsedType+=getCurrentImage();
						getNextToken();
					}
					else  {
						checkGT(parsedType);
						parsedType+=getCurrentImage();
						getNextToken();
						break;
					}
				}			
			}
			
			if(getCurrentId()==tokenDOT()) {
				parsedType+=getCurrentImage();
				
				getNextToken();
				
				parsedType+=getCurrentImage();
				
				getNextToken();
			}
			else	break;
		}
		
		if(!checked) {
			parsedType=checkAnyType(parsedType,imports,typeToParse);
		}
		
		while(getCurrentId()==tokenLBRACKET()) { //arrays
			parsedType+=getCurrentImage();
			getNextToken();
			checkRBRACKET(parsedType);
			parsedType+=getCurrentImage();
			getNextToken();
		}
		
		return parsedType;
	}
	
	
	
	protected String checkAnyType(String parsedType, PImports imports, int typeToParse) throws JascoParseException {
		
		switch(typeToParse) {
		
		case TYPE_METHOD:
			int i = parsedType.lastIndexOf(".");
			if(i!=-1) {
				String type = parsedType.substring(0,i);
				parsedType=checkClass(type,imports)+parsedType.substring(i);
			}
			break; 
			
		case TYPE_CLASS:
			parsedType=checkClass(parsedType,imports);
			break;
			
		default:
			throw new IllegalArgumentException("illegal parsed type");
		}
		
		return parsedType;
	}
	
	public String accumulateMethod(PImports imports) throws JascoParseException {
		
		return accumulateType(imports,TYPE_METHOD);
	}
	
	protected String checkClass(String name, PImports imports) throws JascoParseException {
		
		String orig=name;
		CompileError classNotFound = new CompileError(getFileName(), getPreviousLine(), "Class not found error: "+name);
		if(isPrimitive(name))
			return name;
		
		
		name = getFullClassName(name,imports);
		if(name!=null)
			return name;
		
		int index = orig.lastIndexOf(".");
		if (index!=-1)  {
			String newname = orig.substring(0,index)+"_"+orig.substring(index+1);
			if(!isClass(newname,imports))
				throw new JascoParseException(classNotFound);
			else return newname;
		}
		else throw new JascoParseException(classNotFound);
	}
	
	protected boolean isClass(String className, PImports imports)  {
		
		String name = getFullClassName(className,imports);
		if(name==null) return false;
		else {
			return isExistingClassName(name);
		}
	}
	
	protected void addSyntheticImports(PImports imports, String classname, String packagename) {
		
		if(packagename!=null)
			imports.addImport(new PImport(packagename+".*",PImport.SYNTHETIC));
	}
	
	protected void addSyntheticImports(PJavaClass clz) {
			
		  String pack = null;
		  if(clz.hasPackage())
			  pack=clz.getPackage().getPackageName();
		  addSyntheticImports(clz.getImports(),clz.getFullName(),pack);
	}
	
	protected PImports parseImportDeclaration() throws JascoParseException
	{
		
		PImports pimports = new PImports();
		while (getCurrentId() == tokenIMPORT())
		{	
			boolean staticc = false;
			getNextToken();
			if(getCurrentId() == tokenSTATIC()) {
				staticc=true;
				getNextToken();
			}
			String importname = getCurrentImage();
			getNextToken();
			
			String errorstring;
			
			while (getCurrentId() == tokenDOT())
			{
				getNextToken(); 
				importname = importname + "." + getCurrentImage();	
				getNextToken();
			}
			
			pimports.addImport(new PImport(importname, getLine(), staticc));
			
			errorstring = "import " + importname;
			checkSEMICOLON(errorstring); 
			
			getNextToken();		
		}
		
		return pimports;
	}
	
	public int setPublic(int modifiers) {
		
		modifiers|=java.lang.reflect.Modifier.PUBLIC;
		
		return modifiers;
	}
	
		
	
	protected void parseAnnotation(StringBuffer annotations, PImports imports) throws JascoParseException {
		
		assertToken(getCurrentId(),tokenANNOTATION(),newError("annotation (@) token expected \n"+annotations.toString()));
		annotations.append(getCurrentImage());
		
		getNextToken();
		checkIDENTIFIER(annotations.toString());
		
		annotations.append(accumulateType(imports));
		
		if(getCurrentId()!=tokenLPAREN()) {
			annotations.append(" ");
			return;
		}
		
		annotations.append(getCurrentImage());
		getNextToken();
		
		while(true) {
			annotations.append(getCurrentImage());
			if(getCurrentId()==tokenRPAREN())
				break;
			else if(getCurrentId()==tokenLBRACE()) {
				getNextToken();
				parseAnnotation(annotations,imports);
				while(getCurrentId()==tokenCOMMA()) {
					annotations.append(getCurrentImage());
					getNextToken();
					parseAnnotation(annotations,imports);
				}
			}
			else getNextToken();
		}
		
		getNextToken();
	}
	
	protected String removeArrayStuff(String clz) {
		
		int i = clz.indexOf("[");
		if(i!=-1)
			return clz.substring(0,i);
		else return clz;
	}
	
	protected String addArrayStuff(String clz, String orig) {
		
		int i = orig.indexOf("[");
		if(i!=-1) {
			String n = orig.substring(i);
			return clz+n;
		}
		else return clz;
		
	}
	
	private String[] primitives = new String[]  {"int","float","double","long","short","byte","char","boolean","void"};
	protected boolean isPrimitive(String aClass) 
	{
		for(int i=0;i<primitives.length;++i)
			if(aClass.equals(primitives[i]))
				return true;
		
		return false;	
		
	}
	
	protected String getFullClassName(String origClassName, PImports imports)  {
		
		if(isGenerics(origClassName))
			return origClassName;
		
		String className=removeArrayStuff(origClassName);
		
		if(isExistingClassName(className))
			return addArrayStuff(className,origClassName);	
		
		Vector packages = new Vector();
		Iterator i  = imports.getImports();
		while (i.hasNext())  {
			PImport next = (PImport) i.next();
			String pack =  next.getImportName();
			if(pack.endsWith("*"))
				packages.add(pack.substring(0,pack.length()-1));
			else if (pack.endsWith("."+className)||pack.equals(className)) 	{
				if(isExistingClassName(pack))
					return addArrayStuff(pack,origClassName);
			}
		}
		
		packages.add("java.lang.");
		
		String result=null;
		
		i = packages.iterator();
		while (i.hasNext())  {
			String next = i.next().toString();
			String tmp = next+className;
				if(isExistingClassName(tmp)) {
					if(result!=null&&!result.equals(tmp)) 
						Logger.getInstance().showError("Warning: Ambigious class reference: "+className);
					else result=tmp;
				}
		}
		
		if(result==null&&this.isCheckingClasses())
			return null;
		else if(result==null&&!this.isCheckingClasses())
			return origClassName;
		else return addArrayStuff(result,origClassName);
	}
	
	protected String parseFieldBody() throws JascoParseException {
		
		int cachedlinenumber = getLine();
		String temp = "";
		int numberofleftbraces = 0;
		int numberofleftparent = 0;
		
		String errorstring = "unexpected end of file";
		CompileError error = new CompileError(getFileName(), getPreviousLine(), errorstring);
		
		while (true)
		{
			assertNotToken(getCurrentId(), tokenEOF(), error);
			
			if(getCurrentId()==tokenDOT()||getCurrentId()==tokenCOMMA()||getCurrentImage().startsWith(".")) { //fe numbers: 0.9 (0 .9  is invalid...)
				
				if(temp.length()>0&&temp.charAt(temp.length()-1)==' ') {
					temp=temp.substring(0,temp.length()-1);
				}
			}
			
			if (getCurrentId() == tokenLBRACE()) 
			{
				numberofleftbraces++;
			}
			else if (getCurrentId() == tokenRBRACE()) 
			{	
				numberofleftbraces--;
			}
			else if (getCurrentId() == tokenLPAREN()) 
			{
				numberofleftparent++;
			}
			else if (getCurrentId() == tokenRPAREN()) 
			{	
				numberofleftparent--;
			}
			else if(getCurrentId()==tokenCOMMA()&&numberofleftparent==0&&numberofleftbraces==0)
				break;
			else if(getCurrentId()==tokenSEMICOLON()&&numberofleftparent==0&&numberofleftbraces==0)
				break;
			
			temp = temp + getCurrentImage() + " ";
			
			
			getNextToken();
		}
		
		return temp;
	}
	
	
	protected PJavaCode parseMethodBody() throws JascoParseException {
		
		PJavaCode code = new PJavaCode();
		int cachedlinenumber = getLine();
		String temp = "";
		int numberofleftbraces = 1;
		
		while (numberofleftbraces!=0)
		{
			String errorstring = "unexpected end of file";
			CompileError error = new CompileError(getFileName(), getPreviousLine(), errorstring);
			assertNotToken(getCurrentId(), tokenEOF(), error);
			
			if(getCurrentId()==tokenDOT()||getCurrentId()==tokenCOMMA()||getCurrentImage().startsWith(".")) { //fe numbers: 0.9 (0 .9  is invalid...)
				
				if(temp.length()>0&&temp.charAt(temp.length()-1)==' ') {
					temp=temp.substring(0,temp.length()-1);
				}
			}
			
			if (getCurrentId() == tokenLBRACE()) 
			{
				numberofleftbraces++;
			}
			else if (getCurrentId() == tokenRBRACE()) 
			{	
				numberofleftbraces--;
			}
			
			if(numberofleftbraces!=0) 
				temp = temp + getCurrentImage() + " ";
			
			getNextToken();
			
			if(cachedlinenumber != getLine())
			{
				code.addJavaCode(temp, cachedlinenumber);
				cachedlinenumber = getLine();
				temp = "";
			}
		}
		return code;
	}
	
	protected boolean isArray(String s) {
		
		return s.indexOf("[")!=-1;
	}
	
	protected boolean isGenerics(String s) {
		
		return s.indexOf("<")!=-1;
	}
	protected void parsePackageDeclaration(PJavaClass target) throws JascoParseException {
		if(getCurrentId() == tokenPACKAGE())
		{
			getNextToken();
			String packagename = getCurrentImage();
			getNextToken();
			
			String errorstring;
			
			while (getCurrentId() == tokenDOT()) 
			{
				getNextToken();
				errorstring = "package " + packagename + ".";
				checkIDENTIFIER(errorstring);  
				packagename = packagename + "." + getCurrentImage();
				getNextToken();
			}
			
			target.setPackage(new PPackage(packagename, getLine()));
			
			errorstring = "package " + packagename;
			checkSEMICOLON(errorstring); 
			
			getNextToken();
		}
	}
	
	
}
