/*
 * Created on 7-mrt-2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package jasco.tools.refinements;

import jasco.options.Options;
import jasco.runtime.aspect.HookHelper;
import jasco.runtime.refinements.RefinementManager;
import jasco.tools.aspectparser.AspectClassesGenerator;
import jasco.tools.aspectparser.ClassFileEditor;
import jasco.tools.aspectparser.MixinCastImplementer;
import jasco.tools.jascoparser.JascoParseException;
import jasco.tools.jascoparser.PImport;
import jasco.tools.jascoparser.PImports;
import jasco.tools.jascoparser.PMethod;
import jasco.util.generators.ClassGenerator;
import jasco.util.generators.GImport;
import jasco.util.generators.JavaGenerator;
import jasco.util.generators.MethodGenerator;
import jasco.util.generators.VariableGenerator;
import jasco.util.javacompiler.CompileError;
import jasco.util.javacompiler.JavaCompiler;
import jasco.util.logging.Logger;

import java.io.File;
import java.io.FileWriter;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Iterator;
import java.util.Vector;

/* 
 Copyright (C) 2002-2005 JAsCo Development Team
 http://ssel.vub.ac.be/jasco
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as
 published by the Free Software Foundation; either version 2 of the
 License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 USA 
 */

/**
 * @author Wim
 */

public class RefinementClassesGenerator {
	
	private PRefinement refinement;
	private Vector lastErrors;
	
	/**
	 * 
	 */
	public RefinementClassesGenerator(PRefinement refinement) {
		super();
		this.refinement=refinement;
	}
	
	public void generate()  {
		
			try {
				generateImpl();
				
			}
			catch(JascoParseException ex)  {
				Logger.getInstance().showError(ex.getError());
			}
		}
	
	private boolean translateAndPrint(Vector aErrors)
	{
		String source = refinement.getSourceName();
		
		for (int i = 0 ; i < aErrors.size() ; i++) 
		{
			//Get error and errorstring;
			CompileError error = (CompileError)aErrors.elementAt(i);
			String errorstring = error.getErrorString();
			
		
			
			//Get linenumber of error in original file
			int documentationindex = errorstring.lastIndexOf("//");
		
			if (documentationindex==-1)  {
				//System.out.println("!!!!!!!! JAsCo System Error: !!!!!!!!");
				//System.out.println(error.toString());
				Logger.getInstance().showError(error);
					
				continue;
			}
			//System.out.println("OK"+errorstring);
			int restindex = errorstring.indexOf(Options.NEWLINE,documentationindex);
			String linenumber = errorstring.substring(documentationindex + 2,restindex);
			
			int reallinenumber = (new Integer(linenumber)).intValue();
			
			
			//Get errormessage by extracting documentation
			String realerror = errorstring.substring(0,documentationindex) + "\n" + errorstring.substring(restindex + 1); 
			//System.out.println("OK"+errorstring+reallinenumber);
			CompileError newerror = new CompileError(source,reallinenumber,realerror);
				
			//System.out.print("FULL"+newerror.toString());
			Logger.getInstance().showError(newerror);
		}
		if (aErrors.size() != 0)
		{
			if (aErrors.size() == 1)
			{
				//System.out.println(1 + " error");
				Logger.getInstance().showOutput(1 + " error");
			}
			else 
			{
				//System.out.println(aErrors.size() + " errors");
				Logger.getInstance().showOutput(aErrors.size() + " errors");
			}
			
			return false;
		}
		else return true;
	}
	
	private void generateMixin(ClassGenerator clz) {
		MethodGenerator method = new MethodGenerator("mixinOf","Object");
		method.addVariable("Object","target");
		method.addVariable("Class","mixin");
		method.addModifier(Modifier.PROTECTED);
		
		method.setImplementation("return jasco.runtime.mixin.MixinManager.getInstance().mixinOf(target,mixin);");
		method.addException("jasco.runtime.mixin.NoMixinFound");
		
		clz.addMethod(method);
	}
		
	protected void generateImpl() {
			
		String code = generateRefinement();
			
			
		try 
		{	
		//retrieve the source and classpath
		String classpath = Options.getClassPath();
		String packageDir="";
		packageDir=refinement.getPackage().getPackageName().replace('.',File.separatorChar) +Options.FILE_SEPARATOR;
		
		//String source = (new File(Options.getOutputDir())).getCanonicalPath() + Options.FILE_SEPARATOR + packageDir+"*.java";
		String source = (new File(Options.getOutputDir())).getCanonicalPath() + Options.FILE_SEPARATOR + packageDir+refinement.getName()+".java";
		File f = new File(source);
		f.getParentFile().mkdirs();
		FileWriter writer = new FileWriter(f);
		writer.write(code);
		writer.close();
		
		//compile all files
		JavaCompiler compile = new JavaCompiler();
			
		compile.compile(source,classpath);
		Vector errors = compile.getErrors();
		lastErrors=errors;
		if(Options.deleteTempFiles())
			new File(source).delete();
			
		//translate the errors to the aspect source instead of the class source
		boolean res = translateAndPrint(errors);
		if(res) {
			
			ClassFileEditor editor = new ClassFileEditor(Options.getOutputDir(),refinement.getFullName());
			MixinCastImplementer impl = new MixinCastImplementer(editor);
			impl.process(); 
			editor.writeFile();
		}
		
		
		}
		catch(Exception e)
			{
			Logger.getInstance().showFullError(e);
		}
	}
	
	public Vector getLastErrors() {
		
		return lastErrors;
	}

	/**
	 * 
	 */
	private String generateRefinement() {
		
		Vector methods = checkRefinableMethods();
		addThisHookAndJP();
		
		ClassGenerator gen = refinement.getClassGenerator();
		gen.addInterface("jasco.runtime.refinements.IRefinement");
		
		if(!refinement.hasSuperClass())
			generateMixin(gen);
		
		generateDefaultVar(gen);
		generateIsApplicable(gen);
		generateConstructor(gen,methods);
		generateGetPrio(gen);
		generateImports(gen);
		//generateCompareTo(gen);
		
		return gen.toString();
	}

	/**
	 * @param gen
	 */
	private void generateImports(ClassGenerator gen) {
		PImports imports = refinement.getImports();
		Iterator i = imports.getImports();
		while(i.hasNext()) {
			PImport importx = (PImport) i.next();
			 if(importx.isSynthetic())
			    	 continue;
			gen.addImportPackage(new GImport(importx.getImportName(),importx.getLineNumber()+""));
		}
	}

	/**
	 * @param gen
	 */
	private void generateCompareTo(ClassGenerator gen) {
		MethodGenerator method = new MethodGenerator("compareTo","int");
		method.addModifier(Modifier.PUBLIC);
		method.addVariable("java.lang.Object","o");
		
		method.setImplementation("return impl.compareTo(o);");
		gen.addMethod(method);
	}

	/**
	 * 
	 */
	private void addThisHookAndJP() {
		
		Iterator i = refinement.getMethods();
		while(i.hasNext()) {
			PMethod method = (PMethod) i.next();
			if(!method.isRefinable()) 
				continue;
			
			String tjpname = AspectClassesGenerator.CONST_thisJoinPoint;
			
			method.getMethodInfo().insertVariable("jasco.runtime.MethodJoinpoint", AspectClassesGenerator.CONST_thisJoinPoint);
			method.getMethodInfo().insertVariable("jasco.runtime.aspect.IHook", "_Jasco_thisHookUncasted");
			
			
			String impl = method.getMethodInfo().getImplementation();
			impl=refinement.getRefinementTargetClass()+" "+AspectClassesGenerator.CONST_thisJoinPointObject+" = ("+refinement.getRefinementTargetClass()+") "+tjpname+".getCalledObject();"+JavaGenerator.NEWLINE+impl;
			impl=refinement.getRefinementTargetHook()+" "+RefinementManager.THIS_HOOK +" = ("+refinement.getRefinementTargetHook()+") _Jasco_thisHookUncasted;"+JavaGenerator.NEWLINE+impl;
			 method.getMethodInfo().setImplementation(impl);
		}
		
	}
	
	protected void newParseException(String message, int lineo) {
		
		throw new JascoParseException(new CompileError(refinement.getSourceName(),lineo,message));
	}
	
	protected Class getTargetHookClass() {
		
		try {
		return Options.loadClass(refinement.getRefinementTargetHook());
		}catch(ClassNotFoundException ex) {
			newParseException("class not found "+ex.getMessage(),refinement.getLine());
		}
		
		return null;
	}
	
	
	private Vector checkRefinableMethods() {
		
		Class clz = getTargetHookClass();
		Vector methods = new Vector();
	
		Iterator i = refinement.getMethods();
		while(i.hasNext()) {
			PMethod method = (PMethod) i.next();
			if(!method.isRefinable()) 
				continue;
			
			String name = method.getMethodInfo().getName();
			if(!HookHelper.isRefinableMethod(clz,method.getMethodInfo()))
				this.newParseException("refinable method "+name+" not declared in target hook",method.getLine());
			
			if(methodImplementedInSuper(name))
				this.newParseException("refinable method already implemented in super class, refinable method overriding not yet supported!",method.getLine());
			
			methods.add(name);
		}
		
		return methods;
		
	}

	/**
	 * @param gen
	 */
	private void generateDefaultVar(ClassGenerator gen) {
		VariableGenerator var = new VariableGenerator("jasco.runtime.refinements.IRefinement","impl");
		var.addModifier(Modifier.PRIVATE);
		
		gen.addVariable(var);
		 
	}

	/**
	 * @param gen
	 */
	private void generateIsApplicable(ClassGenerator gen) {
		
		MethodGenerator method = new MethodGenerator("isApplicable","boolean");
		method.addModifier(Modifier.PUBLIC);
		method.addVariable("java.lang.Class","clz");
		method.addVariable("jasco.runtime.aspect.IHook","cp");
		method.addVariable("String","methodname");
		
		String code = "";
		code+="boolean res = impl.isApplicable(clz,cp,methodname);"+JavaGenerator.NEWLINE;
		
		if(superIsRefinement())
			code+="res=res||super.isApplicable(clz,cp,methodname);"+JavaGenerator.NEWLINE;
		
		code+="return res;";
		method.setImplementation(code);
		
		gen.addMethod(method);
	}
	
	/**
	 * @return
	 */
	private boolean superIsRefinement() {
		if(!refinement.hasSuperClass())
			return false;
		
		try {
		Class superClass =Options.loadClass(refinement.getSuperClass());
		
		return RefinementManager.isRefinement(superClass);
		} catch(ClassNotFoundException ex) {
			
			newParseException("class not found "+ex.getMessage(), refinement.getLine());
		}
		
		return false;
			
	}
	
	/**
	 * @return
	 */
	private boolean methodImplementedInSuper(String name) {
		if(!refinement.hasSuperClass())
			return false;
		
		try {
		Class superClass =Options.loadClass(refinement.getSuperClass());
		
		Method[] methods = superClass.getMethods();
		for(int i=0;i<methods.length;++i)
			if(methods[i].getName().equals(name))
				return true;
		
		} catch(ClassNotFoundException ex) {
			
			newParseException("class not found "+ex.getMessage(), refinement.getLine());
		}
		
		return false;
			
	}

	/**
	 * @param gen
	 */
	private void generateGetPrio(ClassGenerator gen) {
		
		MethodGenerator method = new MethodGenerator("getPriority","int");
		method.addModifier(Modifier.PUBLIC);
		
		
		String code = "";
		code+="return impl.getPriority();"+JavaGenerator.NEWLINE;
		
		method.setImplementation(code);
		
		gen.addMethod(method);
	}
	
	/**
	 * @param gen
	 */
	private void generateConstructor(ClassGenerator gen, Vector methods) {
		
		MethodGenerator method = new MethodGenerator(gen.getName(),"");
		method.addModifier(Modifier.PUBLIC);
		
		String code = "";
		code+="java.util.Vector v = new java.util.Vector();"+JavaGenerator.NEWLINE;
		Iterator i = methods.iterator();
		while(i.hasNext()) 
			code+="v.add(\""+(String)i.next()+"\");"+JavaGenerator.NEWLINE;
		
		code+="this.impl = new jasco.runtime.refinements.DefaultRefinement(";
		code+="\""+refinement.getRefinementTargetClass()+"\", ";
		code+="\""+refinement.getRefinementTargetHook()+"\", v);"+JavaGenerator.NEWLINE;
		
		method.setImplementation(code);
		
		gen.addMethod(method);
		
	}
	
	
}
