/*
  Copyright (C) 2002-2005 JAsCo Development Team
  http://ssel.vub.ac.be/jasco

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
 */
/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package jasco.util;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JOptionPane;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

public abstract class AbstractFrame extends JFrame {
  protected final int CANCELED = JOptionPane.CANCEL_OPTION;
  protected final int CONFIRMED = JOptionPane.YES_OPTION;
  protected final int DENIED = JOptionPane.NO_OPTION;
  
  public AbstractFrame(String title, int width, int height) {
    this(title, width, height, true);
  }

  public AbstractFrame(String title, int width, int height, boolean resizable) {
    super(title);
    this.setSize(width, height);
    this.setResizable(resizable);
    this.centerOnScreen();
    this.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        quit();
      } 

    });
  }

  public final void centerOnScreen() {
    Dimension frameSize = this.getSize();
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    this.setLocation(screenSize.width / 2 - frameSize.width / 2, screenSize.height / 2 - frameSize.height / 2);
  } 

  protected final JMenu createLookAndFeelMenu() {
    JMenu windowMenu = new JMenu("Window");
    windowMenu.setMnemonic(KeyEvent.VK_W);
    JRadioButtonMenuItem windowsLook = new JRadioButtonMenuItem("Windows Look and Feel");
    JRadioButtonMenuItem motifLook = new JRadioButtonMenuItem("Motif Look and Feel");
    JRadioButtonMenuItem metalLook = new JRadioButtonMenuItem("Metal Look and Feel", true);
    JRadioButtonMenuItem macLook = new JRadioButtonMenuItem("Macintosh Look and Feel");
    JRadioButtonMenuItem organicLook = new JRadioButtonMenuItem("Organic Look and Feel");
    windowsLook.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
        showWarningDialog("This look And feel is quite buggy");
      } 
    });
    windowsLook.setMnemonic(KeyEvent.VK_W);
    windowsLook.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, KeyEvent.CTRL_MASK));
    motifLook.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
      } 
    });
    motifLook.setMnemonic(KeyEvent.VK_O);
    motifLook.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F2, KeyEvent.CTRL_MASK));
    metalLook.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
      } 
    });
    metalLook.setMnemonic(KeyEvent.VK_M);
    metalLook.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F3, KeyEvent.CTRL_MASK));
    macLook.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        // setLookAndFeel("com\\sun\\java\\swing\\plaf\\mac\\MacLookAndFeel");
        showErrorDialog("This look And feel is not available");
      } 

    });
    macLook.setMnemonic(KeyEvent.VK_W);
    macLook.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4, KeyEvent.CTRL_MASK));
    organicLook.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        // setLookAndFeel("com.sun.java.swing.plaf.organic.OrganicLookAndFeel");
        showErrorDialog("This look And feel is not available");
      } 
    });
    organicLook.setMnemonic(KeyEvent.VK_G);
    organicLook.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F5, KeyEvent.CTRL_MASK));
    windowMenu.add(windowsLook);
    windowMenu.add(motifLook);
    windowMenu.add(metalLook);
    windowMenu.add(macLook);
    windowMenu.add(organicLook);
    ButtonGroup buttonGroup = new ButtonGroup();
    buttonGroup.add(windowsLook);
    buttonGroup.add(motifLook);
    buttonGroup.add(metalLook);
    buttonGroup.add(macLook);
    buttonGroup.add(organicLook);
    return windowMenu;
  } 

  public void quit() {
    System.exit(0);
  } 

  protected final void setLookAndFeel(String lookAndFeel) {
    try {
      UIManager.setLookAndFeel(lookAndFeel);
      SwingUtilities.updateComponentTreeUI(this);
    } catch (UnsupportedLookAndFeelException e) {
      System.out.println("illegal lookAndFeel" + lookAndFeel);
    } catch (Exception e) {
      System.out.println(e);
    } 
  } 

  public final void showErrorDialog(String error) {
    this.showErrorDialog(error, "ERROR");
  } 

  public final void showErrorDialog(String error, String title) {
    JOptionPane.showMessageDialog(this, error, title, JOptionPane.ERROR_MESSAGE);
  } 

  protected final void showInformationDialog(String error) {
    this.showInformationDialog(error, "Info");
  } 

  protected final void showInformationDialog(String error, String title) {
    JOptionPane.showMessageDialog(this, error, title, JOptionPane.INFORMATION_MESSAGE);
  } 

  protected final int showQuestionDialog(String error) {
    return this.showQuestionDialog(error, "Question : select an option");
  } 

  protected final int showQuestionDialog(String error, String title) {
    return JOptionPane.showConfirmDialog(this, error, title, JOptionPane.YES_NO_OPTION);
  } 

  protected final int showQuestionDialogWithCancel(String error) {
    return this.showQuestionDialog(error, "Question : select an option");
  } 

  protected final int showQuestionDialogWithCancel(String error, String title) {
    return JOptionPane.showConfirmDialog(this, error, title, JOptionPane.YES_NO_CANCEL_OPTION);
  } 

  protected final void showWarningDialog(String error) {
    this.showWarningDialog(error, "Warning");
  } 

  protected final void showWarningDialog(String error, String title) {
    JOptionPane.showMessageDialog(this, error, title, JOptionPane.WARNING_MESSAGE);
  } 

}

