/*
  Copyright (C) 2002-2005 JAsCo Development Team
  http://ssel.vub.ac.be/jasco

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
 */
package jasco.util;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JTextField;

public class BrowseField extends JComponent {
	
	private JTextField text = new JTextField();
	private MyFileChooser fileChooser;
	private JButton browse;
	private Vector listeners = new Vector();
	
	public BrowseField(final MyFileChooser fileChooser, final boolean directoriesOnly) {
		super();
		this.fileChooser = fileChooser;
		setLayout(new BorderLayout());
		add(text, "Center");
		browse= new JButton(SwingFactory.getIcon("browse.gif"));
		browse.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				fileChooser.selectFile(new File(text.getText()));
				if (directoriesOnly) fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				else fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
				fileChooser.showOpenDialog();
                if (!fileChooser.isCanceled())	text.setText(fileChooser.getSelectedFile().getPath());
				
				fireActionEvent(new ActionEvent(this,0,""));
			}
		});
		add(browse, "East");
		setPreferredSize(new Dimension(300,30));
	}
	
	protected void fireActionEvent(ActionEvent ev)  {
	
		Iterator i = listeners.iterator();
		while (i.hasNext())  {
			ActionListener l = (ActionListener) i.next();
			l.actionPerformed(ev);
		}
	}
	
	public void addActionListener(ActionListener l)  {
	
		listeners.add(l);
	}
	
	public void removeActionListener(ActionListener l)  {
	
		listeners.remove(l);
	}
	
	public String getSelectedFile() {
		return text.getText();
	}
	
	public void selectFile(String s) {
		text.setText(s);
	}
}
