/*
  Copyright (C) 2002-2005 JAsCo Development Team
  http://ssel.vub.ac.be/jasco

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
 */
package jasco.util;

import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JFrame;


/** klasse die een standaard dialog box representeert om file(s) te selecteren*/
public class MyFileChooser extends JFileChooser {
	
	private boolean isCanceled = true;
	private JFrame parent;
	
	public MyFileChooser(JFrame parent) {
		
		super();
		
		this.setMultiSelectionEnabled(false); //NOT YET IMPLEMENTED IN JFILECHOOSER BEAN
		this.setFileSelectionMode(JFileChooser.FILES_ONLY); //standaard; kan overschreven w door subklassen
		this.parent = parent;
		
	}
	
	public void selectFile(File file) {
		
		if(file.getParent() != null)
			this.setCurrentDirectory(new File(file.getParent()));
		this.setSelectedFile(file);
	}
	
	public void selectDir(File file) {
		
		this.setCurrentDirectory(file);
	}
		
	
	public void showOpenDialog() {
		
		updateUI();
		computeResult(super.showOpenDialog(parent));
		
	}
	
	public void showSaveDialog() {
		
		updateUI();
		computeResult(super.showSaveDialog(parent));
	}
	
	
	public boolean isCanceled()  {
		
		return isCanceled;
	}
		
	protected void computeResult(int result) {
		
		if (result != JFileChooser.APPROVE_OPTION)
			isCanceled = true;
		else isCanceled = false;
	}
}

/**
 * Klasse die het mogelijk maakt meerdere files te selecteren door een directory te selecteren.
 * Alle java files in deze directory en subdirectory's worden dan opgezocht.
 * Ik heb dit zo gedaan omdat meerdere files selecteren nog niet gemplementeerd is in Swing.
 */
 
/*
class MultipleFileChooser extends MyFileChooser {
	
	MultipleFileChooser(JFrame parent) {
		
		super(parent);
		this.setFileSelectionMode(this.FILES_AND_DIRECTORIES);
		//multi-selection mode enablen if implemented in Swing
		//this.setFileFilter(new ExtensionFileFilter(new String[] {"java", "jav"}, "java source files"));
	}
	
	public File[] getSelectedFiles() {
		
		//niet goed, maar deze functie is nog niet gecodeerd in Swing, dus moeten we zelf doen
		File file = this.getSelectedFile(); 
		if(!file.isDirectory()) //recursief alle java files zoeken
			return new File[] {file};
		
		Vector result = new Vector();
		addFiles(file, result);
		
		int size = result.size();
		File[] files = new File[size];
		
		for(int i=0; i<size; ++i)
			files[i] = (File)result.elementAt(i);
		
		return files;
	}
			
			
	protected void addFiles(File dir, Vector result) {
		
		String[] files = dir.list(new JavaFilter());
		for(int i =0; i<files.length; ++i) {
			File temp = new File(dir, files[i]);
			if(temp.isDirectory()) 
				addFiles(temp, result);
			else result.add(temp);
		}
	}
	
	protected class JavaFilter implements FilenameFilter {
		public boolean accept(File dir, String name) {
				
			if(new File(dir, name).isDirectory())
				return true;
			else if(name.toLowerCase().endsWith(".java") || name.toLowerCase().endsWith(".jav"))
				return true;
			else return false;
		}
	}
				
}
*/