/*
  Copyright (C) 2002-2005 JAsCo Development Team
  http://ssel.vub.ac.be/jasco

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
 */
/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package jasco.util;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.MediaTracker;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.Vector;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.border.EtchedBorder;

public class SwingFactory {
	public static JMenu createJMenu(String name, int mnemonic, String[] commands, KeyStroke[] shortcuts, Icon[] icons, int[] mnemonics, String[] labels, ActionListener listener) {
		return createJMenu(name, mnemonic, commands, shortcuts, icons, mnemonics, labels, listener, false);
	} 

	public static JMenu createJMenu(String name, int mnemonic, String[] commands, KeyStroke[] shortcuts, Icon[] icons, int[] mnemonics, String[] labels, ActionListener listener, boolean tearoff) {
		JMenu result = new JMenu(name, tearoff);    // init JMenu
		if (mnemonic != KeyEvent.VK_UNDEFINED)
			result.setMnemonic(mnemonic);
		for (int i = 0; i < commands.length; ++i) {
			if (commands[i] == null) {
				result.add(new JSeparator());
				continue;
			} 
			JMenuItem menuItem = new JMenuItem(labels[i], icons[i]);
			menuItem.setActionCommand(commands[i]);
			if (shortcuts[i] != null)
				menuItem.setAccelerator(shortcuts[i]);
			if (mnemonics[i] != KeyEvent.VK_UNDEFINED)
				menuItem.setMnemonic(mnemonics[i]);
			menuItem.addActionListener(listener);
			result.add(menuItem);
		} 
		return result;
	} 

	public static JPopupMenu createJPopupMenu(String[] commands, Icon[] icons, int[] mnemonics, String[] labels, ActionListener listener) {
		JPopupMenu result = new JPopupMenu();    // init JMenu
		for (int i = 0; i < commands.length; ++i) {
			if (commands[i] == null) {
				result.add(new JSeparator());
				continue;
			} 
			JMenuItem menuItem = new JMenuItem(labels[i], icons[i]);
			menuItem.setActionCommand(commands[i]);
			if (mnemonics[i] != KeyEvent.VK_UNDEFINED)
				menuItem.setMnemonic(mnemonics[i]);
			menuItem.addActionListener(listener);
			result.add(menuItem);
		} 
		result.setBorderPainted(true);
		return result;
	} 

	public static JToolBar createJToolBar(Vector commands, Vector icons, Vector tooltips, ActionListener listener, boolean showText,JToolBar res) {
		return createJToolBar("", commands, icons, tooltips, listener, showText,res);
	} 

	public static JToolBar createJToolBar(String[] commands, Icon[] icons, String[] tooltips, ActionListener listener, boolean showText,JToolBar res) {
		return createJToolBar("", commands, icons, tooltips, listener, showText,res);
	}
	
	public static JToolBar createJToolBar(Vector commands, Vector icons, Vector tooltips, ActionListener listener, boolean showText) {
		return createJToolBar("", commands, icons, tooltips, listener, showText,null);
	} 

	public static JToolBar createJToolBar(String[] commands, Icon[] icons, String[] tooltips, ActionListener listener, boolean showText) {
		return createJToolBar("", commands, icons, tooltips, listener, showText,null);
	}
	
	public static JToolBar createJToolBar(String title, Vector commands, Vector icons, Vector tooltips, ActionListener listener, boolean showText) { 
		return createJToolBar(title,commands,icons,tooltips,listener,showText,null);
	}
	
	public static JToolBar createJToolBar(String title, Vector commands, Vector icons, Vector tooltips, ActionListener listener, boolean showText, JToolBar res) {
		String[] c = new String[commands.size()];
		commands.toArray(c);
		String[] t = new String[tooltips.size()];
		tooltips.toArray(t);
		Icon[] i = new Icon[icons.size()];
		icons.toArray(i);
		return createJToolBar(title, c, i, t, listener, showText,res);
	} 

	public static JToolBar createJToolBar(String title, String[] commands, Icon[] icons, String[] tooltips, ActionListener listener, boolean showText, JToolBar result) {
		if(result== null)
			result = new JToolBar();
		else result.removeAll();
		if ((title != null) && (!(title.equals("")))) {
			JLabel titleLabel = new JLabel(title);
			titleLabel.setMaximumSize(new Dimension(100, 25));
			titleLabel.setMinimumSize(new Dimension(100, 25));
			titleLabel.setPreferredSize(new Dimension(100, 25));
			titleLabel.setBackground(Color.darkGray);
			titleLabel.setForeground(Color.white);
			titleLabel.setOpaque(true);
			titleLabel.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
			result.add(titleLabel);
			result.addSeparator(new Dimension(10, 19));
			result.setOpaque(true);
			result.setBackground(Color.lightGray);
		} 
		for (int i = 0; i < commands.length; ++i) {
			if (commands[i] == null) {
				result.addSeparator(new Dimension(20, 19));
				continue;
			} 
			JButton button = null;
			tooltips[i] = tooltips[i].replace('_',' ');
			if (showText) {
				String shortName = tooltips[i];
				int j = shortName.lastIndexOf(".");
				if (j>0) shortName = shortName.substring(j+1);
				button = new JButton(shortName, icons[i]);
			}
			else button = new JButton(icons[i]);
			button.setToolTipText(tooltips[i]);
			button.setActionCommand(commands[i]);
			button.addActionListener(listener);
			result.add(button);
		} 
		return result;
	} 

	public static Icon getIcon(String name) {
		ImageIcon icon =  new ImageIcon(jasco.options.Options.getInstallPath()+"/icons"+System.getProperty("file.separator")+name);
		while (icon.getImageLoadStatus() != MediaTracker.COMPLETE) {
			if (icon.getImageLoadStatus() != MediaTracker.ABORTED) {
				System.out.println("ERROR: Icon: " + name + " Load was aborted");
				break;
			}
			if (icon.getImageLoadStatus() != MediaTracker.ERRORED) {
				System.out.println("ERROR: Icon: " + name + " Error during load");
				break;
			}

		}
		return icon;
	} 
	
	public static Icon getImage(String name) {
		ImageIcon icon =  new ImageIcon("images"+System.getProperty("file.separator")+name);
		while (icon.getImageLoadStatus() != MediaTracker.COMPLETE) {
			if (icon.getImageLoadStatus() != MediaTracker.ABORTED) {
				System.out.println("ERROR: Icon: " + name + " Load was aborted");
				break;
			}
			if (icon.getImageLoadStatus() != MediaTracker.ERRORED) {
				System.out.println("ERROR: Icon: " + name + " Error during load");
				break;
			}

		}
		return icon;
	} 

	public static int paintInCenter(String s, int width, int ypos, Graphics g) {
		int swidth = g.getFontMetrics().stringWidth(s);
		g.drawString(s, (width - swidth) / 2, ypos);

		return swidth;
	} 

}


