package jasco.util.distribution;

import jasco.options.Options;
import jasco.runtime.distribution.annotation.Distribution;
import jasco.runtime.distribution.annotation.JAsCoGenerated;
import jasco.util.generators.JavaGenerator;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Date;
/**
 * This class helps to generate automatically connectors
 * customized with specific parameters for distribution. 
 * ldbn 28-9-2006
 * */
public class JascoDistributionConnectorGenerator {

	public JascoDistributionConnectorGenerator(){}
	
	
	/**
	 * This method generates automatically the connector
	 * that will instantiate the corresponding aspect to
	 * instrument an specific instance.ldbn 28-09-2006 
	 * */
	public void generateConnForProxy(Class typeOfProxy, long instanceId){
		// TODO Add some optimization for the parameters of thesame type
		// in one call. current implementation only generates two aspects.
		// It will be better to just add a new instance to an existant
		// aspect. ldbn 28-09-2006
		String connectorname = "_JascoGeneratedConnForProxy" +(new Date()).getTime();
		String auxpkg = "jasco.runtime.distribution.aspects";
		boolean localDeployment = true;
		String impl =  auxpkg + "RemoteReferenceRedirection.instanceInterceptor hook0 = " ;
		impl = impl + " new " + auxpkg 
		  + "RemoteReferenceRedirection.instanceInterceptor("
		  +	"* " + typeOfProxy.getName() + ".*(*));";
		impl = impl + " hook0.setInstance(" + instanceId + ");";
		this.loadConnector(connectorname,impl, localDeployment);
	}

	
	
	/** This method writes a distributed connector to the file system
	 *  loading then the connector. It can be deployed locally or to all hosts
	 *   */
	public void loadConnector(String connectorname, String impl, boolean isLocal) {
		
		if(impl.equals(""))
			return;
		
		String localString = "";
		if(isLocal)
			localString="@"+Distribution.class.getName()+"(value="+Distribution.class.getName()+".Type.LOCAL) ";

		impl=localString+"@"+JAsCoGenerated.class.getName()+" static connector "+connectorname+" {"+JavaGenerator.NEWLINE+impl+"}";
		
		File file = new File(connectorname+".con");
		try {
		
		
		FileWriter writer = new FileWriter(file);
		writer.write(impl);
		writer.close();
		
		} catch (IOException e) {
			jasco.util.logging.Logger.getInstance().showError("Connector failed writing: "+connectorname);
			jasco.util.logging.Logger.getInstance().showError(e);
		}
		
		String dir = Options.getOutputDir();
		try {
			Options.setOutputDir(Options.getTempDir().getAbsolutePath());
			jasco.Jasco.compileAndLoadConnector(file.getAbsolutePath());
		} catch (Throwable e) {
			jasco.util.logging.Logger.getInstance().showError("Connector failed loading: "+connectorname);
			jasco.util.logging.Logger.getInstance().showError(e);
		}
		finally{
			Options.setOutputDir(dir);
		}
		
		file.deleteOnExit();
		File classFileDir = new File(Options.getTempDir(),"Connector");
		File classFile = new File(classFileDir,connectorname+".class");
		if(classFile.exists())
			classFile.deleteOnExit();
		
	}
	
	
}
