/*
  Copyright (C) 2002-2005 JAsCo Development Team
  http://ssel.vub.ac.be/jasco

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
 */
package jasco.util.generators;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.Iterator;
import java.util.Vector;

public class JavaGenerator implements Cloneable{
	
	public static final int NOLINE = -1;
    public final static String NEWLINE = System.getProperty("line.separator");
    public final static String TAB = "\t";
    private String             name;
    private int                modifiers = 0;
    private String             documentation = "";
    private Vector annotations = new Vector();
    private int lineno = NOLINE;
    private String generics = "";

    public JavaGenerator(String name) {
        this.name = name;
    }
    
    public void setLineNo(int lineno) {
    	
    	this.lineno=lineno;
    }
    
    public int getLineNo() {
    	
    	return lineno;
    }
    
    public boolean hasLineNo() {
    	return getLineNo()!=NOLINE;
    }
	
    
    public Iterator getAnnotations() {
    	return annotations.iterator();
    }
    
    public void addAnnotation(String a) {
    	annotations.add(a);
    }
    
    public String getAnnotationsString() {
    	
    	String tmp = "";
    	Iterator i = getAnnotations();
    	while(i.hasNext()) {
    		String next = (String) i.next();
    		tmp+=next+" ";
    	}
    	return tmp;
    }
	
	 public boolean definesAnnotation(String type) {
	    	
		 	if(!type.startsWith("@"))
				type+="@";
	    	Iterator i = getAnnotations();
	    	while(i.hasNext()) {
	    		String next = (String) i.next();
	    		if(next.startsWith(type))
					return true;
	    	}
	    	return false;
	    }

    public void addModifier(int mod) {
        modifiers |= mod;
    } 

    public int getModifiers() {
        return modifiers;
    } 
	
	public void setModifiers(int i) {
         modifiers=i;
    } 
	
    public String getName() {
        return name;
    } 

    public void setName(String name) {
        this.name = name;
    } 

    public boolean hasDocumentation() {
        return !documentation.equals("");
    } 

    public void setDocumentation(String doc) {
        documentation = doc;
    } 

    public String getDocumentation() {
        return documentation;
    } 

    protected static String addStringBeforeLines(String s, String b) {
        try {
            StringBuffer   buffer = new StringBuffer("");
            BufferedReader reader = new BufferedReader(new StringReader(s));
            String         line;
            while ((line = reader.readLine()) != null) {
                buffer.append(b);
                buffer.append(line);
                buffer.append(NEWLINE);
            } 
            reader.close();
            return buffer.toString();
        } catch (IOException e) {
            System.out.println(e);
        } 
        return s;
    } 

    public static String lowerCaseFirst(String name) {
        char[] temp = name.toCharArray();
        temp[0] = Character.toLowerCase(temp[0]);
        return new String(temp);
    } 

    public static String upperCaseFirst(String string) {
        char[] temp = string.toCharArray();
        temp[0] = Character.toUpperCase(temp[0]);
        return new String(temp);
    } 

    public static String addTabsBeforeLines(String s) {
        return addStringBeforeLines(s, TAB);
    } 

    protected String createJavadocDocumentation() {
        if (!hasDocumentation())
            return "";

        String doc = "";
        doc += "/**" + NEWLINE;
        doc += addStringBeforeLines(getDocumentation(), " * ");
        doc += " */" + NEWLINE;
        return doc;
    } 

    public String toString() {
        return createJavadocDocumentation();
    }

	public String getGenerics() {
		return generics;
	}

	public void setGenerics(String generics) {
		this.generics = generics;
	} 

}


