package org.cachebench.cachewrappers;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.cachebench.CacheWrapper;
import org.jboss.cache.Cache;
import org.jboss.cache.CacheSPI;
import org.jboss.cache.DefaultCacheFactory;
import org.jboss.cache.Fqn;
import org.jboss.cache.buddyreplication.GravitateResult;
import org.jboss.cache.marshall.NodeData;

import java.net.URL;
import java.util.Map;

public class JBossCache200Wrapper implements CacheWrapper
{
   private Cache cache;
   private Log log = LogFactory.getLog(JBossCache200Wrapper.class);

   public void init(Map parameters) throws Exception
   {
      log.info("Creating cache with the following configuration: " + parameters);
      String configFile = (String) parameters.get("config");
      URL url = Thread.currentThread().getContextClassLoader().getResource(configFile);
      log.info("Loading cache with config file: " + url);
      log.info("Running follwing JBossCacheVersion: " + org.jboss.cache.Version.version);
      log.info("Running follwing JBossCacheCodeName: " + org.jboss.cache.Version.codename);
      cache = DefaultCacheFactory.getInstance().createCache(configFile);
   }

   public void setUp() throws Exception
   {
   }

   public void tearDown() throws Exception
   {
      cache.stop();
   }

   public void put(Object key, Object value) throws Exception
   {
      // make sure the threads don't conflict!
      Fqn f = new Fqn("test", key);
      cache.put(f, key, value);
   }

   public Object get(Object key) throws Exception
   {
      Fqn f = new Fqn("test", key);
      return cache.get(f, key);
   }

   public void empty() throws Exception
   {
      //not removing root because there it fails with buddy replication: http://jira.jboss.com/jira/browse/JBCACHE-1241
      cache.removeNode(new Fqn("test"));
   }

   public int getNumMembers()
   {
      return cache.getMembers().size();
   }

   public String getInfo()
   {
      return "Num direct children: " + cache.getRoot().getChildren().size();
   }

   public Object getReplicatedData(String key) throws Exception
   {
      CacheSPI cacheSpi = (CacheSPI) cache;
      GravitateResult result = cacheSpi.gravitateData(new Fqn("test", key), true);
      if (!result.isDataFound())
      {
         //totall replication?
         return get(key);
      }
      NodeData nodeData = result.getNodeData().get(0);
      return nodeData.getAttributes().get(key);
   }
}
