package org.jboss.cache.util;
/**
 * Copyright (c) 2005 Brian Goetz and Tim Peierls
 * Released under the Creative Commons Attribution License
 * (http://creativecommons.org/licenses/by/2.5)
 * Official home: http://www.jcip.net
 * 
 * ThreadGate <p/> Recloseable gate using wait and notifyAll
 * 
 * @author Brian Goetz and Tim Peierls
 */

public class ThreadGate {
    // CONDITION-PREDICATE: opened-since(n) (isOpen || generation>n)
    private boolean isOpen;

    private int generation;

    public synchronized void close() 
    {
        isOpen = false;
    }

    public synchronized void open() 
    {
        ++generation;
        isOpen = true;
        notifyAll();
    }

    // BLOCKS-UNTIL: opened-since(generation on entry)
    public synchronized void await() throws InterruptedException 
    {
        int arrivalGeneration = generation;
        while(!isOpen && arrivalGeneration == generation)
            wait();
    }
    
    // BLOCKS-UNTIL: opened-since(generation on entry)
    public synchronized void await(long timeout) throws InterruptedException 
    {
        int arrivalGeneration = generation;
        while(!isOpen && arrivalGeneration == generation)
            wait(timeout);
    }
}