/*
  Copyright (C) 2002-2005 JAsCo Development Team
  http://ssel.vub.ac.be/jasco

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
 */
package jasco.runtime.hotswap1;

import gnu.regexp.RE;
import jasco.options.Options;
import jasco.util.generators.ClassGenerator;
import jasco.util.generators.JavaGenerator;
import jasco.util.generators.MethodGenerator;
import jasco.util.generators.VariableGenerator;
import jasco.util.javacompiler.JavaCompiler;
import jasco.util.logging.Logger;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtConstructor;
import javassist.CtField;
import javassist.CtMethod;
import javassist.CtNewConstructor;
import javassist.CtNewMethod;
import javassist.Modifier;

public class RuntimeBeanTransformerJavassist 
{
	private Vector excludedmethods = null;
	private Vector adaptedmethods = new Vector();
	private Vector innerClasses = new Vector();
	private String packageName = "";
	
	private Hashtable methodTodoVector = new Hashtable();
	private static ClassPool classPool = ClassPool.getDefault();
	private CtClass theMakedClass = null;
	private CtClass fieldTheMakedClass = null;
	private String selectedMethod = null;
	
	private FileWriter debugWriter = null;
	private long theID = 0;
	private Vector methodsToTransform = new Vector();
	
	private StringBuffer initCode = new StringBuffer();
	
	public void methodsToTransform(Vector v)  {
	
		methodsToTransform =v;
	}
	
	public void setClassPool(ClassPool p)  {
	
		classPool=p;
	}
	
	

	public CtClass getDecompiledClass()  {
	
		return theMakedClass;
	}
	
	public String getPackageName()  {
	
		return packageName;
	}
	
	public CtClass getFieldClass()  {
		return fieldTheMakedClass;
	}
	
	protected String getFieldClassName(CtClass main)  {
		
		return main.getName()+"$_JAsCo_FIELDS"+theID;
	}
	
	public byte[] getDecompiledClassBytecode() {
		
		try {
		if(getDecompiledClass()==null) return new byte[0];
		else return getDecompiledClass().toBytecode();
		}catch (Exception ex){
			Logger.getInstance().showError(ex);}
		
		return new byte[0];
	} 
	
	protected void initFieldClass(CtClass main)  {
		
		fieldTheMakedClass = classPool.makeClass(getFieldClassName(main));	
		int mod = fieldTheMakedClass.getModifiers();
		Modifier.setPublic(mod);
		fieldTheMakedClass.setModifiers(mod);
	}
	
	protected String accessField(String f)  {
	
		return getFieldClass().getName()+"#"+f;
	}
	
	protected void generateFieldClass(File f) throws Exception {
		generateStaticInit();
		
		FileOutputStream stream = new FileOutputStream(f);
		getFieldClass().stopPruning(true);
		stream.write(getFieldClass().toBytecode());
		stream.close();
		getFieldClass().defrost();
	}
	
	private boolean transformBeanImpl(String classname,Vector theexcludedmethods) throws Exception {
		
		theID++;
		Options.setOutputDir(Options.getTempDir().getPath());
		adaptedmethods=new Vector();
		innerClasses=new Vector();
		methodTodoVector=new Hashtable();
		excludedmethods = theexcludedmethods;
		Logger.getInstance().showOutput("Processing: "+classname);
		String classFileName=classname.replace('.','/');
	
		File classFile = new File(Options.getOutputDir()+File.separator + classFileName + ".class");
	
		try  {
		theMakedClass=classPool.get(classname);
		theMakedClass.stopPruning(true);
		}
		catch(Throwable ex)  {
			Logger.getInstance().showError("Failed loading class: "+classname);
			Logger.getInstance().showError(ex.getMessage());
			Logger.getInstance().showError("Current Classpath: "+System.getProperty("java.class.path"));
			//Logger.getInstance().showError(Options.getOutputDir());
			//Logger.getInstance().showFullError(ex);
			return false;
		}
		
		if(theMakedClass.isInterface()) {
			return false;
		}
		new File(classFile.getParent()).mkdirs();
		theMakedClass.stopPruning(true);
		theMakedClass.defrost();
		CtMethod[] methods = getDecompiledClass().getDeclaredMethods();
		packageName=getDecompiledClass().getPackageName();
		
		initFieldClass(getDecompiledClass());
		File fieldClasFile = new File(Options.getTempDir()+File.separator + getFieldClass().getName().replace('.','/') + ".class");
		//File fieldClasFile = new File(".\\_testiep"+File.separator + getFieldClass().getName().replace('.','/') + ".class");
		//fieldClasFile.getParentFile().mkdirs();
		
		if (Options.showDebugOutput())  {
			//System.out.println("BEEEEEEEEEEEEEEEEEEEEEEEEE"+Options.getTempDir()+File.separator + classFileName + ".java.bak");
			File f = new File(Options.getTempDir()+File.separator + classFileName + ".java.bak");
			debugWriter=new FileWriter(f);
		}		
		
		for(int j=0; j<methods.length; ++j)
		{   
			if(methods[j].getName().indexOf('$')==-1) //do not adapt internal methods!
			{ 
				if (isExcluded(methods[j]))  {
					Logger.getInstance().showOutput("Skipping excluded method: "+methods[j].getName());
					continue;
				}
				else if(Modifier.isAbstract(methods[j].getModifiers())) //do not adapt abstract methods
					continue;
				else if(Modifier.isNative(methods[j].getModifiers())) //do not adapt native methods
					continue;
				else if (Modifier.isPublic(methods[j].getModifiers()))
				{
						if(selectedMethod!=null)
							if(!new RE(makeGNUReg(selectedMethod)).isMatch(makeGNUReg(methods[j].getName())))
								continue;
						Logger.getInstance().showOutput("Adapting public method: "+methods[j].getName());
						methodTodoVector.put(methods[j],new Integer(j));
						adaptedmethods.add(methods[j].getName());
				}
				else if(Options.transformAllMethods()) {
						if(selectedMethod!=null)
							if(!new RE(makeGNUReg(selectedMethod)).isMatch(makeGNUReg(methods[j].getName())))
								continue;
					Logger.getInstance().showOutput("Adapting non-public method: "+methods[j].getName());
					methodTodoVector.put(methods[j],new Integer(j));
					adaptedmethods.add(methods[j].getName());
					
				}
				
			}
		}
		
		if(adaptedmethods.size()==0)
			return false;
		
		generateCallbackClasses();
		
		generateInnerClasses();
		
		generateAdaptedFields();
		
		
		
		generateFieldClass(fieldClasFile);
		
		adaptMethods();
		
		if (Options.showDebugOutput())
		  {
		  	debugWriter.flush();
			debugWriter.close();
		  }
		return true;
	}
	
	public Vector getAdaptedMethods()  {
	
		return adaptedmethods;
	}
	
	protected static boolean isRegularExp(String s)  {
	
		return s.indexOf('*') != -1;
	}
	
	public static String makeGNUReg(String s)  {
		
		return jasco.util.RegexpMatcher.makeGNUReg(s);
	}
	
	protected void adaptMethods() throws Exception {
		Iterator i = methodTodoVector.entrySet().iterator();
		while (i.hasNext())  {
			Map.Entry entry = (Map.Entry) i.next();
			CtMethod method = (CtMethod) entry.getKey();
			Integer id = (Integer) entry.getValue();
			adaptMethod(method,id.intValue());
		}
	}
	
	protected void generateCallbackClasses() throws Exception {
		if(Options.optimizeLoadTime())
				generateCallbackClassJavassist(getDecompiledClass());
		else	generateCallbackClass(getDecompiledClass());
			
	}
	
	protected String getCallbackClass()  {
	
		return "CallbackClass"+theID;
	}
	
	protected String getCallbackClassFullname()  {
	
		return getDecompiledClass().getName()+"$"+getCallbackClass();
	}
	
	protected String getClallBackObject(CtMethod method)  {
	
		return getCallbackClassFullname()+".INSTANCE";
	}
	
	protected void generateAdaptedFields() throws Exception {
		
		Iterator i = methodTodoVector.entrySet().iterator();
		while (i.hasNext())  {
			Map.Entry entry = (Map.Entry) i.next();
			CtMethod method = (CtMethod) entry.getKey();
			Integer id = (Integer) entry.getValue();
			
			String jascoTrapField = "public static final java.lang.String _Jasco_JascoMethod"+id.toString()+"ID = \""+getMethodString(method,getDecompiledClass())+"\";";
			
			addField(jascoTrapField,getFieldClass());
			String callBackObject = getClallBackObject(method);
			
			String jascoMethodField = "public static final jasco.runtime.JascoMethod _Jasco_JascoMethod"+id.toString()+" ";
			jascoMethodField+=" = new jasco.runtime.JascoMethod(\""+getMethodString(method,getDecompiledClass())+"\", "+"\"\""+", new Object[0], \""+method.getName()+"\" , \""+getDecompiledClass().getName()+"\", "+id.toString()+", "+callBackObject+", "+method.getModifiers()+");"+JavaGenerator.NEWLINE;
			addField(jascoMethodField,getFieldClass());
			
			this.generateInitCode(method,"_Jasco_JascoMethod"+id.toString());
		}
	}
	
	public boolean transformBean(String classname,Vector theexcludedmethods)  {
		return transformBean(classname,theexcludedmethods,true,null);
	}
	
	public boolean transformBean(String classname,Vector theexcludedmethods, boolean finish,String selectedMethod)  {
		try {
			this.selectedMethod=selectedMethod;
			boolean res = transformBeanImpl(classname,theexcludedmethods);
			if(!res)
				return res;
			String clname=classname.replace('.','/');
			
			File classFile = new File(Options.getTempDir()+File.separator+clname+".class");
			if(finish)
				finish(classFile);
			
			return res;	
		}
		catch(Throwable ex)  {
			Logger.getInstance().showError("Failed transformging class: "+classname);
			Logger.getInstance().showError(ex.getMessage());
			ex.printStackTrace();
			try {
				if (Options.showDebugOutput()) 
			 		{debugWriter.flush();
					debugWriter.close();
			 	}
			}catch(Exception ex2) {
			}
			return false;
		}
	}
	
	protected void finish(File f) throws Exception {
	
		FileOutputStream stream = new FileOutputStream(f);
		getDecompiledClass().stopPruning(true);
		stream.write(getDecompiledClass().toBytecode());
		stream.close();
	}
	
	protected boolean isExcluded(CtMethod aMethod)  {
		
		return excludedmethods.contains(aMethod.getName())||aMethod.getName().startsWith("_Jasco_");
	}
	
	protected void doDebug(String code) throws Exception {
	
		if(Options.showDebugOutput())
			debugWriter.write(code);
	}
	
		//only adds methods at the moment
	protected CtClass toJavaAssistInner(ClassGenerator classGen) throws Exception {
		
		String fullClassName = classGen.getPackageName()+"."+classGen.getName();
		if(classGen.getPackageName().equals(""))
			fullClassName=classGen.getName();
		CtClass cachedAspect = classPool.makeClass(fullClassName);
		CtConstructor c = CtNewConstructor.defaultConstructor(cachedAspect);
		cachedAspect.addConstructor(c);
		Iterator i = classGen.getMethods();
		while (i.hasNext())  {
			MethodGenerator g = (MethodGenerator) i.next();
			if (g.getReturnType().equals(""))  { //constructor
				System.out.println(g.toString());
				CtConstructor method = CtNewConstructor.make(g.toString(),cachedAspect);
				cachedAspect.addConstructor(method);
			}
			else  {
			//	System.out.println("NOCON"+g.toString());
			CtMethod method = CtNewMethod.make(g.toString(),cachedAspect);
			cachedAspect.addMethod(method);
			}
		}
		
		
		i = classGen.getVariables();
		while (i.hasNext())  {

			VariableGenerator g = (VariableGenerator) i.next();
			if (g.getName().equals("INSTANCE"))  {
				//System.out.println(g.toString());
				//addField(g.toTestString(),cachedAspect);
				CtField field = new CtField(cachedAspect,"INSTANCE",cachedAspect);
				int m = field.getModifiers();
				m=Modifier.setPublic(m);
				m|=Modifier.STATIC;				
				m|=Modifier.FINAL;
				field.setModifiers(m);
				cachedAspect.addField(field,CtField.Initializer.byNew(cachedAspect));
			}
		}
		//System.out.println("done creating fields");
		
		i = classGen.getInterfaces();
		while (i.hasNext())  {
			String g = (String) i.next();
			CtClass cl = classPool.get(g);
			cachedAspect.addInterface(cl);
		}
		return cachedAspect;
	}
	
	protected void addInnerClass(ClassGenerator classGen) throws Exception {
		
		innerClasses.add(classGen);
		
	}
	
	protected void generateInnerClasses()  throws Exception {
	
		Iterator i = innerClasses.iterator();
		while (i.hasNext())  {
			ClassGenerator cg = (ClassGenerator) i.next();
			generateInnerClass(cg);
		}
	}
	
	protected void generateInnerClass(ClassGenerator classGen) throws Exception {
	
		/*
		String classFileName = Options.getOutputDir()+File.separator+getDecompiledClass().getName().replace('.','/')+"\\$"+classGen.getName()+".java";
		String classFileNameWithoutEscape = Options.getOutputDir()+File.separator+getDecompiledClass().getName().replace('.','/')+"$"+classGen.getName()+".java";
		
		if(Options.isWindows())
			classFileName = classFileNameWithoutEscape;
		String className = getDecompiledClass().getSimpleName()+"$"+classGen.getName();
		classGen.setName(className);
		classGen.setPackage(getDecompiledClass().getPackageName());
		classGen.setModifiers(0);
		
		Iterator j = classGen.getConstructors();
		while (j.hasNext())  {
			MethodGenerator gen = (MethodGenerator) j.next();
			gen.setName(className);
		}
		
		FileWriter writer = new FileWriter(classFileNameWithoutEscape);
		writer.write(classGen.toString());
		writer.close();
		
		new JavaCompiler().compile(classFileName,Options.getOutputDir()+Options.PATH_SEPARATOR+Options.getClassPath());
		
		if(Options.deleteTempFiles())
			new File(classFileName).delete();
		
		
		CtClass inner = toJavaAssistInner(classGen);
		File classFile = new File(Options.getOutputDir()+File.separator + classFileName);
		FileOutputStream writer = new FileOutputStream(classFile);
		writer.write(inner.toBytecode());
		writer.close();*/
		
		
		String classFileName = Options.getOutputDir()+File.separator+getDecompiledClass().getName().replace('.','/')+"\\$"+classGen.getName();
		String classFileNameWithoutEscape = Options.getOutputDir()+File.separator+getDecompiledClass().getName().replace('.','/')+"$"+classGen.getName();
		
		if(Options.isWindows())
			classFileName = classFileNameWithoutEscape;
		String className = getDecompiledClass().getSimpleName()+"$"+classGen.getName();
		classGen.setName(className);
		classGen.setPackage(getDecompiledClass().getPackageName());
		classGen.setModifiers(0);
		
		Iterator j = classGen.getConstructors();
		while (j.hasNext())  {
			MethodGenerator gen = (MethodGenerator) j.next();
			gen.setName(className);
		}
		
		if (!Options.optimizeLoadTime())  {
		
			FileWriter writer = new FileWriter(classFileNameWithoutEscape+".java");
			writer.write(classGen.toString());
			writer.close();
		
			new JavaCompiler().compile(classFileName+".java",Options.getOutputDir()+Options.PATH_SEPARATOR+Options.getClassPath());
		
			if(Options.deleteTempFiles())
				new File(classFileName).delete();
		}
		else  {
			CtClass inner = toJavaAssistInner(classGen);
			File classFile = new File(classFileNameWithoutEscape+".class");
			FileOutputStream writer = new FileOutputStream(classFile);
			inner.stopPruning(true);
			writer.write(inner.toBytecode());
			writer.close();
			inner.defrost();
		}
	}
	
	protected void addField(String fields) throws Exception {
		
		addField(fields,getDecompiledClass());
	}
	
	protected void addField(String fields, CtClass cl) throws Exception {
	
		doDebug(fields);
		CtField field = CtField.make(fields,cl);
		cl.addField(field);	
	}
	
	
	
	protected String getClassName(String name)  {
		
		String res = name;
		int i = name.lastIndexOf(".");
		if(i!=-1)
			res=name.substring(i+1);
		return res;
	}	
		

	protected void generateCallbackClass(CtClass javaClass) throws Exception {
		
		String name = getCallbackClass();
		ClassGenerator callbackClass = new ClassGenerator(name,"");
		callbackClass.addModifier(Modifier.PRIVATE);
		callbackClass.addModifier(Modifier.STATIC);
		callbackClass.addInterface("jasco.runtime.JAsCoBeanCallback");
		
		MethodGenerator method = new MethodGenerator("dispatch","Object");
		method.addModifier(Modifier.PUBLIC);	
		method.addException("java.lang.Exception");
		
		method.addVariable("int","i");
		method.addVariable("java.lang.Object","objectA");
		method.addVariable("java.lang.Object[]","v");
		
		
		VariableGenerator instance  = new VariableGenerator(getCallbackClassFullname(),"INSTANCE");
		instance.addModifier(Modifier.PUBLIC);
		instance.addModifier(Modifier.STATIC);
		instance.addModifier(Modifier.FINAL);
		instance.setInitialiser("new "+getCallbackClassFullname()+"();");
		callbackClass.addVariable(instance);
		
		
		String code	="";
		code+=getSourceName(javaClass)+" object = ("+getSourceName(javaClass)+") objectA;"+JavaGenerator.NEWLINE;
		code+="jasco.runtime.ConnectorRegistry.setThreadFlag();"+JavaGenerator.NEWLINE;
		code+="switch(i) {"+JavaGenerator.NEWLINE;
		code+=JavaGenerator.NEWLINE;
		
		String caseCode= "";
		Iterator it = methodTodoVector.entrySet().iterator();
		while (it.hasNext())  {
			Map.Entry entry = (Map.Entry) it.next();
			CtMethod oldMethod = (CtMethod) entry.getKey();
			Integer id = (Integer) entry.getValue();
			
			caseCode+="case "+id.toString()+": "+"{"+JavaGenerator.NEWLINE;
			String runCode = "";
			String endCode="";
			if(oldMethod.getReturnType()!=CtClass.voidType) {
				runCode+="return ";
				if(oldMethod.getReturnType().isPrimitive())	{
					runCode+="new "+makePrimitiveObject(oldMethod.getReturnType())+"(";
					endCode=")";
				}
			}
			boolean isStatic = Modifier.isStatic(oldMethod.getModifiers());
			if(isStatic)
				runCode+=getDecompiledClass().getName()+"."+oldMethod.getName()+"(";
			else runCode+="object."+oldMethod.getName()+"(";	
			CtClass[] params = oldMethod.getParameterTypes();
			for(int i=0;i<params.length;++i) {
				String src = getSourceName(params[i]);
				if(params[i].isPrimitive()) 
					caseCode+=src+" var"+i+"= (("+makePrimitiveObject(params[i])+") v["+i+"])."+params[i].getName()+"Value();"+JavaGenerator.NEWLINE;
				else caseCode+=src+" var"+i+"= ("+src+") v["+i+"];"+JavaGenerator.NEWLINE;
				runCode+="var"+i;
				if(i<params.length-1)
					runCode+=", ";
				
			
			}
			
			runCode+=")"+endCode+";"+JavaGenerator.NEWLINE;
			caseCode+=runCode;
			if(oldMethod.getReturnType()==CtClass.voidType)
				caseCode+="break;"+JavaGenerator.NEWLINE;
			caseCode+="}"+JavaGenerator.NEWLINE+JavaGenerator.NEWLINE;
		}
		
		code+=JavaGenerator.addTabsBeforeLines(caseCode);
		code+="}"+JavaGenerator.NEWLINE;
		code+="return null;"+JavaGenerator.NEWLINE;
		
		method.setImplementation(code);
		callbackClass.addMethod(method);
		
		String thefield = "private static final jasco.runtime.JAsCoBeanCallback INSTANCE = new "+getCallbackClassFullname()+"();"+JavaGenerator.NEWLINE;
		addField(thefield,getFieldClass());
		
		addInnerClass(callbackClass);
	}
	
	protected void generateCallbackClassJavassist(CtClass javaClass) throws Exception {
		
		String name = getCallbackClass();
		ClassGenerator callbackClass = new ClassGenerator(name,"");
		callbackClass.addModifier(Modifier.PRIVATE);
		callbackClass.addModifier(Modifier.STATIC);
		callbackClass.addInterface("jasco.runtime.JAsCoBeanCallback");
		
		MethodGenerator method = new MethodGenerator("dispatch","Object");
		method.addModifier(Modifier.PUBLIC);	
		method.addException("java.lang.Exception");
		
		method.addVariable("int","i");
		method.addVariable("java.lang.Object","objectA");
		method.addVariable("java.lang.Object[]","v");
		
		
		VariableGenerator instance  = new VariableGenerator(getCallbackClassFullname(),"INSTANCE");
		instance.addModifier(Modifier.PUBLIC);
		instance.addModifier(Modifier.STATIC);
		instance.addModifier(Modifier.FINAL);
		instance.setInitialiser("new "+getCallbackClassFullname()+"();");
		callbackClass.addVariable(instance);
		
		
		String code	="";
		code+=getSourceName(javaClass)+" object = ("+getSourceName(javaClass)+") objectA;"+JavaGenerator.NEWLINE;
		code+="jasco.runtime.ConnectorRegistry.setThreadFlag();"+JavaGenerator.NEWLINE;
		
		//String caseCode= "";
		boolean noElse = true;
		Iterator it = methodTodoVector.entrySet().iterator();
		while (it.hasNext())  {
			Map.Entry entry = (Map.Entry) it.next();
			CtMethod oldMethod = (CtMethod) entry.getKey();
			Integer id = (Integer) entry.getValue();
			
			
			if (noElse)  {
				code+="if(i=="+id.toString()+")"+"{"+JavaGenerator.NEWLINE;
				noElse=false;
			}
			else code+="else if(i=="+id.toString()+")"+"{"+JavaGenerator.NEWLINE;
			String runCode = "";
			String endCode="";
			if(oldMethod.getReturnType()!=CtClass.voidType) {
				runCode+="return ";
				if(oldMethod.getReturnType().isPrimitive())	{
					runCode+="new "+makePrimitiveObject(oldMethod.getReturnType())+"(";
					endCode=")";
				}
			}
			boolean isStatic = Modifier.isStatic(oldMethod.getModifiers());
			if(isStatic)
				runCode+=getDecompiledClass().getName()+"."+oldMethod.getName()+"(";
			else runCode+="object."+oldMethod.getName()+"(";	
			CtClass[] params = oldMethod.getParameterTypes();
			for(int i=0;i<params.length;++i) {
				String src = getSourceName(params[i]);
				if(params[i].isPrimitive()) 
					code+=src+" var"+i+"= (("+makePrimitiveObject(params[i])+") v["+i+"])."+params[i].getName()+"Value();"+JavaGenerator.NEWLINE;
				else code+=src+" var"+i+"= ("+src+") v["+i+"];"+JavaGenerator.NEWLINE;
				runCode+="var"+i;
				if(i<params.length-1)
					runCode+=", ";
				
			
			}
			
			runCode+=")"+endCode+";"+JavaGenerator.NEWLINE;
			code+=runCode;
			if(oldMethod.getReturnType()==CtClass.voidType)
				code+="return null;"+JavaGenerator.NEWLINE;
			code+="}"+JavaGenerator.NEWLINE+JavaGenerator.NEWLINE;
		}
		
		code+="return null;"+JavaGenerator.NEWLINE;
		
		method.setImplementation(code);
		callbackClass.addMethod(method);
		
		String thefield = "private static final jasco.runtime.JAsCoBeanCallback INSTANCE = new "+getCallbackClassFullname()+"();"+JavaGenerator.NEWLINE;
		addField(thefield,getFieldClass());
		
		addInnerClass(callbackClass);
	}
	
	public void generateInitCode(CtMethod method, String methodvar) throws Exception {
		
		String res = "";
		methodvar=getFieldClass().getName()+"#"+methodvar;
		CtClass[] fargs = method.getParameterTypes();
		CtClass[] excs = method.getExceptionTypes();
		res+="{String[] fargs = new String["+fargs.length+"];"+JavaGenerator.NEWLINE;
		for(int i=0;i<fargs.length;++i)
			res+="fargs["+i+"] = \""+fargs[i].getName()+"\";"+JavaGenerator.NEWLINE;
		res+=methodvar+".setFormalArgumentTypes(fargs);"+JavaGenerator.NEWLINE;
		res+="String[] excs = new String["+excs.length+"];"+JavaGenerator.NEWLINE;
		for(int i=0;i<excs.length;++i)
			res+="excs["+i+"] = \""+excs[i].getName()+"\";"+JavaGenerator.NEWLINE;
		res+=methodvar+".setExceptionTypes(excs);}"+JavaGenerator.NEWLINE;
		res+=methodvar+".initClassLoader(loader);"+JavaGenerator.NEWLINE;
		
		initCode.append(res);
	}
	
	protected void generateStaticInit() throws Exception{
		
		String code = "public static void init(ClassLoader loader) {"+initCode.toString()+"}";
		
		CtMethod method= CtNewMethod.make(code,getFieldClass());
		getFieldClass().addMethod(method);
		
		CtConstructor c = getFieldClass().makeClassInitializer();
		c.setBody("init(ClassLoader.getSystemClassLoader());");
		
		
	}

	public void adaptMethod(CtMethod method, int pos) throws Exception
	{	
		long tmp = System.currentTimeMillis();
		CtMethod oldMethod = method;
		
		String jascoMethodField = "_Jasco_JascoMethod"+pos;
		String jascoTrapIdField = "_Jasco_JascoMethod"+pos+"ID";
		
		//String callMethodStringImpl = "(";
		CtClass[] params = oldMethod.getParameterTypes();
		StringBuffer argsArrayCreateCode= new StringBuffer("Object[] argsArray = new Object[");
		argsArrayCreateCode.append(params.length);
		argsArrayCreateCode.append("];");
		
		for(int j=0; j<params.length; ++j)  {
			String classRp = getSourceName(params[j]);
			//callMethodStringImpl+="var"+j;
			if (params[j].isPrimitive())   {
				argsArrayCreateCode.append("argsArray[");
				argsArrayCreateCode.append(j);
				argsArrayCreateCode.append("]=new ");
				argsArrayCreateCode.append(makePrimitiveObject(params[j]));
				argsArrayCreateCode.append("($");
				argsArrayCreateCode.append((j+1));
				argsArrayCreateCode.append(");");
			}
			else   {
				argsArrayCreateCode.append("argsArray[");
				argsArrayCreateCode.append((j));
				argsArrayCreateCode.append("]=$");
				argsArrayCreateCode.append((j+1));
				argsArrayCreateCode.append(";");
			}
		}
		argsArrayCreateCode.append(JavaGenerator.NEWLINE);
		StringBuffer codeBuffer=new StringBuffer();
		
		
		codeBuffer.append("if(jasco.runtime.ConnectorRegistry#isAdaptedAndThreadFlag(");
		codeBuffer.append(accessField(jascoMethodField));
		codeBuffer.append(")) {");
		
		codeBuffer.append("try {");
		codeBuffer.append(JavaGenerator.NEWLINE);
		codeBuffer.append(argsArrayCreateCode.toString());
		codeBuffer.append(accessField(jascoMethodField));
		codeBuffer.append(".setArgumentsArray(argsArray);");
		codeBuffer.append(JavaGenerator.NEWLINE);
		String code = codeBuffer.toString();
			
		String thisInstance="$0";
		if(Modifier.isStatic(oldMethod.getModifiers()))
			thisInstance="jasco.runtime.NULL#instance";
		else 	code+=accessField(jascoMethodField)+".setCalledObject($0);"+JavaGenerator.NEWLINE;
		
		String startCode="";
		String returnCode="";
		if (oldMethod.getReturnType()!=CtClass.voidType)    {
			returnCode+="return ";
			
			if (oldMethod.getReturnType().isPrimitive())  {
				String type = makePrimitiveObject(oldMethod.getReturnType());
				startCode+=type+" res = "+"("+type+")";
				returnCode+="res."+oldMethod.getReturnType().getName()+"Value();";
			}
			else  {
				String type = oldMethod.getReturnType().getName();
				startCode+=type+" res = "+" ("+type+") ";
				returnCode+=" res;";
			}
		}
		else returnCode="return;"+JavaGenerator.NEWLINE;
		
		code+=startCode;
		code+="jasco.runtime.ConnectorRegistry#executeTrap("+accessField(jascoTrapIdField)+", "+thisInstance+","+accessField(jascoMethodField)+")"+";"+JavaGenerator.NEWLINE;	
		code+=returnCode;
		
		
		CtClass[] exceptions = method.getExceptionTypes();
		code += "} catch(java.lang.Exception b) {"+ JavaGenerator.NEWLINE;
		for(int i=0; i<exceptions.length; ++i)  {
			code+=JavaGenerator.TAB+"if(b instanceof "+exceptions[i].getName()+")";
			code+=" throw ("+exceptions[i].getName()+") b;"+JavaGenerator.NEWLINE;
		}
		String subCode2 ="";
		subCode2+= JavaGenerator.TAB+"jasco.options.Options#doDebugException(b);"+JavaGenerator.NEWLINE;
		code+=subCode2;
		
		code+="}}"+JavaGenerator.NEWLINE;
		code+="else jasco.runtime.ConnectorRegistry#releaseThreadFlag();"+JavaGenerator.NEWLINE;
		
		doDebug(code);
		
		tmp=System.currentTimeMillis();
		method.insertBefore(code);
		
		//insertBefore(method,code);
		
		//CodeAttribute codes = method.getMethodInfo().getCodeAttribute();
		//System.out.println(codes);
		
	}
	/*
	protected void insertBefore(CtMethod method, String code) throws Exception  {
	
		CodeAttribute ca = methodInfo.getCodeAttribute();
        if (ca == null)
            throw new CannotCompileException("no method body");

        CodeIterator iterator = ca.iterator();
        Javac jv = new Javac(declaringClass);
        
		v.recordParams(getParameterTypes(),Modifier.isStatic(getModifiers()));
        jv.compileStmnt(src);
        Bytecode b = jv.getBytecode();
        int stack = b.getMaxStack();
        int locals = b.getMaxLocals();

        if (stack > ca.getMaxStack())
            ca.setMaxStack(stack);

        if (locals > ca.getMaxLocals())
            ca.setMaxLocals(locals);

        int pos = iterator.insertEx(b.get());
        iterator.insert(b.getExceptionTable(), pos);
	}
	*/
	
	public String getSignatureSourceName(CtClass aClass)  {
		
		String name = aClass.getName();
		//name=name.replace('$','.');
		return name;
		
		
		
		
	}
	
	protected String getSourceName(CtClass aClass) throws Exception {
		
		String name = getSignatureSourceName(aClass);
		return name;
	}	

	protected String getMethodString(CtMethod theMethod, CtClass javaClass, String newName) throws Exception {	
		
		boolean fireMethod=false;
		if (newName.startsWith("_Jasco_fire"))  {
			fireMethod=true;
			newName=newName.substring(11);
		}
		String res = "";
		if(!fireMethod)
			res+=getSignatureSourceName(theMethod.getReturnType());
		else res+="onevent";	

		res+=" "+javaClass.getName()+"."+newName+"(";
		CtClass[] params = theMethod.getParameterTypes();
		for(int j=0; j<params.length; ++j)   {
			res+=getSignatureSourceName(params[j]);
			if(j<params.length-1)
				res+=", ";
		}
		res+=")";
		
			
		return res;	
	}
	
	protected String getMethodString(CtMethod theMethod, CtClass javaClass) throws Exception {	
		
		return getMethodString(theMethod,javaClass,theMethod.getName());
	}
	
	

	
	
	
	public String makePrimitiveObject(CtClass aClass) 
	{
		if(aClass==CtClass.booleanType)
			return "Boolean";
		else if(aClass==CtClass.intType)
			return "Integer";
		else if(aClass==CtClass.floatType)
			return "Float";
		else if(aClass==CtClass.doubleType)
			return "Double";
		else if(aClass==CtClass.charType)
			return "Character";
		else if(aClass==CtClass.byteType)
			return "Byte";
		else if(aClass==CtClass.shortType)
			return "Short";
		else if(aClass==CtClass.longType)
			return "Long";
		else return"Not recognized Primitive!!";
		
	}
}
